/****************************************************
   TORNADO-PX64xxQ on-board DSP peripherals test.
   DSP IOX Control Registers test and MX-SIOX RESET and TM/XIO-0/1 Test. rev.1A
   (C) MicroLAB Systems, 2003


   Description:
   ------------
	This demo continuously scan DSP external IOX control registers
	within the DSP environment and MX-SIOX RESET and TM/XIO-0/1 control pins.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	- DSP bootmode: NOBOOT
*****************************************************/
// include headers
#include "stddef.h"
#include "px64xxq_dsp.h"



main()
{
	int i;

	// initialize DSP environment
   	TPX64XXQ_init_dsp();

	// configure MX-SIOX TM/XIO-0/1 pins as genral purpose OUTPUTS
	TPX64XXQ_SET_SX_XIO0_CNF_OUTPUT;
	TPX64XXQ_SET_SX_XIO1_CNF_OUTPUT;
	if(		!TPX64XXQ_SX_XIO0_CNF_IS_OUTPUT
		||	!TPX64XXQ_SX_XIO1_CNF_IS_OUTPUT
	  )
	  	goto error;


	// enter infinite test loop
	while(1)
    {
		// test MX-SIOX RESET output pin
		// Note, that RESET ON state is default setting after DSP initialization.
		TPX64XXQ_SET_SX_RESET_OFF;
		if(!TPX64XXQ_SX_RESET_IS_OFF) goto error;

		TPX64XXQ_SET_SX_RESET_ON;
		if(!TPX64XXQ_SX_RESET_IS_ON) goto error;


		// test MX-SIOX TM/XIO-0 output pin
		TPX64XXQ_SET_SX_XIO0_DATA_ON;
		if(!TPX64XXQ_SX_XIO0_DATA_IS_ON) goto error;

		TPX64XXQ_SET_SX_XIO0_DATA_OFF;
		if(!TPX64XXQ_SX_XIO0_DATA_IS_OFF) goto error;


		// test MX-SIOX TM/XIO-1 output pin
		TPX64XXQ_SET_SX_XIO1_DATA_ON;
		if(!TPX64XXQ_SX_XIO1_DATA_IS_ON) goto error;

		TPX64XXQ_SET_SX_XIO1_DATA_OFF;
		if(!TPX64XXQ_SX_XIO1_DATA_IS_OFF) goto error;



        // test read-only DSP external IOX control registers first
		if( 	!TPX64XXQ_BOARD
			||	!TPX64XXQ_REV_ID_IS_REV1
			||  !(TPX64XXQ_DSP_SPEED_ID_IS_600MHZ || TPX64XXQ_DSP_SPEED_ID_IS_720MHZ)
		  )
		  	goto error;


		// test DSP external INT4..7/NMI source selectors
		// !!!note!!!: NMI source selector must not be tested since
		//			   no NMI interrupt handler is provided in this demo

		// ensure that NMI selector is OFF
		TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_NMI_SEL);
	  	if(!TPX64XXQ_EXT_INT_SEL_IS_OFF(TPX64XXQ_NMI_SEL)) goto error;


		for(i=0;i<=7;i++)
		{
	          TPX64XXQ_set_ext_int6_sel(i);
	          TPX64XXQ_set_ext_int7_sel(~i);
	          TPX64XXQ_set_ext_int4_sel(i+4);
	          TPX64XXQ_set_ext_int5_sel(i+2);

			  if(	TPX64XXQ_get_ext_int6_sel()!=i
				 || TPX64XXQ_get_ext_int7_sel()!=((~i) &0x7)
				 || TPX64XXQ_get_ext_int4_sel()!=((i+4) &0x7)
				 || TPX64XXQ_get_ext_int5_sel()!=((i+2) &0x7)
				)
				goto error;

		}

		TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT4_SEL);
	    TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT5_SEL);
	    TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT6_SEL);
	    TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT7_SEL);

    }

error:
	// idle here on any error condition
    while(1) ;

}



