/*******************************************************************************
  Shared SRAM (SSRAM) memory test for PX64xxQ DSP Coprocessors.
  (C) MicroLAB Systems, 2003


  Notes:
	1. 	This test automatically checks for SSRAM is installed
		and configures itself for SSRAM length (either 32Kx32 or 64Kx32).

	2.	This test performs continuous test of full on-board SSRAM and
		increments PASS COUNTER on each pass. MX-SIOX TM/XIO-0 pin is toggled at
		each successful pass.

	3.	This test cannot run simultaneously	by all DSP nodes.
*******************************************************************************/

// include headers
#include "stddef.h"
#include "px64xxq_dsp.h"


unsigned 	*ptr=NULL,
			*DMEM_START_PTR=NULL,
			*DMEM_END_PTR=NULL;

unsigned 	start_data = 0xa596e73b,
         	start_data_block,
			mem_inc_data = 0x97b5f3be,
			mem_wr_data,
			*pass_counter;

#define	PASS_COUNTER_ADDR	0xffff0

main()
{
	// initialize DSP environment
   	TPX64XXQ_init_dsp();

   	// initialize pass_counter pointer. PASS_COUNTER allocated at specific address
	// is used by host DSP s/w in order to track successfull proceeding of this test
	pass_counter = (unsigned *) PASS_COUNTER_ADDR;

	// reset PASS COUNTER
	*pass_counter = 0;

	// configure MX-SIOX TM/XIO-0 pin as general purpose OUTPUT in order to use it as
	// successful test procurement indicator
	TPX64XXQ_SET_SX_XIO0_CNF_OUTPUT;
	TPX64XXQ_set_sx_xio0_data(0);

	// assign SSRAM start pointer
	DMEM_START_PTR	= ((unsigned *) TPX64XXQ_SSRAM_SADDR);

	// now check whether SSRAM is installed
	if(TPX64XXQ_SSRAM_IS_NOT_INSTALLED)		goto ssram_not_installed;

    // set SSRAM end pointer
	DMEM_END_PTR	= ((unsigned *) TPX64XXQ_SSRAM_EADDR);

	// start infinite testing
	while(1)
    {
      	start_data_block= start_data;

		for(mem_wr_data=start_data, ptr=DMEM_START_PTR;
	        ptr<=DMEM_END_PTR ;
			*ptr++ = mem_wr_data++);

      	start_data=start_data_block;

		for(mem_wr_data=start_data, ptr=DMEM_START_PTR;
	        ptr<=DMEM_END_PTR ;
		)
   	   	{
        	if(*ptr++ != mem_wr_data++)
				goto error ;
	   	}

		// update start pattern for next pass
        start_data += mem_inc_data ;

        // increment the pass counter
		(*pass_counter)++;

		// toggle MX-SIOX TM/XIO-0 pin as successfull procurement indicator
		TPX64XXQ_set_sx_xio0_data(~TPX64XXQ_get_sx_xio0_data() & 0x1);
    }


error:
	// get here on any error
    while(1) ;

ssram_not_installed:
	// get here if SSRAM is not installed
    while(1) ;

}


