/*************************************************************************
  TORNADO-PX64xxQ DSP Coprocessor DCM Demo S/W Utilities. Rev.1A
  Host DSP Loader for TORNADO-PX64xxQ on-board DSP partial SSRAM test for TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.
  (Host TORNADO-6x/P6x/P64xx/E6x/E64xx driver)

  (C) MicroLAB Systems Ltd, 2003

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com



   File:
   -----
   	Host TORNADO-6x/P6x/P64xx/E6x/E64xx loader.


   Description:
   ------------
	This host DSP demo loads and starts PX64xxQ partial SSRAM test for all PX64xxQ
	DSPs. This demo requires the C6xxx C-BSF file for the corresponding PX64xxQ test.


   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board specific DSP memory map.
	This demo does not utilize any host	PC ISA/PCI resources for TORNADO-6x/P6x/P64xx
	DSP boards and can run in stand-alone mode.

	MIRAGE emulator or COFF-loader shall be used to upload code to host
	TORNADO board. TI CCS tools are required to recompile this demo.


   How to compile and link this demo:
   ----------------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular host TORNADO platform and C64xx DSP on-chip HPI
	port data format, the corresponding run-time compiler definition must be set
	in PX64XXQ_HOST_C6X.CFG file prior compiling the demo.

	Also, the corresonding linker command file must be used to link	demo for
	C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  PX64XXQ_HIF_C6X.CFG file for
	more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/

// include header files
#include	"math.h"
#include	"stddef.h"
#include	"px64xxq_host_c6x.cfg"	// include demo configuration file


// include C6xxx C-BSF file with the PX64xxQ executable code for host TORNADO loader
// Use C6X_CBSF.EXE utility in order to convert C64xx I/O scan executable code (PX64XXQ_SSRAM_PART.OUT)
// into corresponding C6xx C-BSF file (.CBS).
#include	"px64xxq_ssram_part.cbs"

// LED blinking period pause
#define		_wait_blinking_period()		for(k=0;k<10000000;k++)


main()
{
	int j,k,err;
	int n=0;	// this var is used for tracking error stage conditions
				// in case program will get to the ERROR: label

	// initialize host DSP
	init_host_dsp();

	// check whether SSRAM is installed
	if(HTPX64XXQ_DSP_SSRAM_IS_NOT_INSTALLED) goto ssram_is_not_installed;

	n=0;
	// reset PIOX I/F (this will set RESET for all DSPs, HPI16 data format control, no DSP bootmode,
	// clear all HPI errors and will enable HPI error control)
	if((err=HTPX64XXQ_init_piox())!=HTPX64XXQ_OK) goto error;


   	n=1;

 	// load DSP core code fm the C-BSF array
   	for(j=0;j<4;j++)
	{
		if((err=HTPX64XXQ_load_dsp_program(j,C6XXX_CBSF_DATA))!=HTPX64XXQ_OK)	goto error;

		// increment tracking var so we could always know what particular DSP has caused an error
		n++;
	}


	// reset all on-board DSPs
	HTPX64XXQ_SET_DSP_RESET_ON_ALL;

	// now set no DSP bootmode in order all DSPs can start execution immediately
	HTPX64XXQ_SET_DSP_BMODE_CNTR_NONE;

	// now release DSP RESET for all DSPs in order to start synchronous execution
	HTPX64XXQ_SET_DSP_RESET_OFF_ALL;


exit_ok:
	// get here on successful execution
	// idle forever
   	while(1);


ssram_is_not_installed:
	// get here if SSRAM is not installed
	// perform periodical blinking of all DSP RESET LEDs
	while(1)
	{
        HTPX64XXQ_SET_DSP_RESET_OFF_ALL;
		_wait_blinking_period();
        HTPX64XXQ_SET_DSP_RESET_ON_ALL;
		_wait_blinking_period();
	}

error:
	// get here on any error occured
	// Idle forever with the '0101' DSP reset bin pattern.
	HTPX64XXQ_SET_DSP_RESET_ON_ALL;
   	HTPX64XXQ_set_cntr1_rg(0x5);
	while(1);
}


