/****************************************************
   TORNADO-PX64xxQ Demo Utility.
   DSP MX-SIOX test (SIO-0/1 ports, IRQ-0/1, TM/XIO-0/1). rev.1A
   (C) MicroLAB Systems, 2003


   Description:
   ------------
	This demo tests MX-SIOX I/F comprising of tests for SIO-0/1 serial
	ports (routed to DSP on-chip McBSP-0/1 ports), MX-SIOX IRQ-0/1,
	and MX-SIOX TM/XIO-0/1 I/O signals.

	External DX-to-DR shorting jumpers are equired for each of the
	MX-SIOX SIO-0/1ports in order to run this demo. These jumpers can be set
	eaither at TORNADO-PX64xxQ on-board MX-SIOX connectors directly, or
	at SIOX rev.B sites of connected MX-SIOX mini-extender kits. Refer to
	TORNADO-E64xx User's Guide for more details about MX-SIOX site headers.

	This test configures all DSP McBSP-0/1 pins as OUTPUT pins except for
	DR pin, which is configured as INPUT, and continuously generates
	'running '1'' over DX/DR, FSX, FSR, CLKX and CLKR pins
	for each McBSP port. Oscilloscope can be used to view pulses at
	pins of McBSP-0/1/2 ports.

	DSP LINK Configuration register of host PIOX/PIOX-16 environment must
	be set to the '0' state in order to route DSP' McBSP-0/1 serial ports to
	on-board MX-SIOX I/F for each DSP. This i defaul setting on PIOX reset
	condition.

    Each time when active interrupt event is generated externally at MX-SIOX
	IRQ-0/1 external interrupt inputs, MX-SIOX TM/XIO-0/1 outputs are toggled.
	Use external pulse generator (500 kHz, Q=2) in order to apply MX-SIOX IRQ-0/1
	interrupt requests and observe MX-SIOX TM/XIO-0/1 output correspondingl
	togling with the half of IRQ-0/1 input frequency synchronized to the falling
	edge of IRQ-0/1 input signal.




   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
	- DSP bootmode: NOBOOT
	- DSP external vertical/diagonal links shall be disabled via
	  host DSP LINK CONFIGURATION register (default setting on
	  host PIOX reset condition)
*****************************************************/


// include headers
#include "stddef.h"
#include "px64xxq_dsp.h"



// interrupt handler for EXT_INT4
// toggles MX-SIOX TM/XIO-0 output at each IRQ-0 active interrupt request event
interrupt void xint4_isr()
{
  TPX64XXQ_set_sx_xio0_data(~TPX64XXQ_get_sx_xio0_data() &0x1);
}


// interrupt handler for EXT_INT5
// toggles MX-SIOX TM/XIO-1 output at each IRQ-1 active interrupt request event
interrupt void xint5_isr()
{
  TPX64XXQ_set_sx_xio1_data(~TPX64XXQ_get_sx_xio1_data()  &0x1);
}

// interrupt handler for EXT_INT6
// (not used in this demo)
interrupt void xint6_isr()
{
}


// interrupt handler for EXT_INT7
// (not used in this demo)
interrupt void xint7_isr()
{
}


// interrupt handler for NMI
// (not used in this demo)
// NMI ISR requires special declaration convention
#pragma NMI_INTERRUPT(nmi_isr)
void nmi_isr()
{
}

//McBSP output pins state defs
#define DX_OUT_0      		0x3f00
#define DX_OUT_1      		0x3f20
#define FSX_OUT_0     		0x3f00
#define FSX_OUT_1     		0x3f08
#define FSR_OUT_0     		0x3f00
#define FSR_OUT_1     		0x3f04
#define CLKX_OUT_0    		0x3f00
#define CLKX_OUT_1    		0x3f02
#define CLKR_OUT_0    		0x3f00
#define CLKR_OUT_1    		0x3f01

#define DR_BIT_MASK   		0x10
#define CLKS_BIT_MASK 		0x40
#define CLKR_BIT_MASK 		0x1
#define CLKX_BIT_MASK 		0x2
#define FSX_BIT_MASK  		0x8
#define FSR_BIT_MASK  		0x4



// some time delay is required betwen setting DX and testing
// returned DR due to the extender cable delays
#define _pause()			for(i=0;i<30;i++)



main()
{
	int i;

	// initialize DSP environment
   	TPX64XXQ_init_dsp();

	// configure MX-SIOX TM/XIO-0/1 pins as general purpose OUTPUTS
	TPX64XXQ_SET_SX_XIO0_CNF_OUTPUT;
	TPX64XXQ_SET_SX_XIO1_CNF_OUTPUT;
	if(		!TPX64XXQ_SX_XIO0_CNF_IS_OUTPUT
		||	!TPX64XXQ_SX_XIO1_CNF_IS_OUTPUT
	  )
	  	goto error;


	// configure DSP INT4/5 external interrupt selectors
	TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_NMI_SEL);			// disable NMI
	TPX64XXQ_SET_EXT_INT_SEL_SXIRQ0(TPX64XXQ_EXT_INT4_SEL);	// configure EXT_INT4 to SXIRQ0
	TPX64XXQ_SET_EXT_INT_SEL_SXIRQ1(TPX64XXQ_EXT_INT5_SEL);	// configure EXT_INT5 to SXIRQ1
	TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT6_SEL);	// disable EXT_INT6 selector
	TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT7_SEL);	// disable EXT_INT7 selector

	// enable/disable external DSP interrupts via DSP on-chip IER register
	C6X_INT_ENABLE(4);  // enable EXT_INT4/5
   	C6X_INT_ENABLE(5);
   	C6X_INT_DISABLE(6);
   	C6X_INT_DISABLE(7); // disable EXT_INT6/7

   	C6X_INT_CLEAR_ALL;
   	C6X_GINT_ENABLE;    // set GIE bit in order to enable external DSP interrupts


	// configure DSP McBSP-0/1 port pins as outputs
   	*C6X_MCBSP0_SPCR = 0;   /* configure McBSP-0 for I/O pins */
   	*C6X_MCBSP1_SPCR = 0;   /* configure McBSP-1 for I/O pins */

	// enter infinite test loop with interrupts enabled
	while(1)
    {
		// wait a little in order McBSP pin set instructions fm the previous cycle will go out of queue
		_pause();

		// generate active low signal at MX-SIOX RESET output pin in the beginning of each pass
		// Note, that RESET ON state is default setting after DSP initialization.
		TPX64XXQ_SET_SX_RESET_OFF;
		TPX64XXQ_SET_SX_RESET_ON;

    	// check whether McBSP-0 port is not configured as DIAGONAL link
    	if(TPX64XXQ_DIAG_LINK_ENABLE_IS_OFF)
    	{
			// test McBSP-0 port
			// set DX=1 and check returnd DR signal
	      	*C6X_MCBSP0_PCR = DX_OUT_1;
			_pause();
    		if((*C6X_MCBSP0_PCR & DR_BIT_MASK  ) ==0) break;

			// set DX=0 and check returnd DR signal
      		*C6X_MCBSP0_PCR = DX_OUT_0;
			_pause();
      		if((*C6X_MCBSP0_PCR & DR_BIT_MASK  ) !=0) break;

      		*C6X_MCBSP0_PCR = FSX_OUT_1;
      		*C6X_MCBSP0_PCR = FSX_OUT_0;

      		*C6X_MCBSP0_PCR = FSR_OUT_1;
      		*C6X_MCBSP0_PCR = FSR_OUT_0;

      		*C6X_MCBSP0_PCR = CLKX_OUT_1;
      		*C6X_MCBSP0_PCR = CLKX_OUT_0;

      		*C6X_MCBSP0_PCR = CLKR_OUT_1;
      		*C6X_MCBSP0_PCR = CLKR_OUT_0;
     	}



    	// check whether McBSP-1 port is not configured as VERTICAL link
    	if(TPX64XXQ_VERT_LINK_ENABLE_IS_OFF)
    	{
			// test McBSP-1 port
			// set DX=1 and check returnd DR signal
	      	*C6X_MCBSP1_PCR = DX_OUT_1;
			_pause();
    		if((*C6X_MCBSP1_PCR & DR_BIT_MASK  ) ==0) break;

			// set DX=0 and check returnd DR signal
      		*C6X_MCBSP1_PCR = DX_OUT_0;
			_pause();
      		if((*C6X_MCBSP1_PCR & DR_BIT_MASK  ) !=0) break;

      		*C6X_MCBSP1_PCR = FSX_OUT_1;
      		*C6X_MCBSP1_PCR = FSX_OUT_0;

      		*C6X_MCBSP1_PCR = FSR_OUT_1;
      		*C6X_MCBSP1_PCR = FSR_OUT_0;

      		*C6X_MCBSP1_PCR = CLKX_OUT_1;
      		*C6X_MCBSP1_PCR = CLKX_OUT_0;

      		*C6X_MCBSP1_PCR = CLKR_OUT_1;
      		*C6X_MCBSP1_PCR = CLKR_OUT_0;
     	}
	}


error:
	// idle here on any error condition
    while(1) ;

}





