/*************************************************************************
  TORNADO-PX64xxQ DSP Coprocessor DCM Demo S/W Utilities. Rev.1A
  Host DSP to TORNADO-PX64xxQ on-board DSP communication test for host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.
  (Host TORNADO-6x/P6x/P64xx/E6x/E64xx driver)

  (C) MicroLAB Systems Ltd, 2003

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com



   File:
   -----
   	Host TORNADO-6x/P64xx/E6x/E64xx driver.


   Description:
   ------------
	This demo performs communication between host TORNADO-6x/P6x/E6x/E64xx on-board DSP and
	TORNADO-PX64xxQ on-board DSPs via HPI ports and mutual interrupts. This demo requires the
	corresponding demo utility is running inside TORNADO-PX64xxQ on-board DSPs.


   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board specific DSP memory map.
	This demo does not utilize any host	PC ISA/PCI resources for TORNADO-6x/P6x/P64xx
	DSP boards and can run in stand-alone mode.

	MIRAGE emulator or COFF-loader shall be used to upload code to host
	TORNADO board. TI CCS tools are required to recompile this demo.


   How to compile and link this demo:
   ----------------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular host TORNADO platform and C64xx DSP on-chip HPI
	port data format, the corresponding run-time compiler definition must be set
	in PX64XXQ_HOST_C6X.CFG file prior compiling the demo.

	Also, the corresonding linker command file must be used to link	demo for
	C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  PX64XXQ_HIF_C6X.CFG file for
	more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/

// include header files
#include	"math.h"
#include	"stddef.h"
#include	"px64xxq_host_c6x.cfg"	// include demo configuration file

// include C6xx C-BSF file with DSP kernel code for this demo
#include 	"px64xxq_hcomm_dsp.cbs"

//======================== local vars and defs ======================================
// MCD host-to-DSP communication descriptor address in DSP on-chip RAM
#define	MCD_ADDR				0xfff00


// defs for MCD host-to-DSP communication descriptor vars
#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0
#define	MCMND_HPI_COMM			1
#define	MCMND_IRQ_COMM			2
#define	MCMND_HPI_HINT_COMM		3
#define	MCMND_BROADCAST_COMM	4

#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-102
#define	ERR_INV_PARAM			-101
#define	ERR_INV_INIT			-100
#define	ERR_CMND_EXEC_ERR		-103


// Components addresses for MCD host-to-DSP communication descriptor
// Access to MCD is performed via C64xx DSP on-chip HPI port.
#define	MCD_MID_ADDR			MCD_ADDR
#define	MCD_PRG_ID_ADDR			(MCD_ADDR+4)
#define	MCD_CMND_ADDR			(MCD_ADDR+8)
#define	MCD_STATUS_ADDR			(MCD_ADDR+0xc)
#define	MCD_ERROR_ADDR			(MCD_ADDR+0x10)
#define	MCD_UP0_ADDR			(MCD_ADDR+0x14)
#define	MCD_UP1_ADDR			(MCD_ADDR+0x18)
#define	MCD_UP2_ADDR			(MCD_ADDR+0x1c)
#define	MCD_UP3_ADDR			(MCD_ADDR+0x20)

// macros for accessing particular components of MCD host-to-DSP communication descriptor
#define	SET_MCD_MID(j,d)		HTPX64XXQ_write_dsp_hpi_data(j,MCD_MID_ADDR, d)
#define	SET_MCD_PRG_ID(j,d)		HTPX64XXQ_write_dsp_hpi_data(j,MCD_PRG_ID_ADDR, d)
#define	SET_MCD_CMND(j,d)		HTPX64XXQ_write_dsp_hpi_data(j,MCD_CMND_ADDR, d)
#define	SET_MCD_STATUS(j,d)		HTPX64XXQ_write_dsp_hpi_data(j,MCD_STATUS_ADDR, d)
#define	SET_MCD_ERROR(j,d)		HTPX64XXQ_write_dsp_hpi_data(j,MCD_ERROR_ADDR, d)
#define	SET_MCD_UP0(j,d)		HTPX64XXQ_write_dsp_hpi_data(j,MCD_UP0_ADDR, d)
#define	SET_MCD_UP1(j,d)		HTPX64XXQ_write_dsp_hpi_data(j,MCD_UP1_ADDR, d)
#define	SET_MCD_UP2(j,d)		HTPX64XXQ_write_dsp_hpi_data(j,MCD_UP2_ADDR, d)
#define	SET_MCD_UP3(j,d)		HTPX64XXQ_write_dsp_hpi_data(j,MCD_UP3_ADDR, d)

#define	MCD_MID(j)				HTPX64XXQ_read_dsp_hpi_data(j,MCD_MID_ADDR)
#define	MCD_PRG_ID(j)			HTPX64XXQ_read_dsp_hpi_data(j,MCD_PRG_ID_ADDR)
#define	MCD_CMND(j)				HTPX64XXQ_read_dsp_hpi_data(j,MCD_CMND_ADDR)
#define	MCD_STATUS(j)			HTPX64XXQ_read_dsp_hpi_data(j,MCD_STATUS_ADDR)
#define	MCD_ERROR(j)			HTPX64XXQ_read_dsp_hpi_data(j,MCD_ERROR_ADDR)
#define	MCD_UP0(j)				HTPX64XXQ_read_dsp_hpi_data(j,MCD_UP0_ADDR)
#define	MCD_UP1(j)				HTPX64XXQ_read_dsp_hpi_data(j,MCD_UP1_ADDR)
#define	MCD_UP2(j)				HTPX64XXQ_read_dsp_hpi_data(j,MCD_UP2_ADDR)
#define	MCD_UP3(j)				HTPX64XXQ_read_dsp_hpi_data(j,MCD_UP3_ADDR)



int 		n=0,						// this var is used for tracking error conditions
			number_of_xirqs=0;  		// this var is set to the total number of available IRQs upon the board type


//===================================================================================


// This function returns stutus of the IRQ-0..3 flags for XIRQ-0/1/2/3 on-board SIOX/PIOX interrupt requests
// Note, that different TORNADO DSP systems/controllers have different number and require mapping of PIOX
// interrupt requests. Actual host DSP INT# must be '+4' since they are mapped to EXT_INT4..7.

int	xirq_data(n)
{
 	return((C6X_INT_IS_OFF(n+4)) ? 0:1);
}


#define	_wait_display_pause()		for(j=0;j<1000000;j++)

main()
{
	unsigned j,k,kk;
	int err;

	// initialize host DSP
	init_host_dsp();

	n=10;
	// reset PIOX I/F (this will set RESET for all DSPs, HPI16 data format control, no DSP bootmode,
	// clear all HPI errors and will enable HPI error control)
	if((err=HTPX64XXQ_init_piox())!=HTPX64XXQ_OK) goto error;


   	n=11;

	// for TORNADO boards with 32-bit PIOX I/F, set HPI32 data format mode
    if(HTPX64XXQ_PX_FMT_IS_PX32)
	{
		HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI32;
		if(!HTPX64XXQ_DSP_HPI_FMT_CNTR_IS_HPI32) goto error;
	}
	else
	{
		HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI16;
		if(!HTPX64XXQ_DSP_HPI_FMT_CNTR_IS_HPI16) goto error;
	}


	n=12;

 	// load DSP core code fm the C-BSF array
   	for(j=0;j<4;j++)
	{
		if((err=HTPX64XXQ_load_dsp_program(j,C6XXX_CBSF_DATA))!=HTPX64XXQ_OK)	goto error;

		// increment tracking var so we could always know what particular DSP has caused an error
		n++;
	}


	// perform test for each DSP core
   	for(j=0;j<4;j++)
	{
	   	n=20;

		// disable all host IRQs
		HTPX64XXQ_DISABLE_HIRQ_ALL;

	   	if(   	!HTPX64XXQ_HIRQ_EN_IS_OFF(0)
	   		||	!HTPX64XXQ_HIRQ_SEL_IS_OFF(0)
	   		||	!HTPX64XXQ_HIRQ_EN_IS_OFF(1)
	   		||	!HTPX64XXQ_HIRQ_SEL_IS_OFF(1)
	   		||	!HTPX64XXQ_HIRQ_EN_IS_OFF(2)
	   		||	!HTPX64XXQ_HIRQ_SEL_IS_OFF(2)
	   		||	!HTPX64XXQ_HIRQ_EN_IS_OFF(3)
	   		||	!HTPX64XXQ_HIRQ_SEL_IS_OFF(3)
		  )
			goto error;

	   	n=21;

		// configure TORNADO on-board XIRQ selectors and set number of PIOX XIRQs for TP6x/TP64xx/E6x/E64xx boards
		// Note, that if interrupt mapping is available, then PIOX IRQs are mapped starting from EXT_INT4.
		// Number of available PIOX IRQs is returned via 'number_of_xirqs' var.

#if (_E6X_SELECTED)
			// for E6x boards, recognize between E62/E67 and E6202/E6203
			// E62/E67 feature only XIRQ-0..2 with the XIRQ-2 to be selected via EXT_INT6 INT SEL register,
			// whereas E6202/E6203 require all EXT_INT4..7 INT SEL registers to be configured to
			// connect to PX_IRQ-0..3.
		if(TE62_TE67_BOARD)
		{
				// this is E62/E67 board
            TE6X_SET_EXT_INT_SEL_XIRQ2(2);
			number_of_xirqs = 3;
		}
		else
		{
				// this is E6202/E6203 board
            TE6X_SET_EXT_INT_SEL_PXIRQ0(0);
            TE6X_SET_EXT_INT_SEL_PXIRQ1(1);
            TE6X_SET_EXT_INT_SEL_PXIRQ2(2);
            TE6X_SET_EXT_INT_SEL_PXIRQ3(3);
			number_of_xirqs = 4;
		}

		// set appropriate E6x interrupt polarity
		TE6X_INIT_EXT_INT_POLARITY;
#endif


#if (_E64XX_SELECTED)
			// for E64xx boards, all EXT_INT4..7 INT SEL registers to be configured to
			// connect to PX_IRQ-0..3.
		TE6X_SET_EXT_INT_SEL_PXIRQ0(0);
        TE6X_SET_EXT_INT_SEL_PXIRQ1(1);
        TE6X_SET_EXT_INT_SEL_PXIRQ2(2);
        TE6X_SET_EXT_INT_SEL_PXIRQ3(3);

		number_of_xirqs = 4;

		// set appropriate E6x interrupt polarity
		TE6X_INIT_EXT_INT_POLARITY;
#endif



#if (_T6X_SELECTED)
			// for T6x boards, no configuration is required except for the external interrupt polarity
		C6X_SET_XINT_POLARITY_ALL(C6X_XINT_POLARITY_NEG);
		number_of_xirqs = 3;
#endif


#if (_TP6X_SELECTED)
			// for TP62xx/TP67 boards, only XIRQ-2/3 interrupt requests are available from PIOX I/F
		TP6X_SET_EXT_INT_SEL_XIRQ2(0);
		TP6X_SET_EXT_INT_SEL_XIRQ3(1);
		C6X_SET_XINT_POLARITY_ALL(C6X_XINT_POLARITY_POS);
		number_of_xirqs = 2;
#endif


#if (_TP64XX_SELECTED)
			// for TP64xx boards, all PXIRQ-0..3 interrupt requests are available from PIOX I/F
		TP6X_SET_EXT_INT_SEL_PXIRQ0(0);
		TP6X_SET_EXT_INT_SEL_PXIRQ1(1);
		TP6X_SET_EXT_INT_SEL_PXIRQ2(2);
		TP6X_SET_EXT_INT_SEL_PXIRQ3(3);
		C6X_SET_XINT_POLARITY_ALL(C6X_XINT_POLARITY_POS);
		number_of_xirqs = 4;
#endif


		// clear C6x external interrupt requests and verify the result
		C6X_INT_CLEAR_ALL;

		if(   	xirq_data(4)!=0
    	  	||	xirq_data(5)!=0
    	  	||	number_of_xirqs>2 && xirq_data(6)!=0
    	  	||	number_of_xirqs>3 && xirq_data(7)!=0
     	  )
			goto error;


	   	n=22;

	   	// initialize MCD, start DSP execution via HPI DSPINT and verify returned DSP reply via MCD
	   	SET_MCD_MID(j,0xffffffff);
	   	SET_MCD_PRG_ID(j,0xffffffff);
	   	SET_MCD_CMND(j,0xffffffff);
	   	SET_MCD_STATUS(j,0xffffffff);
	   	SET_MCD_ERROR(j,0x7fffffff);
	   	SET_MCD_UP0(j,0xffffffff);
	   	SET_MCD_UP1(j,0xffffffff);

		HTPX64XXQ_START_DSP_AFTER_HPI_BOOT(j);


			// wait for DSP will init
   	   	while (MCD_STATUS(j)!=MSTAT_RDY)
	    {
      		if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(j)) goto error;
    	}


	   	n=23;

		// validate MCD data returned
   		if(	   	MCD_MID(j)!=M_ID
      		||	MCD_PRG_ID(j)!=PRG_ID
      		|| 	MCD_CMND(j)!=MCMND_NOP
      		|| 	MCD_STATUS(j)!=MSTAT_RDY
      		|| 	MCD_ERROR(j)!=ERR_OK
      		|| 	MCD_UP0(j)!=0x55555555
      		|| 	MCD_UP1(j)!=0xaaaaaaaa
     	 )
		     goto error;




	   	n=24;

		// test communication via HPI
	   	SET_MCD_CMND(j,MCMND_HPI_COMM);
	   	SET_MCD_ERROR(j,0x7fed);
	   	SET_MCD_UP0(j,0xffffffff);
	   	SET_MCD_UP1(j,0xffffffff);
	   	SET_MCD_UP2(j,0xffffffff);
	   	SET_MCD_UP3(j,0xffffffff);
	   	SET_MCD_STATUS(j,MSTAT_ATTN);


			// wait for DSP will reply
   	   	while (MCD_STATUS(j)!=MSTAT_RDY)
	    {
      		if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(j)) goto error;
    	}


	   	n=30;

        	// verify returned MCD data
  	   	if(     MCD_ERROR(j)!=ERR_OK
      		|| 	MCD_UP0(j)!=0xfedc1945
      		|| 	MCD_UP1(j)!=0xba98ec13
      		|| 	MCD_UP2(j)!=0x76543921
      		|| 	MCD_UP3(j)!=0x3210fa9c
     	  )
	       	goto error;


	   	n=31;

		// test communication via DSPINT ...
	   	SET_MCD_CMND(j,MCMND_IRQ_COMM);
	   	SET_MCD_ERROR(j,0x7fffff);
	   	SET_MCD_UP0(j,0xffffffff);
	   	SET_MCD_UP1(j,0xffffffff);
	   	SET_MCD_UP2(j,0xffffffff);
	   	SET_MCD_UP3(j,0x514379ef);

	   	HTPX64XXQ_SET_DSP_HPIC_DSPINT_ON(j);


			// wait for DSP will reply via MCD ERROR var
   	   	while(MCD_ERROR(j)!=ERR_OK)
	    {
      		if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(j)) goto error;
    	}


	   	n=40;

        	// verify returned MCD data
  	   	if(     MCD_UP0(j)!=0x01234567
      		|| 	MCD_UP1(j)!=0x89abcdef
      		|| 	MCD_UP2(j)!=0xfedc7654
      		|| 	MCD_UP3(j)!=((~0x514379ef) & 0xffffffff)
     	  )
	       	goto error;




	   	// testing HPI/HINT for different PIOX IRQs
		// PIOX IRQs are mapped/available starting from EXT_INT4 host DSP interrupt.
		// Number of mapped PIOX IRQs is available via 'number_of_xirqs' variable.
		for(k=0;k<number_of_xirqs;k++)
		{
			n=50;
      		if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(j)) goto error;
 		    SET_MCD_STATUS(j,MSTAT_RDY);


				// enable XIRQ#k and select HINT(j) as the IRQ source
		    HTPX64XXQ_SET_HIRQ_EN_ON(k);
	   		HTPX64XXQ_SET_HIRQ_SEL_DSP_HINT(k,j);

			n=51;
		   	if(   	xirq_data(k)!=0
    		  	|| 	!HTPX64XXQ_HIRQ_SEL_IS_DSP_HINT(k,j)
      		  	|| 	!HTPX64XXQ_DSP_HINT_IS_OFF(j)
	      	  	|| 	!HTPX64XXQ_HIRQ_EN_IS_ON(k)
		      )
				goto error;


		   	SET_MCD_CMND(j,MCMND_HPI_HINT_COMM);
		   	SET_MCD_ERROR(j,0x7fed);
		   	SET_MCD_STATUS(j,MSTAT_ATTN);

			n=52;

			// wait for DSP will reply
	   	   	while (MCD_STATUS(j)!=MSTAT_RDY)
		    {
      			if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(j)) goto error;
	    	}

			n=53;
	  	   	if(   	MCD_ERROR(j)!=ERR_OK
				||  !HTPX64XXQ_DSP_HINT_IS_ON(j)
			  )
		       	goto error;


			n=54;
			// check for correct XIRQ has been generated
		   	for(kk=0;kk<number_of_xirqs;kk++)
			{
				if(   	(kk!=k  && xirq_data(kk)!=0)
				  	|| 	(kk==k  && xirq_data(kk)==0)
				 )
		           	goto  error;
			}

			n=55;

			// clear all XINT IRQs
			C6X_INT_CLEAR_ALL;

			// clear HINT
		   	HTPX64XXQ_CLR_DSP_HPIC_HINT(j);
   		   	if(!HTPX64XXQ_DSP_HINT_IS_OFF(j))  goto error;

			// disable this HIRQ
	   		HTPX64XXQ_SET_HIRQ_SEL_OFF(k);
		    HTPX64XXQ_SET_HIRQ_EN_OFF(k);
		}
	}


	// finally, perform test of broadcast interrupt for all DSPs

	n=60;
    	// setup MCD descriptor for each DSP
    	// Note, that MCD.UP[0] and MCD.UP[1] data will be XORed by the DSP
	for(j=0;j<4;j++)
	{
	   	SET_MCD_CMND(j,MCMND_BROADCAST_COMM);
	   	SET_MCD_ERROR(j,0x7fffff);

		switch(j)
		{
			case 0:
	   			SET_MCD_UP0(j,0x9abd57c4);
	   			SET_MCD_UP1(j,0x5555aaaa);
	   			break;

			case 1:
	   			SET_MCD_UP0(j,0x6e25173d);
	   			SET_MCD_UP1(j,0xaaaa5555);
	   			break;

			case 2:
	   			SET_MCD_UP0(j,0xa87eb3d5);
	   			SET_MCD_UP1(j,0x55aa55aa);
	   			break;

			default:
	   			SET_MCD_UP0(j,0x396cdaf7);
	   			SET_MCD_UP1(j,0xaa55aa55);
	   			break;
	 	}

	   	SET_MCD_UP2(j,(MCD_UP0(j) ^ MCD_UP1(j)));	// save the result for further verification into UP[2]
	   	SET_MCD_UP3(j,0xffffffff);					// clear this var, since DSP will put reply here

		// check for HPI error
		if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(j)) goto error;
	}

		// generate common HOST BROADCAST IRQ
    HTPX64XXQ_SET_DSP_BROADCAST_IRQ;

		// now wait until all DSPs will reply via MCD error
	n=61;
			// wait for DSP will reply via MCD ERROR var
   	while(		MCD_ERROR(0)!=ERR_OK
	   		||	MCD_ERROR(1)!=ERR_OK
	   		||	MCD_ERROR(2)!=ERR_OK
	   		||	MCD_ERROR(3)!=ERR_OK
		 )
    {
    		// check for HPI error
    	for(j=0;j<4;j++)
    	{
   			if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(j)) goto error;
   		}
   	}


    	// now verify returned MCD data upon expected data
	for(j=0;j<4;j++)
	{
		n=62;
		if(MCD_UP2(j)!=MCD_UP3(j)) goto error;

		n=63;
		// check for HPI error
		if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(j)) goto error;
	}



	//---------------- execution traps ------------
exit_ok:
	// get here on successful execution
	// perform running of one DSP REST LED thru all LEDs
	HTPX64XXQ_SET_DSP_RESET_ON_ALL;
	while(1)
	{
		for(k=0;k<4;k++)
		{
			HTPX64XXQ_SET_DSP_RESET_OFF(k);
			_wait_display_pause();
			HTPX64XXQ_SET_DSP_RESET_ON(k);
		}
	}


error:
	// get here on any error occured
	// The following visual indication is provided:
	//	- set DSP RESETs to the the '0101' bin pattern in case
	//    error code N<20 (general init errors)
	//	- set DSP RESET to the OFF state for those DSP cores, which have passed test
	//	  successfully in case N>=20, and set DSP RESET to the ON for those, which did
	//	  not pass the test successfully
	HTPX64XXQ_SET_DSP_RESET_ON_ALL;

		// analyse error tracking code (N)
	if(n<20)
		// this was incorrect general init, load '01010101' pattern
	   HTPX64XXQ_set_cntr1_rg(0x5);
	else
	{
		// general initialization has passed successfully
		// Release DSP RESET LEDs for those DSP cores, which have passed
		// test successfully.
		if(j>0)
		{
			HTPX64XXQ_SET_DSPA_RESET_OFF;
			if(j>1)
			{
				HTPX64XXQ_SET_DSPB_RESET_OFF;
				if(j>2)
				{
					HTPX64XXQ_SET_DSPC_RESET_OFF;
					if(j>3)
						HTPX64XXQ_SET_DSPD_RESET_OFF;
				}
			}
		}
	}

	// finally, loop here infinitely at the error condition
    while(1) ;
}


