/****************************************************
  TORNADO-PX64xxQ Demo Utility.
  Host DSP to TORNADO-PX64xxQ on-board DSP communication test for host TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.
  (Driver for TORNADO-P64xxQ on-board DSP)

  (C) MicroLAB Systems Ltd, 2003

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com



   File:
   -----
   	Driver for TORNADO-P64xxQ on-board DSP.


   Description:
   ------------
	This demo performs communication between host TORNADO-6x/P6x/E6x/E64xx on-board DSP and
	TORNADO-PX64xxQ on-board DSPs via HPI ports and mutual interrupts. This demo requires the
	corresponding demo utility is running inside host TORNADO-6x/P6x/P64xx/E6x/E64xx on-board DSP.


   TORNADO platform:
   -----------------
	This demo utilizes host TORNADO DSP. This demo does not utilize any host PC resourses
	and can run in stand-alone mode for host TORNADO DSP system/controller MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
	None, all configuration is handled by host DSP.
*****************************************************/


// include headers
#include "stddef.h"
#include "px64xxq_dsp.h"

// include PX64xxQ C64xx DSP core code in C-BSF file
//#include "px64_mc.cbs"



//======================== local variables ========================================

// Host-to-DSP_MASTER communication descriptor
// -----------------------------------------------
// This descriptor is allocated to predefined memory location inside DSP on-chip memory
// for communication between TORNADO-PX64xxQ on-board C64xx DSP and host TORNADO on-board
// DSP.
struct MCD_DD
{
	int			m_id,		// device ID code
				prg_id,		// program ID code
				cmnd,		// command from host DSP
				status,		// command execution status
				error;		// command execution error code

   	unsigned    up[4];		// bidirectional communication parameters
} *MCD;

// allocation address for host-to-DSP communication descriptor
#define	MCD_ADDR				0xfff00


// defs for communication descriptor variables
#define	M_ID					0x110
#define	PRG_ID					0x111

#define	MCMND_NOP				0
#define	MCMND_HPI_COMM			1
#define	MCMND_IRQ_COMM			2
#define	MCMND_HPI_HINT_COMM		3
#define	MCMND_BROADCAST_COMM	4

#define	MSTAT_BUSY				2
#define	MSTAT_ATTN				1
#define	MSTAT_RDY				0

#define	ERR_OK					0
#define	ERR_INV_CMND			-102
#define	ERR_INV_PARAM			-101
#define	ERR_INV_INIT			-100
#define	ERR_CMND_EXEC_ERR		-103

//=================================================================================




//====================== Interrupt handlers =======================================

// interrupt handler for INT4
// INT4 is mapped to the DSPINT (host-to-DSP interrupt via HPI port)
interrupt void xint4_isr()
{
 	// when DSPINT is received, then specific data pattern must be returned via MCD communication params
	if(MCD->cmnd==MCMND_IRQ_COMM)
	{
		MCD->up[0]= 0x01234567;
	   	MCD->up[1]= 0x89abcdef;
	   	MCD->up[2]= 0xfedc7654;
	   	MCD->up[3]= ~MCD->up[3];
	   	MCD->error=ERR_OK;
	}
	else
	   	MCD->error=ERR_INV_CMND;

	// clear DSPINT interrupt request from host
	TPX64XXQ_CLR_HPIC_DSPINT;
}


// interrupt handler for INT5
// is mapped to EXT_INT5, which is mapped to HOST BROADCAST INTERRUPT
interrupt void xint5_isr()
{
 	// when DSPINT is received, then perform XOR and return data via MCD communication params
	if(MCD->cmnd==MCMND_BROADCAST_COMM)
	{
	   	MCD->up[3]= (MCD->up[0] ^ MCD->up[1]);
	   	MCD->error=ERR_OK;
	}
	else
	   	MCD->error=ERR_INV_CMND;
}


// interrupt handler for INT6
// is not used in theis demo
interrupt void xint6_isr()
{
}


// interrupt handler for INT7
// corresponds to VERTICAL IRQ
interrupt void xint7_isr()
{
}


// interrupt handler for NMI
// is not used in theis demo
#pragma NMI_INTERRUPT(nmi_isr)
void nmi_isr()
{
}
//=================================================================================



main()
{

	// initialize DSP environment
   	TPX64XXQ_init_dsp();

	// configure DSP INT4..7 and EXT_INT4..7/NMI external interrupt selectors
	TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_NMI_SEL);						// disable NMI
	TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT4_SEL);				// disable EXT_INT4
	TPX64XXQ_SET_EXT_INT_SEL_HOST_BROADCAST_IRQ(TPX64XXQ_EXT_INT5_SEL);	// configure EXT_INT5 to host broadcast IRQ
	TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT6_SEL);				// disable EXT_INT6
	TPX64XXQ_SET_EXT_INT_SEL_OFF(TPX64XXQ_EXT_INT7_SEL);				// disable EXT_INT7


	// configure INT4 to the interrupt source from HPI port
	C6X_INTMUX_SET(4,C6X_INTSEL_DSPINT);

	// enable/disable external DSP interrupts via DSP on-chip IER register
	C6X_INT_ENABLE(4);
   	C6X_INT_ENABLE(5);
   	C6X_INT_DISABLE(6);
   	C6X_INT_DISABLE(7);

   	C6X_INT_CLEAR_ALL;
   	C6X_GINT_ENABLE;    // set GIE bit in order to enable external DSP interrupts


	// assign MCD address and initialize MCD data so host DSP can recognize that initialization has passed OK
	MCD=(struct MCD_DD *) MCD_ADDR;

    MCD->m_id=M_ID;
    MCD->prg_id=PRG_ID;
    MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x55555555;
	MCD->up[1]=0xaaaaaaaa;
	MCD->status=MSTAT_RDY;


	// enter infinite test loop
   	while(1)
    {
		// wait for host-to-DSP request via MCD
       	while(MCD->status != MSTAT_ATTN);
       	MCD->status = MSTAT_BUSY;

		// decode received command
       	switch(MCD->cmnd)
		{
			// NOP command just returns OK
          	case MCMND_NOP:
	       		MCD->error=ERR_OK;
	       		break;

			// HPI communication command returns specific data pattern via MCD communication parameters
			case MCMND_HPI_COMM:
	        	MCD->up[0]=0xfedc1945;
				MCD->up[1]=0xba98ec13;
				MCD->up[2]=0x76543921;
				MCD->up[3]=0x3210fa9c;
	        	MCD->error=ERR_OK;
	        	break;

			// request for generation of HINT to host DSP
          	case MCMND_HPI_HINT_COMM:
				TPX64XXQ_SET_HPIC_HINT;
	 	        MCD->error=ERR_OK;
		        break;

			// unrecognized default command
          	default:
	        	MCD->error=ERR_INV_CMND;
	        	break;

		}

		// set command execution status to READY
		MCD->status=MSTAT_RDY;
    }
}

