/*************************************************************************
  TORNADO-PX64xxQ DSP Coprocessor DCM Demo S/W Utilities. Rev.1A
  C64xx DSP default program load from host DSP via PIOX I/F for TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  (C) MicroLAB Systems Ltd, 2003

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo loads default C64xx DSP reset program to all TORNADO-PX64xxQ on-board DSP
	via host PIOX I/F of TORNADO mainboard. If successfull, then all DSP are left in the RUN state, 
	otherwise the first DSP in the RESET state denotes where the error occurs.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board specific DSP memory map.
	This demo does not utilize any host	PC ISA/PCI resources for TORNADO-6x/P6x/P64xx
	DSP boards and can run in stand-alone mode.

	MIRAGE emulator or COFF-loader shall be used to upload code to host
	TORNADO board. TI CCS tools are required to recompile this demo.


   How to compile and link this demo:
   ----------------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular host TORNADO platform and C64xx DSP on-chip HPI
	port data format, the corresponding run-time compiler definition must be set
	in PX64XXQ_HOST_C6X.CFG file prior compiling the demo.

	Also, the corresonding linker command file must be used to link	demo for
	C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  PX64XXQ_HIF_C6X.CFG file for
	more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/

// include header files
#include	"math.h"
#include	"stddef.h"
#include	"px64xxq_host_c6x.cfg"	// include demo configuration file




main()
{
	int i,err;

	// initialize host DSP
	init_host_dsp();


	// set all DSP cores to the RESET state
   	HTPX64XXQ_SET_DSP_RESET_ON_ALL;
   	HTPX64XXQ_SET_DSP_BMODE_CNTR_HPI;
   	HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI16;
   	HTPX64XXQ_SET_DSP_HPI_ERR_EN_ON;
   	HTPX64XXQ_CLR_DSP_HPI_ERR_ALL;
    HTPX64XXQ_DISABLE_HIRQ_ALL;

	// call DSP default program loader for each DSP
	for(i=0;i<4;i++)
	{
		if((err=HTPX64XXQ_load_dsp_default_program(i))!=HTPX64XXQ_OK)
			goto error;
	}

	// now release all DSP RESET signals and put all DSPs to the RUN state
   	HTPX64XXQ_SET_DSP_RESET_OFF_ALL;

exit_ok:
	// get here on successful execution
    while(1);

error:
	// get here on any error occured
	// the first DSP in the RESET state (starting fm DSP#A denotes where the error occurs
    while(1) ;
}


