/*************************************************************************
  TORNADO-PX64xxQ DSP Coprocessor DCM Demo S/W Utilities. Rev.1A
  Host I/F registers test for TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  (C) MicroLAB Systems Ltd, 2003

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs test/scan of host I/F registers of TORNADO-PX64xxQ DSP Coprocessor 
	via host PIOX I/F of TORNADO mainboard.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board specific DSP memory map. 
	This demo does not utilize any host	PC ISA/PCI resources for TORNADO-6x/P6x/P64xx
	DSP boards and can run in stand-alone mode.

	MIRAGE emulator or COFF-loader shall be used to upload code to host
	TORNADO board. TI CCS tools are required to recompile this demo.


   How to compile and link this demo:
   ----------------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular host TORNADO platform and C64xx DSP on-chip HPI
	port data format, the corresponding run-time compiler definition must be set
	in PX64XXQ_HOST_C6X.CFG file prior compiling the demo.

	Also, the corresonding linker command file must be used to link	demo for
	C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  PX64XXQ_HIF_C6X.CFG file for
	more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/

// include header files
#include	"math.h"
#include	"stddef.h"
#include	"px64xxq_host_c6x.cfg"	// include demo configuration file


#define	_wait_display_pause()		for(j=0;j<1000000;j++)

main()
{
	int i,j,
		n;	// this var is used to identify test step, which will cause an error condition
			// once program will idle at the ERROR loop
  	unsigned d,pass_counter;


	// initialize host DSP
	init_host_dsp();

	// loop infinitely
 	while(1)
 	{
		n=0;

		// reset all registers initially
		HTPX64XXQ_set_cntr1_rg(0);
		HTPX64XXQ_set_cntr2_rg(0);

			// release RESET for all DSPs and set it back again in order to have
			// DSP CNF STATUS register to get cleared
		HTPX64XXQ_set_cntr1_rg(0xf);
		HTPX64XXQ_set_cntr1_rg(0);

		HTPX64XXQ_set_dsp_link_cnf_rg(0);
		HTPX64XXQ_CLR_DSP_HPI_ERR_ALL;
		HTPX64XXQ_set_hirq_sel_rg(0,0);
		HTPX64XXQ_set_hirq_sel_rg(1,0);
		HTPX64XXQ_set_hirq_sel_rg(2,0);
		HTPX64XXQ_set_hirq_sel_rg(3,0);

		// verify control registers contents
		if(		HTPX64XXQ_get_cntr1_rg()!=0
			||	(HTPX64XXQ_get_cntr2_rg() & (HTPX64XXQ_DSP_BMODE_CNTR_BITMASK | HTPX64XXQ_DSP_HPI_FMT_CNTR_BITMASK | HTPX64XXQ_DSP_HPI_ERR_EN_BITMASK))!=0
			||	HTPX64XXQ_get_dsp_cnf_stat_rg()!=0
			||	HTPX64XXQ_get_dsp_link_cnf_rg()!=0
			||	!HTPX64XXQ_BOARD
			||	!HTPX64XXQ_REV_ID_IS_REV1
			||	!(HTPX64XXQ_DSP_SPEED_ID_IS_600MHZ || HTPX64XXQ_DSP_SPEED_ID_IS_720MHZ)
			||	HTPX64XXQ_get_int_stat_rg()!=0
			||	HTPX64XXQ_get_hirq_sel_rg(0)!=0
			||	HTPX64XXQ_get_hirq_sel_rg(1)!=0
			||	HTPX64XXQ_get_hirq_sel_rg(2)!=0
			||	HTPX64XXQ_get_hirq_sel_rg(3)!=0
		  )
		  	goto error;


		n=1;

		// now test that D0..D3 databits are connected perfectly
		// (use CNTR1 and DSP LINK CNF registers with different data in order to exclude PIOX latched data read)
   		for(i=0,d=1;i<4;i++)
     	{
       		HTPX64XXQ_set_cntr1_rg(d);
       		HTPX64XXQ_set_dsp_link_cnf_rg(d>>1);

       		if(   	HTPX64XXQ_get_cntr1_rg()!=d
          		|| 	HTPX64XXQ_get_dsp_link_cnf_rg()!=(d>>1)
	 		  )
				goto error;
       		d <<= 1;
     	}


		n=2;

		// for T6x/TP6x/TP64xx boards, test D4..7 bits by means of setting HPI32 bit data format via CNTR2 register
		// and reading HPI DATA FORMAT STATUS bits via DSP CNF STATUS register
#if(_T6X_SELECTED || _TP6X_SELECTED || _TP64XX_SELECTED)
		HTPX64XXQ_SET_DSP_BMODE_CNTR_NONE;
		HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI16;
		HTPX64XXQ_SET_DSP_RESET_ON_ALL;

		// verify whether all HPI DATA FORMAT STATUS bits are reset
		if(	HTPX64XXQ_get_dsp_cnf_stat_rg()!=0)  goto error;

		HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI32;
		// verify whether all HPI DATA FORMAT STATUS bits are set
		if(	HTPX64XXQ_get_dsp_cnf_stat_rg()!=0xf0)  goto error;

		// now modify HPI data format control for different DSPs while releasing them fm RESET
		HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI16;

		n=3;
        HTPX64XXQ_SET_DSPA_RESET_OFF;
        if(HTPX64XXQ_get_dsp_cnf_stat_rg()!=0x00) goto error;
		HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI32;
        if(HTPX64XXQ_get_dsp_cnf_stat_rg()!=0xe0) goto error;

		n=4;
        HTPX64XXQ_SET_DSPB_RESET_OFF;
        if(HTPX64XXQ_get_dsp_cnf_stat_rg()!=0xe0) goto error;
		HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI16;
        if(HTPX64XXQ_get_dsp_cnf_stat_rg()!=0x20) goto error;

		n=5;
        HTPX64XXQ_SET_DSPC_RESET_OFF;
        if(HTPX64XXQ_get_dsp_cnf_stat_rg()!=0x20) goto error;
		HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI32;
        if(HTPX64XXQ_get_dsp_cnf_stat_rg()!=0xa0) goto error;

		n=6;
        HTPX64XXQ_SET_DSPD_RESET_OFF;
        if(HTPX64XXQ_get_dsp_cnf_stat_rg()!=0xa0) goto error;
		HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI16;
        if(HTPX64XXQ_get_dsp_cnf_stat_rg()!=0xa0) goto error;

		HTPX64XXQ_set_cntr2_rg(0);
		HTPX64XXQ_set_cntr1_rg(0);
#endif


		n=7;

        // now test address lines
		// (write/verify data patterns to different control registers)
		HTPX64XXQ_set_cntr1_rg(0xa);
		HTPX64XXQ_set_cntr2_rg(0x81);
		HTPX64XXQ_set_dsp_link_cnf_rg(0x5);
		HTPX64XXQ_set_hirq_sel_rg(0,0x82);
		HTPX64XXQ_set_hirq_sel_rg(1,0x4);
		HTPX64XXQ_set_hirq_sel_rg(2,0x1);
		HTPX64XXQ_set_hirq_sel_rg(3,0xff);


		// verify control registers contents
		if(		HTPX64XXQ_get_cntr1_rg()!=0xa
			||	(HTPX64XXQ_get_cntr2_rg() & (HTPX64XXQ_DSP_BMODE_CNTR_BITMASK | HTPX64XXQ_DSP_HPI_FMT_CNTR_BITMASK | HTPX64XXQ_DSP_HPI_ERR_EN_BITMASK))!=0x81
			||	HTPX64XXQ_get_dsp_link_cnf_rg()!=0x5
			||	HTPX64XXQ_get_hirq_sel_rg(0)!=0x82
			||	HTPX64XXQ_get_hirq_sel_rg(1)!=0x4
			||	HTPX64XXQ_get_hirq_sel_rg(2)!=0x1
			||	HTPX64XXQ_get_hirq_sel_rg(3)!=0x87
		  )
		  	goto error;


		// repeat this test with inversed data patterns
		n=8;

		HTPX64XXQ_set_cntr1_rg(~HTPX64XXQ_get_cntr1_rg());
		HTPX64XXQ_set_cntr2_rg(~HTPX64XXQ_get_cntr2_rg());
		HTPX64XXQ_set_dsp_link_cnf_rg(~HTPX64XXQ_get_dsp_link_cnf_rg());
		HTPX64XXQ_set_hirq_sel_rg(0,~HTPX64XXQ_get_hirq_sel_rg(0));
		HTPX64XXQ_set_hirq_sel_rg(1,~HTPX64XXQ_get_hirq_sel_rg(1));
		HTPX64XXQ_set_hirq_sel_rg(2,~HTPX64XXQ_get_hirq_sel_rg(2));
		HTPX64XXQ_set_hirq_sel_rg(3,~HTPX64XXQ_get_hirq_sel_rg(3));

		// verify control registers contents
		if(		HTPX64XXQ_get_cntr1_rg()!=0x5
			||	(HTPX64XXQ_get_cntr2_rg() & (HTPX64XXQ_DSP_BMODE_CNTR_BITMASK | HTPX64XXQ_DSP_HPI_FMT_CNTR_BITMASK | HTPX64XXQ_DSP_HPI_ERR_EN_BITMASK))!=0x2
			||	HTPX64XXQ_get_dsp_link_cnf_rg()!=0xa
			||	HTPX64XXQ_get_hirq_sel_rg(0)!=0x5
			||	HTPX64XXQ_get_hirq_sel_rg(1)!=0x83
			||	HTPX64XXQ_get_hirq_sel_rg(2)!=0x86
			||	HTPX64XXQ_get_hirq_sel_rg(3)!=0x0
		  )
		  	goto error;

		// increment pass counter
		pass_counter++;

		// get here after each pass on successful execution
		// perform running of one DSP REST LED thru all LEDs once in order user can view successfull procurement
		HTPX64XXQ_SET_DSP_RESET_ON_ALL;
		for(i=0;i<4;i++)
		{
			HTPX64XXQ_SET_DSP_RESET_OFF(i);
			_wait_display_pause();
			HTPX64XXQ_SET_DSP_RESET_ON(i);
		}
	}

error:
	// get here on any error condition
	// clear all control registers and idle forever
	HTPX64XXQ_set_cntr1_rg(0);
	HTPX64XXQ_set_cntr2_rg(0);
	HTPX64XXQ_set_dsp_link_cnf_rg(0);
	HTPX64XXQ_CLR_DSP_HPI_ERR_ALL;
	HTPX64XXQ_set_hirq_sel_rg(0,0);
	HTPX64XXQ_set_hirq_sel_rg(1,0);
	HTPX64XXQ_set_hirq_sel_rg(2,0);
	HTPX64XXQ_set_hirq_sel_rg(3,0);

	while(1) ;
}


