/*************************************************************************
  TORNADO-PX64xxQ DSP Coprocessor DCM Demo S/W Utilities. Rev.1A
  C64xx DSP memory test via HPI port from host DSP via PIOX I/F for TORNADO-6x/P6x/P64xx/E6x/E64xx DSP Systems/Controllers.

  (C) MicroLAB Systems Ltd, 2003

  WEB:    http://www.mlabsys.com
  FTP:    ftp://ftp.mlabsys.com
  E-mail: techsupport@mlabsys.com


   Description:
   ------------
	This demo performs C64xx DSP memory test by host DSP via C64xx DSP on-chip HPI port.
	If test procurement is successfull, then DSP RESET LEDs are all in the OFF state, otherwise 
	all DSP RESET LEDs are in the ON state.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board specific DSP memory map.
	This demo does not utilize any host	PC ISA/PCI resources for TORNADO-6x/P6x/P64xx
	DSP boards and can run in stand-alone mode.

	MIRAGE emulator or COFF-loader shall be used to upload code to host
	TORNADO board. TI CCS tools are required to recompile this demo.


   How to compile and link this demo:
   ----------------------------------
	This demo is TORNADO board specific, i.e. it must be compiled separately
	for each of TORNADO DSP platform (T6x/TP6x/TP64xx/E6x/E64xx).

	In order to select particular host TORNADO platform and C64xx DSP on-chip HPI
	port data format, the corresponding run-time compiler definition must be set
	in PX64XXQ_HOST_C6X.CFG file prior compiling the demo.

	Also, the corresonding linker command file must be used to link	demo for
	C6201/C6202/C6203/C6701 and C64xx DSP. Refer to  PX64XXQ_HIF_C6X.CFG file for
	more details.


   Copyright:
   ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
          - DCM is installed onto TORNADO-6x/E6x/E64xx/P6x/P64xx mainboard
*****************************************************************************/

// include header files
#include	"math.h"
#include	"stddef.h"
#include	"px64xxq_host_c6x.cfg"	// include demo configuration file



unsigned 	addr=0,
			DMEM_START_ADDR=0,
			DMEM_END_ADDR=0,
			start_data = 0xa596e73b,
         	start_data_block,
			mem_inc_data = 0x97b5f3be,
			mem_wr_data,
			mem_rd_data,
			pass_counter=0;



int test_dsp_memory(int dsp_node,unsigned 	saddr, unsigned eaddr, unsigned sdata)
// performs test within specified memory addresses
// returns:
//		0  		- no errors
//		-100	- HPI error
//		-101	- memory verification error
{
	// perform memory fill

   	// fill-in HPIA register
	HTPX64XXQ_set_dsp_hpia_rg(dsp_node,saddr);

   	// perform memory write using HPID-AINC register
	for(mem_wr_data=sdata, addr=saddr; addr<=eaddr ; )
	{
		HTPX64XXQ_set_dsp_hpid_ainc_rg(dsp_node,mem_wr_data);

		// check for HPI error
		if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(dsp_node))	return -100;

		addr += 4;
		mem_wr_data++;
	}

  	// perform memory verification using HPID register
	for(mem_wr_data=sdata, addr=saddr; addr<=eaddr ; )
   	{
   		mem_rd_data = HTPX64XXQ_read_dsp_hpi_data(dsp_node,addr);

		// check for HPI error
		if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(dsp_node))	return -100;

		if( mem_rd_data != mem_wr_data++) return -101;

		addr += 4;
	}

	return 0;
}





main()
{
	int i,err,
		n;		// this var allows to identify test stage, which caused and error condition

	// initialize host DSP
	init_host_dsp();

	n=0;
	// reset PIOX I/F (this will set RESET for all DSPs, HPI16 data format control, no DSP bootmode
	// and will enable HPI error control)
	if((err=HTPX64XXQ_init_piox())!=HTPX64XXQ_OK) goto error;

	// select DSP bootmode to HPI in order to keep DSP kernel in the RESET state while testing DSP memory
    HTPX64XXQ_SET_DSP_BMODE_CNTR_HPI;

	// perform infinite DSP on-chip, SSRAM and SDRAM memory tests for each DSP
	while(1)
	{
		// select between HPI16/HPI32 data format modes and configure DSP EMIF settings in each pass
		// if host TORNADO board has 32-bit PIOX I/F installed. For PIOX-16 I/F, configure DSP EMIF
		// setting at the first pass only.
        if(HTPX64XXQ_PX_FMT_IS_PX32 || (HTPX64XXQ_PX_FMT_IS_PX16 && pass_counter==0))
		{
			if((pass_counter%2)==0)
				HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI16;
			else
				HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI32;
		}


		// reset all DSPs in order to configure HPI data format mode
		HTPX64XXQ_SET_DSP_RESET_ON_ALL;

		// perform DSP memory tests for each DSP
    	for(i=0;i<=3;i++)
    	{
			n=1;
			// initialize DSP environment for DSP#i and release it from the RESET
			// all DSPs will stay in the RUN state after initialization of DSP environment
			if((err=HTPX64XXQ_init_dsp(i))!=HTPX64XXQ_OK)	goto error;


        	n=2;

			// perform test for DSP on-chip memory
			// ------------------------------------
			// Note:
			//		perform DSP on-chip RAM test up to the -1 word from the end in order to
			// 		exclude HPI DMA problems when crossing internal DSP on-chip DRAM up boundary, 
			//		which can result in stalling HPI DMA until DSP reset is applied	(this is true for C6203 DSP at least)

			if((err=test_dsp_memory(i,HTPX64XXQ_DSPE_ONCHIP_RAM_SADDR,HTPX64XXQ_DSPE_ONCHIP_RAM_EADDR-4,start_data))!=0)
				goto error;

			// update start pattern for next memory test
    	    start_data += mem_inc_data ;

        	n=3;

			// perform test for DSP external SSRAM memory (if it is installed)
			// ---------------------------------------------------------------
			if(HTPX64XXQ_DSP_SSRAM_IS_INSTALLED)
			{
				if((err=test_dsp_memory(i,HTPX64XXQ_DSPE_SSRAM_SADDR,HTPX64XXQ_DSPE_SSRAM_EADDR-4,start_data))!=0)
					goto error;

				// update start pattern for next memory test
    		    start_data += mem_inc_data ;
   			}


        	n=4;

			// perform test for DSP external SDRAM memory (if it is installed)
			// ---------------------------------------------------------------
			if(HTPX64XXQ_DSP_SDRAM_IS_INSTALLED)
			{
				if((err=test_dsp_memory(i,HTPX64XXQ_DSPE_SDRAM_SADDR,HTPX64XXQ_DSPE_SDRAM_EADDR-4,start_data))!=0)
					goto error;

				// update start pattern for next memory test
    		    start_data += mem_inc_data ;
   			}

		}

       	// increment the pass counter
		pass_counter++;
    }

error:
	// get here on any error occured
	// I, ERR, and N variables will identify the test stage at which the error occured
	// Put all DSPs into RESET state.
	HTPX64XXQ_SET_DSP_RESET_ON_ALL;
    while(1) ;
}


