/***************************************************************
  TORNADO-PX6414Q/PX6415Q/PX6416Q DSP utilities (C64xx DSP Environment). rev.1A-1
  (C) MicroLAB Systems Ltd, 2002-2003

  http://www.mlabsys.com
  ftp://ftp.mlabsys.com
  email: techsupport@mlabsys.com

  covers:
  -------
  	PX64xxQ rev.1A

  Description:
  ------------
	This file contains definitions for TORNADO-PX64xxQ DSP environment and
	must be included in user DSP C-code.


  Revision history:
  -----------------
	rev.1A	- 2002, initial pre-release

	rev.1A-1- 2003, final release, minor changes



  Notes:
  ------
	1. 	This C-header file is an include file for TI C6x C-compiler tools, which
		must be invoked to compile for C64xx DSP platform. MLS' C6X_DEF.H header
		file with general C6x DSP definitions must also be available for inclusion
		into user C-code along with this file.

	2. This file can be read without confusions with TAB setting equal to 4.

	3.	This header file is externally controlled from user C-code by run-time
		compiler keys definitions as the following:
			__TPX64XXQ_EXCLUDE_LOCAL_FUNCTIONS__
				- if defined in the user C-code prior inclusion of this header file,
				  then the DSP environment initialization, SIOX reset control, and
				  SIOX TM/XIO control function bodies are excluded from the user
				  source C-code. In case user application comprises of several .C linked
				  together modules, then this run-time compiler key must be included in
				  all modules except for the one module in order to exclude multiple
				  function body appearance. Default is INCLUDED.


	4. Definitions and functions in this file have been created using the following
	   TP64xxQ on-board DSP node enumaration and DSP-to-DSP communication structure via McBSP
	   serial link ports and mutual interrupt requests (DIAGONAL McBSP serial links
	   and IRQs are not shown due to impossibility to draw them in the text file):

		 				 -------   (horiz Link/IRQ)	 -------
						| DSP-A | <=============>	| DSP-B |
						 ------- 					-------
	    			   		/\    	 				   /\
	    (vertical Link/IRQ)	|    					   | (vertical Link/IRQ)
		    	   		   \/		   	   		      \/
	 	       		 	-------   (horiz Link/IRQ)	-------
					   | DSP-C | <=============>   | DSP-D |
				 	    -------	 	 			   -------

			DSP-to-DSP interrupt generation (always enabled):
			~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	 			Horizontal DSP-to-DSP IRQs are generated as the following:
		 			DSP-A <--> DSP-B, DSP-C <--> DSP-D

	 			Vertical DSP-to-DSP IRQs are generated as the following:
	 				DSP-A <--> DSP-C, DSP-B <--> DSP-D

	 			Diagonal DSP-to-DSP IRQs (not shown at the diagram) are generated as the following:
		 			DSP-A <--> DSP-D, DSP-B <--> DSP-C


			DSP-to-DSP communication via McBSP serial link ports (partially enaled by host):
			~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	 			Horizontal DSP-to-DSP McBSP serial links are always enabled and are performed
				via McBSP-2 ports as the following:
		 			DSP-A <--> DSP-B, DSP-C <--> DSP-D

	 			Vertical DSP-to-DSP McBSP serial links are enabled via corresponding bits of
				HTPX64XXQ_LINK_CNF_RG register and are performed via McBSP-0 ports as the following:
		 			DSP-A <--> DSP-C (enabled via bit DSPAC_LINK_EN of HTPX64XXQ_LINK_CNF_RG register)
					DSP-B <--> DSP-D (enabled via bit DSPBD_LINK_EN of HTPX64XXQ_LINK_CNF_RG register)

	 			Diagonal DSP-to-DSP McBSP serial links (not shown at the diagrma) are enabled via
				corresponding bits of HTPX64XXQ_LINK_CNF_RG register and are performed via McBSP-1
				ports as the following:
		 			DSP-A <--> DSP-D (enabled via bit DSPAD_LINK_EN of HTPX64XXQ_LINK_CNF_RG register)
					DSP-B <--> DSP-C (enabled via bit DSPBC_LINK_EN of HTPX64XXQ_LINK_CNF_RG register)



  Copyright:
  ----------
	This utility is supplied free of charge as it is without any obligation
	from MicroLAB Systems. No responsibility is assumed	for any use or misuse
	of these utilities.

***********************************************************************/

// check whether this header file has been already included
#ifndef __TPX64XXQ_DEF__
#define __TPX64XXQ_DEF__ 			1


// define C64xx DSP defs selection for C6X_DEF.H header file
// (in case C6X_DEF.H file will be used in user application)
#ifndef	__C6X_SELECT_C64XX_DSP__
#define	__C6X_SELECT_C64XX_DSP__	1
//#include	"c6x_def.h"
#endif


// general definitions
#define	ON							1
#define	OFF							0
#define	TPX64XXQ_ON					1
#define	TPX64XXQ_OFF				0


//================ General TPX64xxQ DSP defs =================================

	//----- TPX64XxQ internal DSP clock defs
#define	TPX64XXQ_DSP_CLK_FREQ_600MHZ		6.0e+8	// 600 MHz DSP clock with C64xx silicon 1.03
#define	TPX64XXQ_DSP_CLK_FREQ_720MHZ		7.2e+8	// 720 MHz DSP clock with C64xx silicon 1.1
//============================================================================



//====================== C64xx DSP Memory/IO Areas ===========================
// DSP memory and I/O areas for TPX64xxQ

	//----------- SSRAM (x32) area at @EMIF-A/CE-0
		// different shared static RAM (quad-port RAM) length defs (in x32 words)
		// available options are 0K, 32Kx32 (128KB) and 64Kx32 (256KB)
#define	TPX64XXQ_SSRAM_LEN_0K				0
#define	TPX64XXQ_SSRAM_LEN_32K				0x00020000
#define	TPX64XXQ_SSRAM_LEN_64K				0x00040000
#define	TPX64XXQ_SSRAM_LEN_MAX				TPX64XXQ_SSRAM_LEN_64K	// max available so far SSRAM length

		// SSRAM size aliases in bytes
#define	TPX64XXQ_SSRAM_LEN_0KB				TPX64XXQ_SSRAM_LEN_0K
#define	TPX64XXQ_SSRAM_LEN_128KB			TPX64XXQ_SSRAM_LEN_32K
#define	TPX64XXQ_SSRAM_LEN_256KB			TPX64XXQ_SSRAM_LEN_64K


		// SSRAM area mapping
		// Notes:
		// 		 1. Particular SSRAM length ID can be read by DSP from TPX64XXQ_XMEM_LEN_ID_RG
		//		    read-only register.
		// 		 2. Care must be taken, since TPX64XXQ_SSRAM_EADDR macro returns (TPX64XXQ_SSRAM_SADDR-1)
		// 		    value in case SSRAM is not installed!!!
#define	TPX64XXQ_SSRAM_SADDR				0x80000000	   		   		 		 				// SSRAM start address
#define	TPX64XXQ_SSRAM_EADDR				(TPX64XXQ_SSRAM_SADDR+TPX64XXQ_get_ssram_len()-1)  	// last SSRAM address
#define	TPX64XXQ_SSRAM_EADDR_MAX			(TPX64XXQ_SSRAM_SADDR+TPX64XXQ_SSRAM_LEN_MAX-1) 	// maximum SSRAM last address

      	// SSRAM addresses for direct inter-DSP interrupt/message passing via SSRAM
		// (valid for installed SSRAM only)
#define	TPX64XXQ_SSRAM_DSPA_RQ_ADDR			(TPX64XXQ_SSRAM_EADDR-3) 	 // SSRAM address for IRQ to DSP-A
#define	TPX64XXQ_SSRAM_DSPB_RQ_ADDR			(TPX64XXQ_SSRAM_EADDR-7)	 // SSRAM address for IRQ to DSP-B
#define	TPX64XXQ_SSRAM_DSPC_RQ_ADDR			(TPX64XXQ_SSRAM_EADDR-11)	 // SSRAM address for IRQ to DSP-C
#define	TPX64XXQ_SSRAM_DSPD_RQ_ADDR			(TPX64XXQ_SSRAM_EADDR-15)	 // SSRAM address for IRQ to DSP-D



	//----------- SDRAM (x32) area at @EMIF-A/CE-2
		// different SDRAM length per DSP defs (in x32 words)
		// available options are 0M, 4Mx32 (16MB) and 16Mx32 (64MB)
#define	TPX64XXQ_SDRAM_LEN_0M				0
#define	TPX64XXQ_SDRAM_LEN_4M				0x01000000
#define	TPX64XXQ_SDRAM_LEN_16M				0x04000000
#define	TPX64XXQ_SDRAM_LEN_MAX				TPX64XXQ_SDRAM_LEN_16M    	// max available so far SDRAM length

		// SDRAM size aliases (in bytes)
#define	TPX64XXQ_SDRAM_LEN_0MB				TPX64XXQ_SDRAM_LEN_0M
#define	TPX64XXQ_SDRAM_LEN_16MB				TPX64XXQ_SDRAM_LEN_4M
#define	TPX64XXQ_SDRAM_LEN_64MB				TPX64XXQ_SDRAM_LEN_16M

		// SDRAM area mapping
		// Notes:
		// 		 1. Particular SDRAM length ID can be read by DSP from TPX64XXQ_XMEM_LEN_ID_RG
		//		    read-only register.
		// 		 2. Care must be taken, since TPX64XXQ_SDRAM_EADDR macro returns (TPX64XXQ_SDRAM_SADDR-1)
		// 		    value in case SDRAM is not installed!!!
#define	TPX64XXQ_SDRAM_SADDR				0xA0000000	   		   		 		 				// SDRAM start address
#define	TPX64XXQ_SDRAM_EADDR				(TPX64XXQ_SDRAM_SADDR+TPX64XXQ_get_sdram_len()-1)   // last SDRAM address
#define	TPX64XXQ_SDRAM_EADDR_MAX			(TPX64XXQ_SDRAM_SADDR+TPX64XXQ_SDRAM_LEN_MAX-1) 	// maximum SDRAM last address



	//----------- IOX control registers sub-area at @EMIF-B/CE-0
	// External IOX control registers of TPX64xxQ are allocated as 8-bit LSB of 16-bit datawords,
	// and can be accessed as either 8-bit and 16-bit datawords.
#define TPX64XXQ_EXT_INT4_SEL_RG_ADDR		0x60000000  // r/w: NMI/EXT_INT4..7 source selectors
#define TPX64XXQ_EXT_INT5_SEL_RG_ADDR		0x60000002
#define TPX64XXQ_EXT_INT6_SEL_RG_ADDR		0x60000004
#define TPX64XXQ_EXT_INT7_SEL_RG_ADDR		0x60000006
#define TPX64XXQ_NMI_SEL_RG_ADDR			0x60000008
#define TPX64XXQ_DEV_ID_RG_ADDR				0x6000000a	// r: Device ID
#define TPX64XXQ_REV_ID_RG_ADDR				0x6000000c	// r: DSP speed ID and board revision ID
#define TPX64XXQ_SYS_STAT_RG_ADDR			0x6000000e	// r: system status info (HPI port format ID, bootmode ID, VERT/DIAG LINKs enable)
#define TPX64XXQ_NODE_ID_RG_ADDR			0x60000010	// r: DSP node ID (identifies particular DSP on-board)
#define TPX64XXQ_XMEM_LEN_ID_RG_ADDR		0x60000012	// r: on-board SSRAM/SDRAM capacity IDs
#define TPX64XXQ_BROADCAST_RQ_ADDR			0x60000018	// w: generate broadcast IRQ to horiz/vert/diag DSP neighbour nodes simultaneously (data is ignored)
#define TPX64XXQ_HORIZ_RQ_ADDR				0x6000001a	// w: generate IRQ to horizontal DSP neighbour node (data is ignored)
#define TPX64XXQ_DIAG_RQ_ADDR				0x6000001c	// w: generate IRQ to diagonal DSP neighbour node (data is ignored)
#define TPX64XXQ_VERT_RQ_ADDR				0x6000001e	// w: generate IRQ to vertical DSP neighbour node (data is ignored)

//============================================================================




//===== recommended TPX64XxQ EMIF configuration defs =========================
// Recommended TP6x EMIF configuration settings are provided for reference only
// and are not recommended for direct setting by user application.
// Instead, DSP EMIF settings are automatically hadled by TPX64XXQ_init_dsp_emif()
// extended TPX64XxQ API function (see below) upon the particular DSP speed and
// external memory installe. It is recommended that TPX64XXQ_init_dsp_emif() function
// is invoked as the first line of user C-code.

	// EMIF-A space configuration registers: areas CE-0/2 are valid, CE-1/CE-3 is reserved, EMIF-A_clk=100MHz/120MHz
#define TPX64XXQ_EMIFA_GCR_DATA				0x0001207c  // default EMIF-A GCR setting
#define TPX64XXQ_EMIFA_CE0_DATA				0x40		// configure EMIF-A CE0 as 32-bit SBSRAM for external SSRAM support
#define TPX64XXQ_EMIFA_CE0_SEC_DATA			0x00000002	// EMIF-A CE-0 secondary ext
#define TPX64XXQ_EMIFA_CE1_DATA				0xffffff03	// EMIF-A CE1 space is reserved, leave as default
#define TPX64XXQ_EMIFA_CE1_SEC_DATA			0x00000002	// EMIF-A CE-1 secondary ext (can be ignored)
#define TPX64XXQ_EMIFA_CE2_DATA				0x30		// configure EMIF-A CE2 as 32-bit SDRAM
#define TPX64XXQ_EMIFA_CE2_SEC_DATA			0x00000002	// EMIF-A CE-2 secondary ext (can be ignored)
#define TPX64XXQ_EMIFA_CE3_DATA				0xffffff03	// EMIF-A CE3 space is reserved, leave as default
#define TPX64XXQ_EMIFA_CE3_SEC_DATA			0x00000002	// EMIF-A CE-3 secondary ext (can be ignored)

		// EMIF-A SDRAM configuration registers shall be set upon the DSP clock
		// EMIF-A speed is 100MHz (10ns) for 600MHz C64xx and 120MHz (8.33ns) for 720MHz C64xx
#define TPX64XXQ_EMIFA_SDRAM16M_DATA_600MHZ			0x63115000	// 16Mx32 SDRAM: Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TPX64XXQ_EMIFA_SDRAM4M_DATA_600MHZ			0x57115000	// 4Mx32 SDRAM:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TPX64XXQ_EMIFA_SDRAM_EXT_DATA_600MHZ		0x00014d29	// SDRAM ext
#define	TPX64XXQ_EMIFA_SDRAM16M_TIMER_DATA_600MHZ	781			// REFR period = 64mS@8192rows => 7.81us
#define	TPX64XXQ_EMIFA_SDRAM4M_TIMER_DATA_600MHZ	1562		// REFR period = 64mS@4096rows => 15.62us

#define TPX64XXQ_EMIFA_SDRAM16M_DATA_720MHZ			0x63117000	// 16Mx32 SDRAM: Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TPX64XXQ_EMIFA_SDRAM4M_DATA_720MHZ			0x57117000	// 4Mx32 SDRAM:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron -8ns SDRAM
#define TPX64XXQ_EMIFA_SDRAM_EXT_DATA_720MHZ		0x00014d3b	// SDRAM ext
#define	TPX64XXQ_EMIFA_SDRAM16M_TIMER_DATA_720MHZ	937			// REFR period = 64mS@8192rows => 7.81us
#define	TPX64XXQ_EMIFA_SDRAM4M_TIMER_DATA_720MHZ	1875		// REFR period = 64mS@4096rows => 15.62us

		// 16-bit EMIF-B: area CE-0 is valid, CE-1/2/3 are reserved, EMIF-B_clk value has no matter
		// Secondary ext registers are not used for EMIF-B since no SBSRAM-type memory is used.
#define TPX64XXQ_EMIFB_GCR_DATA				0x0001207c  // default GCR setting
#define TPX64XXQ_EMIFB_CE0_DATA				0x10914211	// configure EMIF-B CE-0 as 16-bit ASYNC, stb=2, setup=1,hold=1, TA=1
#define TPX64XXQ_EMIFB_CE1_DATA				0xffffff03	// reserved, leave as default
#define TPX64XXQ_EMIFB_CE2_DATA				0xffffff03	// reserved, leave as default
#define TPX64XXQ_EMIFB_CE3_DATA				0xffffff03	// reserved, leave as default
//================================================================



//============== IOX registers bits defs =========================
	// IOX data register bitmask
#define	TPX64XXQ_IOX_DATA_BITMASK				0xf


	//~~~ DSP Node ID register (r)
	//	  (contains unique node ID for each of on-board DSP in multi-DSP architecture)
#define	TPX64XXQ_NODE_ID_BITMASK				0x3

#define	TPX64XXQ_NODE_ID_DSPA					0	// DSP-A node ID
#define	TPX64XXQ_NODE_ID_DSPB					1	// DSP-B node ID
#define	TPX64XXQ_NODE_ID_DSPC					2	// DSP-C node ID
#define	TPX64XXQ_NODE_ID_DSPD					3	// DSP-D node ID

	//~~~ defs, which are used by TPX64XXQ_get_node_position(tn) neighbour position macro below
	//	  to identify position (me/horiz/vert/diag) of requested target node
#define	TPX64XXQ_NODE_POSITION_MYSELF			0	// denotes, that the requested node is actually this node
#define	TPX64XXQ_NODE_POSITION_HORIZ			1	// denotes, that the requested node is the horizontal neighbour
#define	TPX64XXQ_NODE_POSITION_VERT				2	// denotes, that the requested node is the vertical neoghbour
#define	TPX64XXQ_NODE_POSITION_DIAG				3	// denotes, that the requested node is the diagonal neighbour



	//~~~ Device ID register (r)
	//	  (contains device ID data for recognition between TPX6414Q/TPX6415Q/TPX6416Q boards)
#define	TPX64XXQ_DEV_ID_BITMASK					0xf

#define	TPX6414Q_DEV_ID							0	// TPX6414Q device ID
#define	TPX6415Q_DEV_ID							1	// TPX6415Q device ID
#define	TPX6416Q_DEV_ID							2	// TPX6416Q device ID


	//~~~ Revision ID register (r)
	//	  (contains f/w revision ID data and DSP speed grade ID)
#define	TPX64XXQ_REV_ID_BITMASK					0x3
#define	TPX64XXQ_DSP_SPEED_ID_BITMASK			0xc

#define	TPX64XXQ_REV_ID_REV1					0x1	// ID for board f/w revision 1 (so far the only f/w revision)

#define	TPX64XXQ_DSP_SPEED_ID_600MHZ			0x0	// 600 MHz C64xx DSP speed grade ID
#define	TPX64XXQ_DSP_SPEED_ID_720MHZ			0x4	// 720 MHz C64xx DSP speed grade ID


	//~~~ SYS STAT register (r)
	//	  (contains HPI port data format ID (latched on release of DSP reset), DSP bootmode ID (latched on release of DSP reset),
	//     and LINK ports enable flags (can be modified by host at run-time), which are all set by host DSP via PIOX I/F)
#define	TPX64XXQ_BMODE_BITMASK					0x1
#define	TPX64XXQ_HPI_FMT_BITMASK				0x2
#define	TPX64XXQ_VERT_LINK_ENABLE_BITMASK		0x4
#define	TPX64XXQ_DIAG_LINK_ENABLE_BITMASK		0x8

#define	TPX64XXQ_BMODE_NONE						0	// DSP has started without boot
#define	TPX64XXQ_BMODE_HPI						1	// DSP has started with HPI boot

#define	TPX64XXQ_HPI_FMT_HPI16					0	// HPI is configured in 16-bit mode
#define	TPX64XXQ_HPI_FMT_HPI32					2	// HPI is configured in 32-bit mode

#define	TPX64XXQ_VERT_LINK_ENABLE_OFF			0	// serial LINK with vertical neighbour DSP node is disabled
#define	TPX64XXQ_VERT_LINK_ENABLE_ON			4	// serial LINK with vertical neighbour DSP node is enabled

#define	TPX64XXQ_DIAG_LINK_ENABLE_OFF			0	// serial LINK with diagonal neighbour DSP node is disabled
#define	TPX64XXQ_DIAG_LINK_ENABLE_ON			8	// serial LINK with diagonal neighbour DSP node is enabled

		// bitshift codes for universal _USET_/_UGET_ macros using _ON/_OFF params
#define	TPX64XXQ_VERT_LINK_ENABLE_BITSHIFT		2
#define	TPX64XXQ_DIAG_LINK_ENABLE_BITSHIFT		3


	//~~~ On-board SSRAM/SDRAM capacity ID register (r)
	//	  (contains IDs for on-board SSRAM capacity and SDRAM per DSP capacity)
#define	TPX64XXQ_SSRAM_LEN_ID_BITMASK			0x3
#define	TPX64XXQ_SDRAM_LEN_ID_BITMASK			0xc

#define	TPX64XXQ_SSRAM_LEN_ID_0K				0	// SSRAM is not installed
#define	TPX64XXQ_SSRAM_LEN_ID_32K				1	// SSRAM capacity is 32Kx32
#define	TPX64XXQ_SSRAM_LEN_ID_64K				2	// SSRAM capacity is 64Kx32
		// aliases (for length in bytes)
#define	TPX64XXQ_SSRAM_LEN_ID_0KB				TPX64XXQ_SSRAM_LEN_ID_0K
#define	TPX64XXQ_SSRAM_LEN_ID_128KB				TPX64XXQ_SSRAM_LEN_ID_32K
#define	TPX64XXQ_SSRAM_LEN_ID_256KB				TPX64XXQ_SSRAM_LEN_ID_64K

#define	TPX64XXQ_SDRAM_LEN_ID_0M				0	// SDRAM is not installed
#define	TPX64XXQ_SDRAM_LEN_ID_4M				4	// SDRAM capacity is 4Mx32 per DSP
#define	TPX64XXQ_SDRAM_LEN_ID_16M				8	// SDRAM capacity is 16Mx32 per DSP
		// aliases (for length in bytes)
#define	TPX64XXQ_SDRAM_LEN_ID_0MB				TPX64XXQ_SDRAM_LEN_ID_0M
#define	TPX64XXQ_SDRAM_LEN_ID_16MB				TPX64XXQ_SDRAM_LEN_ID_4M
#define	TPX64XXQ_SDRAM_LEN_ID_64MB				TPX64XXQ_SDRAM_LEN_ID_16M


	// EXT_INT4..7(MIRQ-0..3) and NMI source selector registers (r/w)
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// 	(contain source selector bits for corresponding DSP external interrupts)
#define	TPX64XXQ_EXT_INT_SEL_BITMASK			0x7

#define	TPX64XXQ_EXT_INT_SEL_OFF				0x0	// (*)	no IRQ source, is set as default on PIOX RESET conditions
#define	TPX64XXQ_EXT_INT_SEL_SSRAM_IRQ			0x1	// 		IRQ from the corresponding SSRAM IRQ/message location
#define	TPX64XXQ_EXT_INT_SEL_SXIRQ0				0x2	// 		IRQ-0 from MX-SIOX site
#define	TPX64XXQ_EXT_INT_SEL_SXIRQ1				0x3	// 		IRQ-1 from MX-SIOX site
#define	TPX64XXQ_EXT_INT_SEL_HORIZ_IRQ			0x4	// 		IRQ from the horizontal DSP neighbour node
#define	TPX64XXQ_EXT_INT_SEL_DIAG_IRQ			0x5	// 		IRQ from the diagonal DSP neighbour node
#define	TPX64XXQ_EXT_INT_SEL_VERT_IRQ			0x6	// 		IRQ from the vertical DSP neighbour node
#define	TPX64XXQ_EXT_INT_SEL_HOST_BROADCAST_IRQ	0x7	// 		broadcast IRQ from host

		// EXT INT IDs for universal TPX64XXQ_set/get_ext_int_sel_rg(irq,v) macros to configure
		// interrupt selector regs
#define	TPX64XXQ_EXT_INT4_SEL					0
#define	TPX64XXQ_EXT_INT5_SEL					1
#define	TPX64XXQ_EXT_INT6_SEL					2
#define	TPX64XXQ_EXT_INT7_SEL					3
#define	TPX64XXQ_NMI_SEL						4
				//aliases
#define	TPX64XXQ_EXT_INT4						TPX64XXQ_EXT_INT4_SEL
#define	TPX64XXQ_EXT_INT5						TPX64XXQ_EXT_INT5_SEL
#define	TPX64XXQ_EXT_INT6						TPX64XXQ_EXT_INT6_SEL
#define	TPX64XXQ_EXT_INT7						TPX64XXQ_EXT_INT7_SEL
#define	TPX64XXQ_NMI							TPX64XXQ_NMI_SEL

//================================================================



//============ MX-SIOX Reset and TM/XIO-IO control defs ==========
// MX-SIOX RESET is controlled via C64xx DSP GPIO-0 bit, which must be configured as the output pin.
// Output enable for MX-SIOX TM/XIO-0/1 timer I/O pins is controlled via C64xx DSP GPIO-14/15 bits,
// which shall be both configred as output pins.
// MX-SIOX RESET and TM/XIO OE are controlled via the corresponding TPX64XXQ extended API functions.

	// MS-SIOX RESET states definition for TPX64XXQ_set/get_sx_reset() extended API functions.
	// These defs are actually TPX64XXQ_ON/OFF defs for uniform I/F.
	// Note, that MX-SIOX RESET output pin is active low, so actually written values are inversed.
#define	TPX64XXQ_SX_RESET_OFF					0	// MX-SIOX RESET output is set to inactive high 'OFF' state
#define	TPX64XXQ_SX_RESET_ON					1	// MX-SIOX RESET output is set to active low 'ON' state

	// MS-SIOX TM/XIO-0/1 pins function control for TPX64XXQ_set/get_xio0/1_cnf() extended API functions.
	// Note, that TPX64xxQ extended API functions configure MX-SIOX TM/XIO-0/1 pins generally as INPUT and OUTPUT.
	// Particular pin function designation GP/IO or TIMER/IO is defined via bits FUNC and CLKSRC of
	// the corresponding C64xx DSP on-chip TIMER CONTROL register.
#define	TPX64XXQ_SX_XIO_CNF_INPUT				0	// MX-SIOX TM/XIO-0/1 pin configured as either GP/INPUT or TIMER INPUT
#define	TPX64XXQ_SX_XIO_CNF_OUTPUT				1	// MX-SIOX TM/XIO-0/1 pin configured as GP/OUTPUT or TIMER output

	// MS-SIOX TM/XIO-0/1 states definition for TPX64XXQ_set/get_sx_xio0/1_data() extended API functions.
	// These defs are actually TPX64XXQ_ON/OFF defs for uniform I/F.
#define	TPX64XXQ_SX_XIO_DATA_OFF				0	// MX-SIOX TM/XIO-0/1 pin data 'OFF' (for GP/INPUT and GP/OUTPUT configuration modes)
#define	TPX64XXQ_SX_XIO_DATA_ON					1	// MX-SIOX TM/XIO-0/1 pin data 'ON' (for GP/INPUT and GP/OUTPUT configuration modes)

	// direct C64xx GPIO-0/14/15 bit setting corresponding to MX-SIOX RESET and TM/XIO-0/1 states
	// These defs are handled by TP64XXQ extended API functions and shall not be used by user applications.
#define	TPX64XXQ_GPIO_SX_RESET_OFF				1		// state of GPIO-0 pin corresponding to MX-SIOX RESET 'OFF' state
#define	TPX64XXQ_GPIO_SX_RESET_ON				0       // state of GPIO-0 pin corresponding to MX-SIOX RESET 'ON' state

#define	TPX64XXQ_GPIO_SX_XIO0_OE_INPUT			0x4000	// state of GPIO-14 pin corresponding to MX-SIOX TM/XIO-0 pin configuraed as INPUT
#define	TPX64XXQ_GPIO_SX_XIO0_OE_OUTPUT			0		// state of GPIO-14 pin corresponding to MX-SIOX TM/XIO-0 pin configuraed as OUTPUT

#define	TPX64XXQ_GPIO_SX_XIO1_OE_INPUT			0x8000	// state of GPIO-15 pin corresponding to MX-SIOX TM/XIO-1 pin configuraed as INPUT
#define	TPX64XXQ_GPIO_SX_XIO1_OE_OUTPUT			0		// state of GPIO-15 pin corresponding to MX-SIOX TM/XIO-1 pin configuraed as OUTPUT

#define	TPX64XXQ_GPIO_SX_RESET_BITSHIFT			0
#define	TPX64XXQ_GPIO_SX_XIO0_OE_BITSHIFT		14
#define	TPX64XXQ_GPIO_SX_XIO1_OE_BITSHIFT		15

#define	TPX64XXQ_GPIO_SX_RESET_BITMASK			0x00000001
#define	TPX64XXQ_GPIO_SX_XIO0_OE_BITMASK		0x00004000
#define	TPX64XXQ_GPIO_SX_XIO1_OE_BITMASK		0x00008000

//================================================================



//================== Access macros & Functions ===================
//
//	TP64xxQ utilities provide unified utility macros/functions synopsis:
//	  	- TPX64XXQ_set_..._rg(..)    - sets the control register contents
//		- TPX64XXQ_get_..._rg()      - returns contents of control register
//  	- TPX64XXQ_set_...(..)       - sets bit of control register (using particular bit set value)
//	  	- TPX64XXQ_get_...()         - returns bit state of control register (using particular bit set value)
//  	- TPX64XXQ_uset_...(ON/OFF)  - sets bit of control register using unified ON/OFF values (may be not applicable for all bits)
//	  	- TPX64XXQ_uget_...()        - returns bit state of control register using unified ON/OFF values (may be not applicable for all bits)
//
// 	Notes:
// 	------
// 		None.

// definition of data type returned by all TP64XXQ macros
typedef		unsigned short						__TPX64XXQ_IOX_DATA_TYPE__;

// definition of data types used during generation of DSP-to-DSP IRQ via SSRAM
#ifndef BYTE
typedef		unsigned char						BYTE;
#endif


	// DSP node ID register and reference defs
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//	Returns MY NODE ID and node IDs and positions for all DSP neighbours.
#define TPX64XXQ_get_node_id_rg()				((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_NODE_ID_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)

		// dedicated bit specific macros
#define TPX64XXQ_get_my_node_id()				(TPX64XXQ_get_node_id_rg() & TPX64XXQ_NODE_ID_BITMASK)
			// aliases
#define TPX64XXQ_my_node_id()					TPX64XXQ_get_node_id()

		// conditional macros (for use in IF() and other conditional operators)
		// (a variety of board ID combinations are provided in order to detect board family)
#define TPX64XXQ_MY_NODE_ID_IS_DSPA				(TPX64XXQ_get_my_node_id()==TPX64XXQ_NODE_ID_DSPA)
#define TPX64XXQ_MY_NODE_ID_IS_DSPB				(TPX64XXQ_get_my_node_id()==TPX64XXQ_NODE_ID_DSPB)
#define TPX64XXQ_MY_NODE_ID_IS_DSPC				(TPX64XXQ_get_my_node_id()==TPX64XXQ_NODE_ID_DSPC)
#define TPX64XXQ_MY_NODE_ID_IS_DSPD				(TPX64XXQ_get_my_node_id()==TPX64XXQ_NODE_ID_DSPD)

		// macros returning node ID for the DSP neighbours (horizontal, vertical, diagonal)
#define TPX64XXQ_get_horiz_node_id()			(TPX64XXQ_MY_NODE_ID_IS_DSPA ? TPX64XXQ_NODE_ID_DSPB:\
													(TPX64XXQ_MY_NODE_ID_IS_DSPB ? TPX64XXQ_NODE_ID_DSPA:\
														(TPX64XXQ_MY_NODE_ID_IS_DSPC ? TPX64XXQ_NODE_ID_DSPD:\
															TPX64XXQ_NODE_ID_DSPC)))
#define TPX64XXQ_get_vert_node_id()				(TPX64XXQ_MY_NODE_ID_IS_DSPA ? TPX64XXQ_NODE_ID_DSPC:\
													(TPX64XXQ_MY_NODE_ID_IS_DSPB ? TPX64XXQ_NODE_ID_DSPD:\
														(TPX64XXQ_MY_NODE_ID_IS_DSPC ? TPX64XXQ_NODE_ID_DSPA:\
															TPX64XXQ_NODE_ID_DSPB)))
#define TPX64XXQ_get_diag_node_id()				(TPX64XXQ_MY_NODE_ID_IS_DSPA ? TPX64XXQ_NODE_ID_DSPD:\
													(TPX64XXQ_MY_NODE_ID_IS_DSPB ? TPX64XXQ_NODE_ID_DSPC:\
														(TPX64XXQ_MY_NODE_ID_IS_DSPC ? TPX64XXQ_NODE_ID_DSPB:\
															TPX64XXQ_NODE_ID_DSPA)))
			// aliases
#define TPX64XXQ_horiz_node_id()				TPX64XXQ_get_horiz_node_id()
#define TPX64XXQ_vert_node_id()					TPX64XXQ_get_vert_node_id()
#define TPX64XXQ_diag_node_id()					TPX64XXQ_get_diag_node_id()


		// macros returning position of requested neighbour node ID
		// This combined macro is opposite to TPX64XXQ_get_node_id(), TPX64XXQ_get_horiz_node_id(),
		// and TPX64XXQ_get_vert_node_id(), TPX64XXQ_get_diag_node_id() macros, since it returns position
		// of requested target node re position of the current node.
		// Returned are the following defs (returned values are defined above):
		//		TPX64XXQ_NODE_POSITION_MYSELF in case requested neighbour node is actually this node
		//		TPX64XXQ_NODE_POSITION_HORIZ in case requested neighbour node is the horizontal neighbour
		//		TPX64XXQ_NODE_POSITION_VERT in case requested neighbour node is the vertical neighbour
		//		TPX64XXQ_NODE_POSITION_DIAG in case requested neighbour node is the diagonal neighbour
#define TPX64XXQ_get_node_position(n)			((TPX64XXQ_get_my_node_id()==(n&TPX64XXQ_NODE_ID_BITMASK))? TPX64XXQ_NODE_POSITION_MYSELF:\
												 	((TPX64XXQ_get_horiz_node_id()==(n&TPX64XXQ_NODE_ID_BITMASK))? TPX64XXQ_NODE_POSITION_HORIZ:\
														((TPX64XXQ_get_vert_node_id()==(n&TPX64XXQ_NODE_ID_BITMASK))? TPX64XXQ_NODE_POSITION_VERT:\
															TPX64XXQ_NODE_POSITION_DIAG)))
			// aliases
#define TPX64XXQ_node_position(n)				TPX64XXQ_get_node_position(n)

			// conditional macros (for use in IF() and other conditional operators)
#define TPX64XXQ_NODE_POSITION_IS_MYSELF(n)		(TPX64XXQ_get_node_position(n)==TPX64XXQ_NODE_POSITION_MYSELF)
#define TPX64XXQ_NODE_POSITION_IS_HORIZ(n)		(TPX64XXQ_get_node_position(n)==TPX64XXQ_NODE_POSITION_HORIZ)
#define TPX64XXQ_NODE_POSITION_IS_VERT(n)		(TPX64XXQ_get_node_position(n)==TPX64XXQ_NODE_POSITION_VERT)
#define TPX64XXQ_NODE_POSITION_IS_DIAG(n)		(TPX64XXQ_get_node_position(n)==TPX64XXQ_NODE_POSITION_DIAG)



	// device ID register
	// ~~~~~~~~~~~~~~~~~~~~
#define TPX64XXQ_get_dev_id_rg()				((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_DEV_ID_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)

		// dedicated bit specific macros
#define TPX64XXQ_get_dev_id()					(TPX64XXQ_get_dev_id_rg() & TPX64XXQ_DEV_ID_BITMASK)
			// aliases
#define TPX64XXQ_dev_id()						TPX64XXQ_get_dev_id()

		// conditional macros (for use in IF() and other conditional operators)
		// (a variety of board ID combinations are provided in order to detect board family)
#define TPX6414Q_BOARD							(TPX64XXQ_get_dev_id()==TPX6414Q_DEV_ID)
#define TPX6415Q_BOARD							(TPX64XXQ_get_dev_id()==TPX6415Q_DEV_ID)
#define TPX6416Q_BOARD							(TPX64XXQ_get_dev_id()==TPX6416Q_DEV_ID)

#define TPX6414Q_TPX6415Q_BOARD					(TPX6414Q_BOARD || TPX6415Q_BOARD)
#define TPX64XXQ_BOARD							(TPX6414Q_BOARD || TPX6415Q_BOARD || TPX6416Q_BOARD)


	// revision ID register
	// ~~~~~~~~~~~~~~~~~~~~
#define TPX64XXQ_get_rev_id_rg()				((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_REV_ID_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)

		// dedicated bit specific macros
#define TPX64XXQ_get_rev_id()					(TPX64XXQ_get_rev_id_rg() & TPX64XXQ_REV_ID_BITMASK)
#define TPX64XXQ_get_dsp_speed_id()				(TPX64XXQ_get_rev_id_rg() & TPX64XXQ_DSP_SPEED_ID_BITMASK)
			// aliases
#define TPX64XXQ_rev_id()						TPX64XXQ_get_rev_id()
#define TPX64XXQ_dsp_speed_id()					TPX64XXQ_get_dsp_speed_id()

		// condition check macros (for use in IF() and other conditional operators)
#define TPX64XXQ_REV_ID_IS_REV1					(TPX64XXQ_get_rev_id()==TPX64XXQ_REV_ID_REV1)

#define TPX64XXQ_DSP_SPEED_ID_IS_600MHZ			(TPX64XXQ_get_dsp_speed_id()==TPX64XXQ_DSP_SPEED_ID_600MHZ)
#define TPX64XXQ_DSP_SPEED_ID_IS_720MHZ			(TPX64XXQ_get_dsp_speed_id()==TPX64XXQ_DSP_SPEED_ID_720MHZ)



	// SYSTEM STATUS register
	// ~~~~~~~~~~~~~~~~~~~~~~
#define TPX64XXQ_get_sys_stat_rg()				((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_SYS_STAT_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)

		// dedicated bit macros
#define TPX64XXQ_get_bmode()					(TPX64XXQ_get_sys_stat_rg() & TPX64XXQ_BMODE_BITMASK)
#define TPX64XXQ_get_hpi_fmt()					(TPX64XXQ_get_sys_stat_rg() & TPX64XXQ_HPI_FMT_BITMASK)
#define TPX64XXQ_get_vert_link_enable()			(TPX64XXQ_get_sys_stat_rg() & TPX64XXQ_VERT_LINK_ENABLE_BITMASK)
#define TPX64XXQ_get_diag_link_enable()			(TPX64XXQ_get_sys_stat_rg() & TPX64XXQ_DIAG_LINK_ENABLE_BITMASK)
			// aliases
#define TPX64XXQ_bmode()						TPX64XXQ_get_bmode()
#define TPX64XXQ_hpi_fmt()              	    TPX64XXQ_get_hpi_fmt()
#define TPX64XXQ_vert_link_enable()				TPX64XXQ_get_vert_link_enable()
#define TPX64XXQ_diag_link_enable()         	TPX64XXQ_get_diag_link_enable()

		// unified bit macros with with V = {TPX64XXQ_ON,TPX64XXQ_OFF}
#define TPX64XXQ_uget_vert_link_enable()		(TPX64XXQ_vert_link_enable() >> TPX64XXQ_VERT_LINK_ENABLE_BITSHIFT)
#define TPX64XXQ_uget_diag_link_enable()		(TPX64XXQ_diag_link_enable() >> TPX64XXQ_DIAG_LINK_ENABLE_BITSHIFT)

		// direct bit-specific get macros
#define TPX64XXQ_BMODE_IS_NONE					(TPX64XXQ_get_bmode()==TPX64XXQ_BMODE_NONE)
#define TPX64XXQ_BMODE_IS_HPI					(TPX64XXQ_get_bmode()==TPX64XXQ_BMODE_HPI)

#define	TPX64XXQ_HPI_FMT_IS_HPI16				(TPX64XXQ_get_hpi_fmt()==TPX64XXQ_HPI_FMT_HPI16)
#define	TPX64XXQ_HPI_FMT_IS_HPI32				(TPX64XXQ_get_hpi_fmt()==TPX64XXQ_HPI_FMT_HPI32)

#define	TPX64XXQ_VERT_LINK_ENABLE_IS_OFF		(TPX64XXQ_get_vert_link_enable()==TPX64XXQ_VERT_LINK_ENABLE_OFF)
#define	TPX64XXQ_VERT_LINK_ENABLE_IS_ON			(TPX64XXQ_get_vert_link_enable()==TPX64XXQ_VERT_LINK_ENABLE_ON)

#define	TPX64XXQ_DIAG_LINK_ENABLE_IS_OFF		(TPX64XXQ_get_diag_link_enable()==TPX64XXQ_DIAG_LINK_ENABLE_OFF)
#define	TPX64XXQ_DIAG_LINK_ENABLE_IS_ON			(TPX64XXQ_get_diag_link_enable()==TPX64XXQ_DIAG_LINK_ENABLE_ON)



	// DSP external memory (SSRAM/SDRAM) length ID register
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#define TPX64XXQ_get_xmem_len_id_rg()			((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_XMEM_LEN_ID_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)

		// dedicated bit specific macros
#define TPX64XXQ_get_ssram_len_id()				(TPX64XXQ_get_xmem_len_id_rg() & TPX64XXQ_SSRAM_LEN_ID_BITMASK)
#define TPX64XXQ_get_sdram_len_id()				(TPX64XXQ_get_xmem_len_id_rg() & TPX64XXQ_SDRAM_LEN_ID_BITMASK)
			// aliases
#define TPX64XXQ_ssram_len_id()					TPX64XXQ_get_ssram_len_id()
#define TPX64XXQ_sdram_len_id()					TPX64XXQ_get_sdram_len_id()

		// condition check macros (for use in IF() and other conditional operators)
#define TPX64XXQ_SSRAM_LEN_IS_0K				(TPX64XXQ_get_ssram_len_id()==TPX64XXQ_SSRAM_LEN_ID_0K)
#define TPX64XXQ_SSRAM_LEN_IS_32K				(TPX64XXQ_get_ssram_len_id()==TPX64XXQ_SSRAM_LEN_ID_32K)
#define TPX64XXQ_SSRAM_LEN_IS_64K				(TPX64XXQ_get_ssram_len_id()==TPX64XXQ_SSRAM_LEN_ID_64K)
#define TPX64XXQ_SSRAM_IS_INSTALLED				(TPX64XXQ_SSRAM_LEN_IS_32K || TPX64XXQ_SSRAM_LEN_IS_64K)
#define TPX64XXQ_SSRAM_IS_NOT_INSTALLED			!TPX64XXQ_SSRAM_IS_INSTALLED

#define TPX64XXQ_SDRAM_LEN_IS_0M				(TPX64XXQ_get_sdram_len_id()==TPX64XXQ_SDRAM_LEN_ID_0M)
#define TPX64XXQ_SDRAM_LEN_IS_4M				(TPX64XXQ_get_sdram_len_id()==TPX64XXQ_SDRAM_LEN_ID_4M)
#define TPX64XXQ_SDRAM_LEN_IS_16M				(TPX64XXQ_get_sdram_len_id()==TPX64XXQ_SDRAM_LEN_ID_16M)
#define TPX64XXQ_SDRAM_IS_INSTALLED				(TPX64XXQ_SDRAM_LEN_IS_4M || TPX64XXQ_SDRAM_LEN_IS_16M)
#define TPX64XXQ_SDRAM_IS_NOT_INSTALLED			!TPX64XXQ_SDRAM_IS_INSTALLED

			// aliases for length in bytes
#define TPX64XXQ_SSRAM_LEN_IS_0KB				TPX64XXQ_SSRAM_LEN_IS_0K
#define TPX64XXQ_SSRAM_LEN_IS_128KB				TPX64XXQ_SSRAM_LEN_IS_32K
#define TPX64XXQ_SSRAM_LEN_IS_256KB				TPX64XXQ_SSRAM_LEN_IS_64K

#define TPX64XXQ_SDRAM_LEN_IS_0MB				TPX64XXQ_SDRAM_LEN_IS_0M
#define TPX64XXQ_SDRAM_LEN_IS_16MB				TPX64XXQ_SDRAM_LEN_IS_4M
#define TPX64XXQ_SDRAM_LEN_IS_64MB				TPX64XXQ_SDRAM_LEN_IS_16M

		// macros returning true SSRAM/SDRAM length (in bytes)
#define TPX64XXQ_get_ssram_len()     			(TPX64XXQ_SSRAM_LEN_IS_0K ? TPX64XXQ_SSRAM_LEN_0K:\
													(TPX64XXQ_SSRAM_LEN_IS_32K ? TPX64XXQ_SSRAM_LEN_32K:\
														TPX64XXQ_SSRAM_LEN_64K ))

#define TPX64XXQ_get_sdram_len()     			(TPX64XXQ_SDRAM_LEN_IS_0M ? TPX64XXQ_SDRAM_LEN_0M:\
													(TPX64XXQ_SDRAM_LEN_IS_4M ? TPX64XXQ_SDRAM_LEN_4M:\
														TPX64XXQ_SDRAM_LEN_16M ))
			//aliases
#define TPX64XXQ_ssram_len()					TPX64XXQ_get_ssram_len()
#define TPX64XXQ_sdram_len()					TPX64XXQ_get_sdram_len()

	// DSP-to-DSP interrupt request generation via dedicated IOX registers
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//	  IOX registers support generation of direct request to HORIZONTAL, VERTICAL and DIAGONAL neighbour DSP nodes.
	//    No data is transferred to the requested DSP in case request is generated via dedicated IOX registers.
	//	  DSP broadcast request generates request to all HORIZONTAL, VERTICAL and DIAGONAL neighbour DSP nodes simultaneously.
#define TPX64XXQ_set_horiz_rq()					(*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_HORIZ_RQ_ADDR)=0
#define TPX64XXQ_set_vert_rq()					(*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_VERT_RQ_ADDR)=0
#define TPX64XXQ_set_diag_rq()					(*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_DIAG_RQ_ADDR)=0
#define TPX64XXQ_set_broadcast_rq()				(*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_BROADCAST_RQ_ADDR)=0
		// aliases
#define	TPX64XXQ_SET_HORIZ_RQ					TPX64XXQ_set_horiz_rq()
#define	TPX64XXQ_SET_VERT_RQ					TPX64XXQ_set_vert_rq()
#define	TPX64XXQ_SET_DIAG_RQ					TPX64XXQ_set_diag_rq()
#define TPX64XXQ_SET_BROADCAST_RQ				TPX64XXQ_set_broadcast_rq()

		// macros, which generates request via IOX registers to the DSP with particular node ID.
		// Requested DSP node is used to identify target NODE POSITION, which is used to generate
		// request via IOX register. If requested node ID is MYSELF, then no request is generated
		// (this actually can be interpreted as an error condition).
#define TPX64XXQ_set_rq_to_node(n)				if(TPX64XXQ_NODE_POSITION_IS_MYSELF(n)) {}\
													else if(TPX64XXQ_NODE_POSITION_IS_HORIZ(n)) TPX64XXQ_set_horiz_rq();\
														else if(TPX64XXQ_NODE_POSITION_IS_VERT(n)) TPX64XXQ_set_vert_rq();\
															else TPX64XXQ_set_vert_rq()


	// DSP-to-DSP interrupt request generation via dedicated SSRAM memory locations
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//		Note: These macros can be used only in case SSRAM is installed!!!
	//
	//    DSP-to-DSP interrupt requests can be generated via four dedicated SSRAM locations and allow
	//	  to transfer 8/16/32-bit data token to the requested DSP node. These four SSRAM locations are
	//	  allocated at the most top of SSRAM memory map (see SSRAM address map above in this file).
	//	  IRQ to the requested node is generated in case any number of DSP nodes (except for the requested
	//	  DSP node) write to any byte of dedicated corresponding SSRAM location, i.e. IRQ is generated
	//	  as the logical OR from all requesting DSP neighbour nodes. Generated request is removed, when the
	//	  requested DSP node reads from any byte of its corresponding dedicated SSRAM location.
	//
	//	  Due to the true quad-port RAM nature of SSRAM memory, and no access arbitration used, the transferred
	//	  request token code written by one DSP node can be overwritten by another node, which is writing to
	//	  the same SSRAM location a bit later. In order to keep integrity of transferred tokens, the solution
	//	  can be to use s/w shared semaphores to guarantee non-conflicting access to specific SSRAM locations,
	//	  however this requires extra DSP code and will reduce DSP performance.
	//
	//    Strongly recommended is the simple and feasible solution used in this PX64xxQ DSP utility,
	//	  which reduces DSP-to-DSP transferred token to 8-bits by means of assigning dedicated bytes for
	//	  each of 32-bit dedicated SSRAM IRQ generating locations as the tokens transferred from particular
	//	  DSP nodes. This excludes requirement for s/w semaphores while preserving transferred token data integrity.
	//	  The following is the byte map of 32-bit SSRAM datawords corresponding to generation of DSP-to-DSP IRQs
	//	  via SSRAM:
	//		SSRAM_address = TPX64XXQ_SSRAM_DSPA_RQ_ADDR (generation of IRQ to DSP #A)
	//				byte #0: <unused>		byte #1: DSPB->DSPA		byte #2: DSPC->DSPA		byte #3: DSPD->DSPA
	//
	//		SSRAM_address = TPX64XXQ_SSRAM_DSPB_RQ_ADDR (generation of IRQ to DSP #B)
	//				byte #0: DSPA->DSPB		byte #1: <unused>		byte #2: DSPC->DSPB		byte #3: DSPD->DSPB
	//
	//		SSRAM_address = TPX64XXQ_SSRAM_DSPC_RQ_ADDR (generation of IRQ to DSP #C)
	//				byte #0: DSPA->DSPC		byte #1: DSPB->DSPC		byte #2: <unused>		byte #3: DSPD->DSPC
	//
	//		SSRAM_address = TPX64XXQ_SSRAM_DSPD_RQ_ADDR (generation of IRQ to DSP #D)
	//				byte #0: DSPA->DSPD		byte #1: DSPB->DSPD		byte #2: DSPC->DSPD		byte #3: <unused>
	//
	//	  In accordance with the above scheme, this PX64xxQ utility returns BYTE pointers for access to
	//	  dedicated SSRAM bytes, which are used for generation of DSP-to-DSP IRQ using either destination
	//	  DSP node ID or node POSITION from the requesting DSP node. Also provided are similar macros for
	//	  generation of DSP-to-DSP IRQs via dedicated SSRAM bytes, however they deliver execution delays due
	//	  to conditional operators used. Recommended is that the user application requests BYTE pointers and
	//	  save them for further usage.

		// macros returning BYTE pointer for token write and IRQ generation to specified DSP node ID (TPX64XXQ_NODE_ID_DSPA, etc).
		// Returned BYTE pointer is defined by the requesting DSP node ID and requestor DSP node ID.
			// This macro returns valid pointer even in case requested node ID is equal to the requestor node ID.
			// However if requestor writes to such byte, then this has no effect and no IRQ is generated.
#define TPX64XXQ_get_w8_ssram_rq_to_node_ptr(n)		((BYTE*)(TPX64XXQ_SSRAM_DSPA_RQ_ADDR - ((n&TPX64XXQ_NODE_ID_BITMASK)*4) + TPX64XXQ_get_my_node_id()))
			// This macro returns NULL pointer in case requested node ID is equal to the requestor node ID.
			// At code run-time, this will result in missing write operation at NULL pointer.
//#define TPX64XXQ_get_w8_ssram_rq_to_node_ptr(n)		((BYTE*)((TPX64XXQ_get_my_node_id()==(n&TPX64XXQ_NODE_ID_BITMASK))? NULL:(TPX64XXQ_SSRAM_DSPA_RQ_ADDR - ((n&TPX64XXQ_NODE_ID_BITMASK)*4) + TPX64XXQ_get_my_node_id())))
			// aliases
#define TPX64XXQ_w8_ssram_rq_to_node_ptr(n)			TPX64XXQ_get_w8_ssram_rq_to_node_ptr(n)
			// direct defs for all node IDs
#define TPX64XXQ_W8_SSRAM_RQ_TO_DSPA_PTR			((BYTE*)(TPX64XXQ_SSRAM_DSPA_RQ_ADDR + TPX64XXQ_get_my_node_id()))
#define TPX64XXQ_W8_SSRAM_RQ_TO_DSPB_PTR			((BYTE*)(TPX64XXQ_SSRAM_DSPB_RQ_ADDR + TPX64XXQ_get_my_node_id()))
#define TPX64XXQ_W8_SSRAM_RQ_TO_DSPC_PTR			((BYTE*)(TPX64XXQ_SSRAM_DSPC_RQ_ADDR + TPX64XXQ_get_my_node_id()))
#define TPX64XXQ_W8_SSRAM_RQ_TO_DSPD_PTR			((BYTE*)(TPX64XXQ_SSRAM_DSPD_RQ_ADDR + TPX64XXQ_get_my_node_id()))

		// macros returning BYTE pointer for token write and IRQ generation to specified DSP node
		// using node releative position (HORIZ, VERT, DIAG).
		// Returned BYTE pointer is defined by the requesting DSP node ID and requestor DSP node ID.
#define TPX64XXQ_get_w8_ssram_rq_to_horiz_ptr()		TPX64XXQ_get_w8_ssram_rq_to_node_ptr(TPX64XXQ_get_horiz_node_id())
#define TPX64XXQ_get_w8_ssram_rq_to_vert_ptr()		TPX64XXQ_get_w8_ssram_rq_to_node_ptr(TPX64XXQ_get_vert_node_id())
#define TPX64XXQ_get_w8_ssram_rq_to_diag_ptr()		TPX64XXQ_get_w8_ssram_rq_to_node_ptr(TPX64XXQ_get_diag_node_id())
			// aliases
#define TPX64XXQ_w8_ssram_rq_to_horiz_ptr()			TPX64XXQ_get_w8_ssram_rq_to_horiz_ptr()
#define TPX64XXQ_w8_ssram_rq_to_vert_ptr()			TPX64XXQ_get_w8_ssram_rq_to_vert_ptr()
#define TPX64XXQ_w8_ssram_rq_to_diag_ptr()			TPX64XXQ_get_w8_ssram_rq_to_diag_ptr()

#define TPX64XXQ_W8_SSRAM_RQ_TO_HORIZ_PTR			TPX64XXQ_get_w8_ssram_rq_to_horiz_ptr()
#define TPX64XXQ_W8_SSRAM_RQ_TO_VERT_PTR			TPX64XXQ_get_w8_ssram_rq_to_vert_ptr()
#define TPX64XXQ_W8_SSRAM_RQ_TO_DIAG_PTR			TPX64XXQ_get_w8_ssram_rq_to_diag_ptr()


		// macros writing at BYTE pointers for generation of DSP-to-DSP IRQ via SSRAM to the target node
		// Note, that these complex macros degradate system performance. Use pointers instead for maximum performance.
#define TPX64XXQ_set_w8_ssram_rq_to_node(n,v)		(*TPX64XXQ_get_w8_ssram_rq_to_node_ptr(n))=(BYTE)v
			// direct defs
#define TPX64XXQ_SET_W8_SSRAM_RQ_TO_DSPA(v)			(*TPX64XXQ_W8_SSRAM_RQ_TO_DSPA_PTR)=((BYTE)v)
#define TPX64XXQ_SET_W8_SSRAM_RQ_TO_DSPB(v)         (*TPX64XXQ_W8_SSRAM_RQ_TO_DSPB_PTR)=((BYTE)v)
#define TPX64XXQ_SET_W8_SSRAM_RQ_TO_DSPC(v)			(*TPX64XXQ_W8_SSRAM_RQ_TO_DSPC_PTR)=((BYTE)v)
#define TPX64XXQ_SET_W8_SSRAM_RQ_TO_DSPD(v)			(*TPX64XXQ_W8_SSRAM_RQ_TO_DSPD_PTR)=((BYTE)v)

			// macros writing at BYTE pointer for target IRQ generation to specified DSP node using
			// node relative position
#define TPX64XXQ_set_w8_ssram_rq_to_horiz(v)		(*TPX64XXQ_get_w8_ssram_rq_to_horiz_ptr())=((BYTE)v)
#define TPX64XXQ_set_w8_ssram_rq_to_vert(v)			(*TPX64XXQ_get_w8_ssram_rq_to_vert_ptr())=((BYTE)v)
#define TPX64XXQ_set_w8_ssram_rq_to_diag(v)			(*TPX64XXQ_get_w8_ssram_rq_to_diag_ptr())=((BYTE)v)

#define TPX64XXQ_SET_W8_SSRAM_RQ_TO_HORIZ(v)		TPX64XXQ_set_w8_ssram_rq_to_horiz(v)
#define TPX64XXQ_SET_W8_SSRAM_RQ_TO_VERT(v)			TPX64XXQ_set_w8_ssram_rq_to_vert(v)
#define TPX64XXQ_SET_W8_SSRAM_RQ_TO_DIAG(v)			TPX64XXQ_set_w8_ssram_rq_to_diag(v)


		// macros returning BYTE pointer for token read and IRQ reset from specified DSP node ID (TPX64XXQ_NODE_ID_DSPA, etc).
			// This macro returns valid pointer even in case requested node ID is equal to the requestor node ID.
			// However if requestor reads from such byte, then this IRQ is reset.
#define TPX64XXQ_get_w8_ssram_rq_from_node_ptr(n)	((BYTE*)(TPX64XXQ_SSRAM_DSPA_RQ_ADDR + (n&TPX64XXQ_NODE_ID_BITMASK) - (TPX64XXQ_get_my_node_id()*4)))
			// This macro returns NULL pointer in case requested node ID is equal to the requestor node ID.
			// At code run-time, this will result in missing read operation at NULL pointer and IRQ is not reset.
//#define TPX64XXQ_get_w8_ssram_rq_from_node_ptr(n)		((BYTE*)((TPX64XXQ_get_my_node_id()==(n&TPX64XXQ_NODE_ID_BITMASK))? NULL:(TPX64XXQ_SSRAM_DSPA_RQ_ADDR + (n&TPX64XXQ_NODE_ID_BITMASK) - (TPX64XXQ_get_my_node_id()*4))))
			// alias
#define TPX64XXQ_w8_ssram_rq_from_node_w8ptr(n)		TPX64XXQ_get_w8_ssram_rq_from_node_ptr(n)

			// direct defs for all node IDs
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DSPA_PTR			TPX64XXQ_get_w8_ssram_rq_from_node_ptr(TPX64XXQ_NODE_ID_DSPA)
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DSPB_PTR			TPX64XXQ_get_w8_ssram_rq_from_node_ptr(TPX64XXQ_NODE_ID_DSPB)
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DSPC_PTR			TPX64XXQ_get_w8_ssram_rq_from_node_ptr(TPX64XXQ_NODE_ID_DSPC)
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DSPD_PTR			TPX64XXQ_get_w8_ssram_rq_from_node_ptr(TPX64XXQ_NODE_ID_DSPD)


		// macros returning BYTE pointer for token read and IRQ reset from specified DSP node
		// using node releative position (HORIZ, VERT, DIAG).
#define TPX64XXQ_get_w8_ssram_rq_from_horiz_ptr()	TPX64XXQ_get_w8_ssram_rq_from_node_ptr(TPX64XXQ_get_horiz_node_id())
#define TPX64XXQ_get_w8_ssram_rq_from_vert_ptr()	TPX64XXQ_get_w8_ssram_rq_from_node_ptr(TPX64XXQ_get_vert_node_id())
#define TPX64XXQ_get_w8_ssram_rq_from_diag_ptr()	TPX64XXQ_get_w8_ssram_rq_from_node_ptr(TPX64XXQ_get_diag_node_id())
			// aliases
#define TPX64XXQ_w8_ssram_rq_from_horiz_ptr()		TPX64XXQ_get_w8_ssram_rq_from_horiz_ptr()
#define TPX64XXQ_w8_ssram_rq_from_vert_ptr()		TPX64XXQ_get_w8_ssram_rq_from_vert_ptr()
#define TPX64XXQ_w8_ssram_rq_from_diag_ptr()		TPX64XXQ_get_w8_ssram_rq_from_diag_ptr()

#define TPX64XXQ_W8_SSRAM_RQ_FROM_HORIZ_PTR			TPX64XXQ_get_w8_ssram_rq_from_horiz_ptr()
#define TPX64XXQ_W8_SSRAM_RQ_FROM_VERT_PTR			TPX64XXQ_get_w8_ssram_rq_from_vert_ptr()
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DIAG_PTR			TPX64XXQ_get_w8_ssram_rq_from_diag_ptr()

		// macros for reading at BYTE pointer from SSRAM and reset of DSP-to-DSP IRQ via SSRAM from the target node
		// Note, that these complex macros degradate system performance. Use pointers instead for maximum performance.
#define TPX64XXQ_get_w8_ssram_rq_from_node(n)		(*TPX64XXQ_get_w8_ssram_rq_from_node_ptr(n))
			// aliases
#define TPX64XXQ_w8_ssram_rq_from_node(n)			TPX64XXQ_get_w8_ssram_rq_from_node(n)
			// direct defs
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DSPA				(*TPX64XXQ_get_w8_ssram_rq_from_node(TPX64XXQ_NODE_ID_DSPA))
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DSPB				(*TPX64XXQ_get_w8_ssram_rq_from_node(TPX64XXQ_NODE_ID_DSPB))
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DSPC				(*TPX64XXQ_get_w8_ssram_rq_from_node(TPX64XXQ_NODE_ID_DSPC))
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DSPD				(*TPX64XXQ_get_w8_ssram_rq_from_node(TPX64XXQ_NODE_ID_DSPD))

			// macros writing at BYTE pointer for target IRQ generation to specified DSP node using
			// node relative position
#define TPX64XXQ_get_w8_ssram_rq_from_horiz()		(*TPX64XXQ_get_w8_ssram_rq_from_horiz_ptr())
#define TPX64XXQ_get_w8_ssram_rq_from_vert()		(*TPX64XXQ_get_w8_ssram_rq_from_vert_ptr())
#define TPX64XXQ_get_w8_ssram_rq_from_diag()		(*TPX64XXQ_get_w8_ssram_rq_from_diag_ptr())
				// aliases
#define TPX64XXQ_w8_ssram_rq_from_horiz()			TPX64XXQ_get_w8_ssram_rq_from_horiz()
#define TPX64XXQ_w8_ssram_rq_from_vert()			TPX64XXQ_get_w8_ssram_rq_from_vert()
#define TPX64XXQ_w8_ssram_rq_from_diag()			TPX64XXQ_get_w8_ssram_rq_from_diag()
				// direct defs
#define TPX64XXQ_GET_W8_SSRAM_RQ_FROM_HORIZ			TPX64XXQ_get_w8_ssram_rq_from_horiz()
#define TPX64XXQ_GET_W8_SSRAM_RQ_FROM_VERT          TPX64XXQ_get_w8_ssram_rq_from_vert()
#define TPX64XXQ_GET_W8_SSRAM_RQ_FROM_DIAG          TPX64XXQ_get_w8_ssram_rq_from_diag()

#define TPX64XXQ_W8_SSRAM_RQ_FROM_HORIZ				TPX64XXQ_get_w8_ssram_rq_from_horiz()
#define TPX64XXQ_W8_SSRAM_RQ_FROM_VERT          	TPX64XXQ_get_w8_ssram_rq_from_vert()
#define TPX64XXQ_W8_SSRAM_RQ_FROM_DIAG          	TPX64XXQ_get_w8_ssram_rq_from_diag()


		// macros returning pointers to 32-bit words for token read/IRQ_reset and write/IRQ_set from/to specified DSP node ID (TPX64XXQ_NODE_ID_DSPA, etc).
		// Note, that write to SSRAM at the returned pointer will write full 32-bit word and can result in written
		// data conflict when other nodes are writing at the same pointer.
#define TPX64XXQ_get_w32_ssram_rq_to_node_ptr(n)	((unsigned*)(TPX64XXQ_SSRAM_DSPA_RQ_ADDR - ((n&TPX64XXQ_NODE_ID_BITMASK)*4)))
#define TPX64XXQ_get_w32_ssram_rq_to_me_ptr()		TPX64XXQ_get_w32_ssram_rq_to_node_ptr(TPX64XXQ_get_my_node_id())
			// alias
#define TPX64XXQ_w32_ssram_rq_to_node_ptr(n)		TPX64XXQ_get_w32_ssram_rq_to_node_ptr(n)
#define TPX64XXQ_w32_ssram_rq_to_me_ptr()			TPX64XXQ_get_w32_ssram_rq_to_me_ptr()

			// direct defs
#define TPX64XXQ_W32_SSRAM_RQ_TO_DSPA_PTR			((unsigned*)TPX64XXQ_SSRAM_DSPA_RQ_ADDR)
#define TPX64XXQ_W32_SSRAM_RQ_TO_DSPB_PTR           ((unsigned*)TPX64XXQ_SSRAM_DSPB_RQ_ADDR)
#define TPX64XXQ_W32_SSRAM_RQ_TO_DSPC_PTR			((unsigned*)TPX64XXQ_SSRAM_DSPC_RQ_ADDR)
#define TPX64XXQ_W32_SSRAM_RQ_TO_DSPD_PTR			((unsigned*)TPX64XXQ_SSRAM_DSPD_RQ_ADDR)

#define TPX64XXQ_W32_SSRAM_RQ_TO_ME_PTR				TPX64XXQ_w32_get_ssram_rq_to_me_ptr()

			// macros writing at W32 pointer for target IRQ generation to specified DSP node using node ID
#define TPX64XXQ_set_w32_ssram_rq_to_node(n,v)		(*TPX64XXQ_get_w32_ssram_rq_to_node_ptr(n))=(unsigned)v
#define TPX64XXQ_get_w32_ssram_rq_to_node(n)		(*TPX64XXQ_get_w32_ssram_rq_to_node_ptr(n))
				// direct defs
#define TPX64XXQ_SET_W32_SSRAM_RQ_TO_DSPA(v)		(*TPX64XXQ_W32_SSRAM_RQ_TO_DSPA_PTR)=v
#define TPX64XXQ_SET_W32_SSRAM_RQ_TO_DSPB(v)		(*TPX64XXQ_W32_SSRAM_RQ_TO_DSPB_PTR)=v
#define TPX64XXQ_SET_W32_SSRAM_RQ_TO_DSPC(v)		(*TPX64XXQ_W32_SSRAM_RQ_TO_DSPC_PTR)=v
#define TPX64XXQ_SET_W32_SSRAM_RQ_TO_DSPD(v)		(*TPX64XXQ_W32_SSRAM_RQ_TO_DSPD_PTR)=v

#define TPX64XXQ_GET_W32_SSRAM_RQ_TO_DSPA			(*TPX64XXQ_W32_SSRAM_RQ_TO_DSPA_PTR)
#define TPX64XXQ_GET_W32_SSRAM_RQ_TO_DSPB			(*TPX64XXQ_W32_SSRAM_RQ_TO_DSPB_PTR)
#define TPX64XXQ_GET_W32_SSRAM_RQ_TO_DSPC			(*TPX64XXQ_W32_SSRAM_RQ_TO_DSPC_PTR)
#define TPX64XXQ_GET_W32_SSRAM_RQ_TO_DSPD			(*TPX64XXQ_W32_SSRAM_RQ_TO_DSPD_PTR)
					// aliases
#define TPX64XXQ_W32_SSRAM_RQ_TO_DSPA				TPX64XXQ_GET_W32_SSRAM_RQ_TO_DSPA
#define TPX64XXQ_W32_SSRAM_RQ_TO_DSPB				TPX64XXQ_GET_W32_SSRAM_RQ_TO_DSPB
#define TPX64XXQ_W32_SSRAM_RQ_TO_DSPC				TPX64XXQ_GET_W32_SSRAM_RQ_TO_DSPC
#define TPX64XXQ_W32_SSRAM_RQ_TO_DSPD				TPX64XXQ_GET_W32_SSRAM_RQ_TO_DSPD

			// macros writing/reading at W32 pointer for target IRQ generation to specified DSP node using node relative position
#define TPX64XXQ_set_w32_ssram_rq_to_horiz(v)		(*TPX64XXQ_get_w32_ssram_rq_to_node_ptr(TPX64XXQ_get_horiz_node_id()))=v
#define TPX64XXQ_set_w32_ssram_rq_to_vert(v)		(*TPX64XXQ_get_w32_ssram_rq_to_node_ptr(TPX64XXQ_get_vert_node_id()))=v
#define TPX64XXQ_set_w32_ssram_rq_to_diag(v)		(*TPX64XXQ_get_w32_ssram_rq_to_node_ptr(TPX64XXQ_get_diag_node_id()))=v

#define TPX64XXQ_get_w32_ssram_rq_to_horiz()		(*TPX64XXQ_get_w32_ssram_rq_to_node_ptr(TPX64XXQ_get_horiz_node_id()))
#define TPX64XXQ_get_w32_ssram_rq_to_vert()			(*TPX64XXQ_get_w32_ssram_rq_to_node_ptr(TPX64XXQ_get_vert_node_id()))
#define TPX64XXQ_get_w32_ssram_rq_to_diag()			(*TPX64XXQ_get_w32_ssram_rq_to_node_ptr(TPX64XXQ_get_diag_node_id()))

#define TPX64XXQ_SET_W32_SSRAM_RQ_TO_HORIZ(v)		TPX64XXQ_set_w32_ssram_rq_to_horiz(v)
#define TPX64XXQ_SET_W32_SSRAM_RQ_TO_VERT(v)		TPX64XXQ_set_w32_ssram_rq_to_vert(v)
#define TPX64XXQ_SET_W32_SSRAM_RQ_TO_DIAG(v)		TPX64XXQ_set_w32_ssram_rq_to_diag(v)

#define TPX64XXQ_GET_W32_SSRAM_RQ_TO_HORIZ			TPX64XXQ_get_w32_ssram_rq_to_horiz()
#define TPX64XXQ_GET_W32_SSRAM_RQ_TO_VERT			TPX64XXQ_get_w32_ssram_rq_to_vert()
#define TPX64XXQ_GET_W32_SSRAM_RQ_TO_DIAG			TPX64XXQ_get_w32_ssram_rq_to_diag()
				// aliases
#define TPX64XXQ_W32_SSRAM_RQ_TO_HORIZ				TPX64XXQ_get_w32_ssram_rq_to_horiz()
#define TPX64XXQ_W32_SSRAM_RQ_TO_VERT				TPX64XXQ_get_w32_ssram_rq_to_vert()
#define TPX64XXQ_W32_SSRAM_RQ_TO_DIAG				TPX64XXQ_get_w32_ssram_rq_to_diag()



			// macros reading at W32 pointer for target IRQ reset
			// Note, that when reading from SSRAM at W32 pointer, all requests are reading in one cycle.
#define TPX64XXQ_set_w32_ssram_rq_to_me(v)			(*TPX64XXQ_get_w32_ssram_rq_to_me_ptr())=v
#define TPX64XXQ_get_w32_ssram_rq_to_me()			(*TPX64XXQ_get_w32_ssram_rq_to_me_ptr())
				// alias
#define TPX64XXQ_SET_W32_SSRAM_RQ_TO_ME(v)			TPX64XXQ_set_w32_ssram_rq_to_me(v)
#define TPX64XXQ_GET_W32_SSRAM_RQ_TO_ME				TPX64XXQ_get_w32_ssram_rq_to_me()
#define TPX64XXQ_W32_SSRAM_RQ_TO_ME					TPX64XXQ_get_w32_ssram_rq_to_me()



	// Macros for SSRAM access
	// ~~~~~~~~~~~~~~~~~~~~~~~~
	// Provided macros perform 32-bit data access. SSRAM address must be in bytes count.
	// For performance reasons, provided SSRAM address parameter is not checked against installed SSRAM length
	// and two LSBs are not checked for '0'. If required, user can replace 'a' parameter by
	// '((a&~0x3)%TPX64XXQ_get_ssram_len())'
#define TPX64XXQ_set_ssram_data(a,v)				(*((unsigned *)(TPX64XXQ_SSRAM_SADDR+a)))=v
#define TPX64XXQ_get_ssram_data(a)					(*((unsigned *)(TPX64XXQ_SSRAM_SADDR+a)))
		// alias
#define TPX64XXQ_ssram_data(a)						TPX64XXQ_get_ssram_data(a)




	// EXT_INT4..EXT_INT7 and NMI IRQ source selector IOX registers
	// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//    Provided macros include both individual INT selectors SET/GET macros and
	//    universal SET/GET macro, which uses INT_SEL ID to select particular EXT_INTx/NMI.

		// individual INT selectors access macros
#define TPX64XXQ_set_ext_int4_sel_rg(v)			(*(__TPX64XXQ_IOX_DATA_TYPE__ *)  TPX64XXQ_EXT_INT4_SEL_RG_ADDR)=((__TPX64XXQ_IOX_DATA_TYPE__)v)
#define TPX64XXQ_get_ext_int4_sel_rg()			((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_EXT_INT4_SEL_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)
#define TPX64XXQ_set_ext_int5_sel_rg(v)			(*(__TPX64XXQ_IOX_DATA_TYPE__ *)  TPX64XXQ_EXT_INT5_SEL_RG_ADDR)=((__TPX64XXQ_IOX_DATA_TYPE__)v)
#define TPX64XXQ_get_ext_int5_sel_rg()			((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_EXT_INT5_SEL_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)
#define TPX64XXQ_set_ext_int6_sel_rg(v)			(*(__TPX64XXQ_IOX_DATA_TYPE__ *)  TPX64XXQ_EXT_INT6_SEL_RG_ADDR)=((__TPX64XXQ_IOX_DATA_TYPE__)v)
#define TPX64XXQ_get_ext_int6_sel_rg()			((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_EXT_INT6_SEL_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)
#define TPX64XXQ_set_ext_int7_sel_rg(v)			(*(__TPX64XXQ_IOX_DATA_TYPE__ *)  TPX64XXQ_EXT_INT7_SEL_RG_ADDR)=((__TPX64XXQ_IOX_DATA_TYPE__)v)
#define TPX64XXQ_get_ext_int7_sel_rg()			((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_EXT_INT7_SEL_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)
#define TPX64XXQ_set_nmi_sel_rg(v)				(*(__TPX64XXQ_IOX_DATA_TYPE__ *)  TPX64XXQ_NMI_SEL_RG_ADDR)=((__TPX64XXQ_IOX_DATA_TYPE__)v)
#define TPX64XXQ_get_nmi_sel_rg()				((*(__TPX64XXQ_IOX_DATA_TYPE__ *) TPX64XXQ_NMI_SEL_RG_ADDR) & TPX64XXQ_IOX_DATA_BITMASK)
			// dedicated bit-specific macros
#define	TPX64XXQ_set_ext_int4_sel(v)			TPX64XXQ_set_ext_int4_sel_rg(v)
#define TPX64XXQ_get_ext_int4_sel()				TPX64XXQ_get_ext_int4_sel_rg()
#define	TPX64XXQ_set_ext_int5_sel(v)			TPX64XXQ_set_ext_int5_sel_rg(v)
#define TPX64XXQ_get_ext_int5_sel()				TPX64XXQ_get_ext_int5_sel_rg()
#define	TPX64XXQ_set_ext_int6_sel(v)			TPX64XXQ_set_ext_int6_sel_rg(v)
#define TPX64XXQ_get_ext_int6_sel()				TPX64XXQ_get_ext_int6_sel_rg()
#define	TPX64XXQ_set_ext_int7_sel(v)			TPX64XXQ_set_ext_int7_sel_rg(v)
#define TPX64XXQ_get_ext_int7_sel()				TPX64XXQ_get_ext_int7_sel_rg()
#define	TPX64XXQ_set_nmi_sel(v)					TPX64XXQ_set_nmi_sel_rg(v)
#define TPX64XXQ_get_nmi_sel()					TPX64XXQ_get_nmi_sel_rg()
			// aliases
#define TPX64XXQ_ext_int4_sel()					TPX64XXQ_get_ext_int4_sel()
#define TPX64XXQ_ext_int5_sel()					TPX64XXQ_get_ext_int5_sel()
#define TPX64XXQ_ext_int6_sel()					TPX64XXQ_get_ext_int6_sel()
#define TPX64XXQ_ext_int7_sel()					TPX64XXQ_get_ext_int7_sel()
#define TPX64XXQ_nmi_sel()						TPX64XXQ_get_nmi_sel_rg()


		// universal macros to set/get EXT_INT/EXT_NMI selector regs using EXT_INT ID
		// input params: IRQ={TPX64XXQ_EXT_INTx_SEL/TPX64XXQ_NMI_SEL}, V={TPX64XXQ_EXT_INT_SEL_OFF,etc..}
#define	TPX64XXQ_set_ext_int_sel_rg(irq,v)		if(irq==0) TPX64XXQ_set_ext_int4_sel_rg(v); \
										  	 		else if(irq==1) TPX64XXQ_set_ext_int5_sel_rg(v);\
										      			else if(irq==2) TPX64XXQ_set_ext_int6_sel_rg(v);\
										       				else if(irq==3) TPX64XXQ_set_ext_int7_sel_rg(v);\
											    				else TPX64XXQ_set_nmi_sel_rg(v)
#define	TPX64XXQ_get_ext_int_sel_rg(irq)		((irq==0)? TPX64XXQ_get_ext_int4_sel(): \
										  	 		((irq==1)? TPX64XXQ_get_ext_int5_sel():\
										      			((irq==2)? TPX64XXQ_get_ext_int6_sel():\
										       				((irq==3)? TPX64XXQ_get_ext_int7_sel():TPX64XXQ_get_nmi_sel()))))
			// aliases
#define	TPX64XXQ_set_ext_int_sel(irq,v)			TPX64XXQ_set_ext_int_sel_rg(irq,v)
#define	TPX64XXQ_get_ext_int_sel(irq)			TPX64XXQ_get_ext_int_sel_rg(irq)

			// direct universal set/get macros
#define	TPX64XXQ_SET_EXT_INT_SEL_OFF(irq)					TPX64XXQ_set_ext_int_sel(irq,TPX64XXQ_EXT_INT_SEL_OFF)
#define	TPX64XXQ_SET_EXT_INT_SEL_SSRAM_IRQ(irq)				TPX64XXQ_set_ext_int_sel(irq,TPX64XXQ_EXT_INT_SEL_SSRAM_IRQ)
#define	TPX64XXQ_SET_EXT_INT_SEL_SXIRQ0(irq)				TPX64XXQ_set_ext_int_sel(irq,TPX64XXQ_EXT_INT_SEL_SXIRQ0)
#define	TPX64XXQ_SET_EXT_INT_SEL_SXIRQ1(irq)				TPX64XXQ_set_ext_int_sel(irq,TPX64XXQ_EXT_INT_SEL_SXIRQ1)
#define	TPX64XXQ_SET_EXT_INT_SEL_HORIZ_IRQ(irq)				TPX64XXQ_set_ext_int_sel(irq,TPX64XXQ_EXT_INT_SEL_HORIZ_IRQ)
#define	TPX64XXQ_SET_EXT_INT_SEL_VERT_IRQ(irq)				TPX64XXQ_set_ext_int_sel(irq,TPX64XXQ_EXT_INT_SEL_VERT_IRQ)
#define	TPX64XXQ_SET_EXT_INT_SEL_DIAG_IRQ(irq)				TPX64XXQ_set_ext_int_sel(irq,TPX64XXQ_EXT_INT_SEL_DIAG_IRQ)
#define	TPX64XXQ_SET_EXT_INT_SEL_HOST_BROADCAST_IRQ(irq)	TPX64XXQ_set_ext_int_sel(irq,TPX64XXQ_EXT_INT_SEL_HOST_BROADCAST_IRQ)

#define	TPX64XXQ_EXT_INT_SEL_IS_OFF(irq)					(TPX64XXQ_get_ext_int_sel(irq)==TPX64XXQ_EXT_INT_SEL_OFF)
#define	TPX64XXQ_EXT_INT_SEL_IS_SSRAM_IRQ(irq)				(TPX64XXQ_get_ext_int_sel(irq)==TPX64XXQ_EXT_INT_SEL_SSRAM_IRQ)
#define	TPX64XXQ_EXT_INT_SEL_IS_SXIRQ0(irq)					(TPX64XXQ_get_ext_int_sel(irq)==TPX64XXQ_EXT_INT_SEL_SXIRQ0)
#define	TPX64XXQ_EXT_INT_SEL_IS_SXIRQ1(irq)					(TPX64XXQ_get_ext_int_sel(irq)==TPX64XXQ_EXT_INT_SEL_SXIRQ1)
#define	TPX64XXQ_EXT_INT_SEL_IS_HORIZ_IRQ(irq)				(TPX64XXQ_get_ext_int_sel(irq)==TPX64XXQ_EXT_INT_SEL_HORIZ_IRQ)
#define	TPX64XXQ_EXT_INT_SEL_IS_VERT_IRQ(irq)				(TPX64XXQ_get_ext_int_sel(irq)==TPX64XXQ_EXT_INT_SEL_VERT_IRQ)
#define	TPX64XXQ_EXT_INT_SEL_IS_DIAG_IRQ(irq)				(TPX64XXQ_get_ext_int_sel(irq)==TPX64XXQ_EXT_INT_SEL_DIAG_IRQ)
#define	TPX64XXQ_EXT_INT_SEL_IS_HOST_BROADCAST_IRQ(irq)		(TPX64XXQ_get_ext_int_sel(irq)==TPX64XXQ_EXT_INT_SEL_HOST_BROADCAST_IRQ)

//======================================================================


//====================== HPI port specific macros ======================
// C64xx DSP can perform the following actions over HPIC register bits:
//		- read DSPINT state and clear DSPINT
//		- read HINT state and set HINT

	// HPI registers addresses for local usage (in case C6X_DEF.H file is not included)
#define	__TPX64XXQ_C64XX_HPIC_ADDR						0x01880000
#define	__TPX64XXQ_C64XX_HPIAR_ADDR						0x01880008
#define	__TPX64XXQ_C64XX_HPIAW_ADDR						0x01880004

	// HPIC register bits
#define	__TPX64XXQ_C64XX_HPIC_DSPINT_BITMASK			0x2
#define	__TPX64XXQ_C64XX_HPIC_HINT_BITMASK				0x4

#define	__TPX64XXQ_C64XX_HPIC_DSPINT_ON					0x2
#define	__TPX64XXQ_C64XX_HPIC_DSPINT_OFF				0

#define	__TPX64XXQ_C64XX_HPIC_HINT_ON					0x4
#define	__TPX64XXQ_C64XX_HPIC_HINT_OFF					0

	// HPIC register access macros
#define TPX64XXQ_get_hpic_dspint()						((*(unsigned *) __TPX64XXQ_C64XX_HPIC_ADDR)&__TPX64XXQ_C64XX_HPIC_DSPINT_BITMASK)
#define TPX64XXQ_get_hpic_hint()						((*(unsigned *) __TPX64XXQ_C64XX_HPIC_ADDR)&__TPX64XXQ_C64XX_HPIC_HINT_BITMASK)
#define TPX64XXQ_clr_hpic_dspint()						(*(unsigned *) __TPX64XXQ_C64XX_HPIC_ADDR)= __TPX64XXQ_C64XX_HPIC_DSPINT_BITMASK
#define TPX64XXQ_set_hpic_hint()						(*(unsigned *) __TPX64XXQ_C64XX_HPIC_ADDR)= __TPX64XXQ_C64XX_HPIC_HINT_BITMASK
		// aliases
#define TPX64XXQ_hpic_dspint()							TPX64XXQ_get_hpic_dspint()
#define TPX64XXQ_hpic_hint()                        	TPX64XXQ_get_hpic_hint()
		// direct set/get macros
#define TPX64XXQ_CLR_HPIC_DSPINT						TPX64XXQ_clr_hpic_dspint()
#define TPX64XXQ_SET_HPIC_HINT							TPX64XXQ_set_hpic_hint()

#define TPX64XXQ_HPIC_DSPINT_IS_OFF						(TPX64XXQ_get_hpic_dspint()==__TPX64XXQ_C64XX_HPIC_DSPINT_OFF)
#define TPX64XXQ_HPIC_DSPINT_IS_ON						(TPX64XXQ_get_hpic_dspint()==__TPX64XXQ_C64XX_HPIC_DSPINT_ON)

#define TPX64XXQ_HPIC_HINT_IS_OFF						(TPX64XXQ_get_hpic_hint()==__TPX64XXQ_C64XX_HPIC_HINT_OFF)
#define TPX64XXQ_HPIC_HINT_IS_ON						(TPX64XXQ_get_hpic_hint()==__TPX64XXQ_C64XX_HPIC_HINT_ON)

//======================================================================





//================= extended TP64xxQ API functions =====================
// Extended TP64xxQ DSP API functions include functions for DSP environment INIT.
// SIOX RESET control, and SIOX TM/XIO-0/1 I/O pins control.
// In order to reduce the size of DSP code and in order to exclude mutiple
// function body appearance, any of the Extended API functions can be included/excluded
// during the compilation process. Inclusion/exclusion of particular Extended API function
// is controled via run-time compiler key, which shall be defined in the user C-code prior
// inclusion of this header file:
//
//			__TPX64XXQ_EXCLUDE_LOCAL_FUNCTIONS__
//				- if defined in the user C-code prior inclusion of this header file,
//				  then the DSP environment initialization, SIOX reset control, and
//				  SIOX TM/XIO control function bodies are excluded from the user
//				  source C-code. In case user application comprises of several .C linked
//				  together modules, then this run-time compiler key must be included in
//				  all modules except for the one module in order to exclude multiple
//				  function body appearance. Default is INCLUDED.
//

//~~~~~~~~~~~~ extended API functions declaration ~~~~~~~~~~~~~~~~~~~~~~
void		TPX64XXQ_init_dsp();

	// extended API functions and macros for MX-SIOX RESET control
	// input/returned value is either V={TPX64XXQ_SX_RESET_OFF,TPX64XXQ_SX_RESET_ON} or V={TP64XX_OFF,TP64XX_ON}
void 		TPX64XXQ_set_sx_reset(int v);
int 		TPX64XXQ_get_sx_reset();

		// aliases
#define		TPX64XXQ_sx_reset()					TPX64XXQ_get_sx_reset()
		// direct set/get macros
#define		TPX64XXQ_SET_SX_RESET_ON			TPX64XXQ_set_sx_reset(TPX64XXQ_ON)
#define		TPX64XXQ_SET_SX_RESET_OFF			TPX64XXQ_set_sx_reset(TPX64XXQ_OFF)
#define		TPX64XXQ_SX_RESET_IS_ON				(TPX64XXQ_get_sx_reset()==TPX64XXQ_ON)
#define		TPX64XXQ_SX_RESET_IS_OFF			(TPX64XXQ_get_sx_reset()==TPX64XXQ_OFF)


	// extended API functions and macros for MX-SIOX TM/XIO-0/1 pins control
void 		TPX64XXQ_set_sx_xio0_cnf(int cnf);
void 		TPX64XXQ_set_sx_xio1_cnf(int cnf);
int 		TPX64XXQ_get_sx_xio0_cnf();
int			TPX64XXQ_get_sx_xio1_cnf();
void 		TPX64XXQ_set_sx_xio0_data(int v);
void 		TPX64XXQ_set_sx_xio1_data(int v);
int 		TPX64XXQ_get_sx_xio0_data();
int 		TPX64XXQ_get_sx_xio1_data();

		// aliases
#define		TPX64XXQ_sx_xio0_cnf()				TPX64XXQ_get_sx_xio0_cnf()
#define		TPX64XXQ_sx_xio1_cnf()				TPX64XXQ_get_sx_xio1_cnf()
#define		TPX64XXQ_sx_xio0_data()				TPX64XXQ_get_sx_xio0_data()
#define		TPX64XXQ_sx_xio1_data()				TPX64XXQ_get_sx_xio1_data()

		// direct set/get macros
#define		TPX64XXQ_SET_SX_XIO0_CNF_INPUT		TPX64XXQ_set_sx_xio0_cnf(TPX64XXQ_SX_XIO_CNF_INPUT)
#define		TPX64XXQ_SET_SX_XIO0_CNF_OUTPUT		TPX64XXQ_set_sx_xio0_cnf(TPX64XXQ_SX_XIO_CNF_OUTPUT)

#define		TPX64XXQ_SET_SX_XIO1_CNF_INPUT		TPX64XXQ_set_sx_xio1_cnf(TPX64XXQ_SX_XIO_CNF_INPUT)
#define		TPX64XXQ_SET_SX_XIO1_CNF_OUTPUT		TPX64XXQ_set_sx_xio1_cnf(TPX64XXQ_SX_XIO_CNF_OUTPUT)

#define		TPX64XXQ_SX_XIO0_CNF_IS_INPUT		(TPX64XXQ_get_sx_xio0_cnf()==TPX64XXQ_SX_XIO_CNF_INPUT)
#define		TPX64XXQ_SX_XIO0_CNF_IS_OUTPUT		(TPX64XXQ_get_sx_xio0_cnf()==TPX64XXQ_SX_XIO_CNF_OUTPUT)

#define		TPX64XXQ_SX_XIO1_CNF_IS_INPUT		(TPX64XXQ_get_sx_xio1_cnf()==TPX64XXQ_SX_XIO_CNF_INPUT)
#define		TPX64XXQ_SX_XIO1_CNF_IS_OUTPUT		(TPX64XXQ_get_sx_xio1_cnf()==TPX64XXQ_SX_XIO_CNF_OUTPUT)

#define		TPX64XXQ_SET_SX_XIO0_DATA_OFF		TPX64XXQ_set_sx_xio0_data(TPX64XXQ_OFF)
#define		TPX64XXQ_SET_SX_XIO0_DATA_ON		TPX64XXQ_set_sx_xio0_data(TPX64XXQ_ON)

#define		TPX64XXQ_SET_SX_XIO1_DATA_OFF		TPX64XXQ_set_sx_xio1_data(TPX64XXQ_OFF)
#define		TPX64XXQ_SET_SX_XIO1_DATA_ON		TPX64XXQ_set_sx_xio1_data(TPX64XXQ_ON)

#define		TPX64XXQ_SX_XIO0_DATA_IS_OFF		(TPX64XXQ_get_sx_xio0_data()==TPX64XXQ_OFF)
#define		TPX64XXQ_SX_XIO0_DATA_IS_ON			(TPX64XXQ_get_sx_xio0_data()==TPX64XXQ_ON)

#define		TPX64XXQ_SX_XIO1_DATA_IS_OFF		(TPX64XXQ_get_sx_xio1_data()==TPX64XXQ_OFF)
#define		TPX64XXQ_SX_XIO1_DATA_IS_ON			(TPX64XXQ_get_sx_xio1_data()==TPX64XXQ_ON)


	// extended API functions for C64xx DSP on=chip McBSP-0/1/2 serial port configration and I/O
	// for DSP-to-DSP serial communication.
void		TPX64XXQ_init_mcbsp0();
void		TPX64XXQ_set_mcbsp0_data(unsigned d);
unsigned	TPX64XXQ_get_mcbsp0_data();

void		TPX64XXQ_init_mcbsp1();
void		TPX64XXQ_set_mcbsp1_data(unsigned d);
unsigned	TPX64XXQ_get_mcbsp1_data();

void		TPX64XXQ_init_mcbsp2();
void		TPX64XXQ_set_mcbsp2_data(unsigned d);
unsigned	TPX64XXQ_get_mcbsp2_data();


	// direct macros for C64xx DSP McBSP ports access
	// (these macros can be used in user application in order to poll X/R ready conditions and read/write data)
		// C6x McBSP local defs in case C6X_DEF.H file is not included into user code
#define	__TPX64XXQ_C6X_MCBSP0_DRR      			((unsigned *) 0x018C0000)
#define	__TPX64XXQ_C6X_MCBSP0_DXR      			((unsigned *) 0x018C0004)
#define	__TPX64XXQ_C6X_MCBSP0_SPCR     			((unsigned *) 0x018C0008)
#define	__TPX64XXQ_C6X_MCBSP1_DRR      			((unsigned *) 0x01900000)
#define	__TPX64XXQ_C6X_MCBSP1_DXR      			((unsigned *) 0x01900004)
#define	__TPX64XXQ_C6X_MCBSP1_SPCR     			((unsigned *) 0x01900008)
#define	__TPX64XXQ_C6X_MCBSP2_DRR      			((unsigned *) 0x01a40000)
#define	__TPX64XXQ_C6X_MCBSP2_DXR      			((unsigned *) 0x01a40004)
#define	__TPX64XXQ_C6X_MCBSP2_SPCR     			((unsigned *) 0x01a40008)
#define	__TPX64XXQ_C6X_MCBSP_SPCR_XRDY_BITMASK	0x00020000
#define	__TPX64XXQ_C6X_MCBSP_SPCR_RRDY_BITMASK	0x00000002

		// direct macros for McBSP-0/1/2 ports X/R ready conditions
#define	TPX64XXQ_MCBSP0_XRDY_IS_OFF	   			(!((*__TPX64XXQ_C6X_MCBSP0_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_XRDY_BITMASK))
#define	TPX64XXQ_MCBSP0_XRDY_IS_ON	   			((*__TPX64XXQ_C6X_MCBSP0_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_XRDY_BITMASK)
#define	TPX64XXQ_MCBSP0_RRDY_IS_OFF    			(!((*__TPX64XXQ_C6X_MCBSP0_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_RRDY_BITMASK))
#define	TPX64XXQ_MCBSP0_RRDY_IS_ON    			((*__TPX64XXQ_C6X_MCBSP0_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_RRDY_BITMASK)

#define	TPX64XXQ_MCBSP1_XRDY_IS_OFF	   			(!((*__TPX64XXQ_C6X_MCBSP1_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_XRDY_BITMASK))
#define	TPX64XXQ_MCBSP1_XRDY_IS_ON	   			((*__TPX64XXQ_C6X_MCBSP1_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_XRDY_BITMASK)
#define	TPX64XXQ_MCBSP1_RRDY_IS_OFF    			(!((*__TPX64XXQ_C6X_MCBSP1_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_RRDY_BITMASK))
#define	TPX64XXQ_MCBSP1_RRDY_IS_ON    			((*__TPX64XXQ_C6X_MCBSP1_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_RRDY_BITMASK)

#define	TPX64XXQ_MCBSP2_XRDY_IS_OFF	   			(!((*__TPX64XXQ_C6X_MCBSP2_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_XRDY_BITMASK))
#define	TPX64XXQ_MCBSP2_XRDY_IS_ON	   			((*__TPX64XXQ_C6X_MCBSP2_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_XRDY_BITMASK)
#define	TPX64XXQ_MCBSP2_RRDY_IS_OFF    			(!((*__TPX64XXQ_C6X_MCBSP2_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_RRDY_BITMASK))
#define	TPX64XXQ_MCBSP2_RRDY_IS_ON    			((*__TPX64XXQ_C6X_MCBSP2_SPCR)&__TPX64XXQ_C6X_MCBSP_SPCR_RRDY_BITMASK)

#define	TPX64XXQ_WAIT_MCBSP0_XRDY           	while(TPX64XXQ_MCBSP0_XRDY_IS_OFF)
#define	TPX64XXQ_WAIT_MCBSP0_RRDY           	while(TPX64XXQ_MCBSP0_RRDY_IS_OFF)
#define	TPX64XXQ_WAIT_MCBSP1_XRDY           	while(TPX64XXQ_MCBSP1_XRDY_IS_OFF)
#define	TPX64XXQ_WAIT_MCBSP1_RRDY           	while(TPX64XXQ_MCBSP1_RRDY_IS_OFF)
#define	TPX64XXQ_WAIT_MCBSP2_XRDY           	while(TPX64XXQ_MCBSP2_XRDY_IS_OFF)
#define	TPX64XXQ_WAIT_MCBSP2_RRDY           	while(TPX64XXQ_MCBSP2_RRDY_IS_OFF)

		// direct macros for McBSP-0/1/2 ports X/R data write/read
#define	TPX64XXQ_SET_MCBSP2_DXR_DATA(d)			(*__TPX64XXQ_C6X_MCBSP2_DXR) = d
#define	TPX64XXQ_GET_MCBSP2_DRR_DATA           	(*__TPX64XXQ_C6X_MCBSP2_DRR)
#define	TPX64XXQ_SET_MCBSP1_DXR_DATA(d)        	(*__TPX64XXQ_C6X_MCBSP1_DXR) = d
#define	TPX64XXQ_GET_MCBSP1_DRR_DATA           	(*__TPX64XXQ_C6X_MCBSP1_DRR)
#define	TPX64XXQ_SET_MCBSP0_DXR_DATA(d)        	(*__TPX64XXQ_C6X_MCBSP0_DXR) = d
#define	TPX64XXQ_GET_MCBSP0_DRR_DATA           	(*__TPX64XXQ_C6X_MCBSP0_DRR)
			// aliases
#define	TPX64XXQ_MCBSP2_DRR_DATA               	TPX64XXQ_GET_MCBSP2_DRR_DATA
#define	TPX64XXQ_MCBSP1_DRR_DATA               	TPX64XXQ_GET_MCBSP1_DRR_DATA
#define	TPX64XXQ_MCBSP0_DRR_DATA               	TPX64XXQ_GET_MCBSP0_DRR_DATA
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



//~~~~~~~~~~~~~ Source code for Extended API functions ~~~~~~~~~~~~~~~~~
#ifndef __TPX64XXQ_EXCLUDE_LOCAL_FUNCTIONS__
#include "c6x_def.h"	// include C6x DSP defs

// DSP init function
// ~~~~~~~~~~~~~~~~~~
// This function performs initialization of TPX64xxQ DSP EMIF configuration registers
// in accordance with the DSP speed grade and external memory installed,
// GPIO pins configuration and McBSP port configuration (for default operation).
//
// 	Notes:
//	------
//		1.	McBSP ports are configured via corresponding extended API functions, which
// 		 	are using default McBSP ports configuration.
//		2.	In case default McBSP-0/1/2 ports configuration used in TPX64XXQ_init_mcbsp0/1/2()
//			extended API function does not meet user application requirements, then these functions
//			either shall be modified or these function calls shall be commented out from
//			TPX64XXQ_init_dsp() function.
// 		3.	McBPS2 serial port for HORIZONTAL DSP-to-DSP communication is always initialized.
//		4.	McBSP-0 and McBSP-1 serail ports are initialized only in case the corresponding
//			on-board DIAGONAL and VERTICAL serial links are enabled.
//		5.	In case McBSP-0 and/or McBSP-1 serial ports are/is disabled as on-board DSP-to-DSP
//			DIAGONAL and VERTICAL serial links, then TPX64XXQ_init_dsp() extended API function
//			does not intialize corresponding McBSP-0/1 serial port(s) in order to exclude
//			confisuon with probable external MX-SIOX h/w, which can be connected via
//			MX-SIOX site. However, in case McBSP-0/1 serial port(s) is/are used for board-to-board
//			communication via MX-SIOX site, then user application must perform initialization of
//			involved McBSP serial port via TPX64XXQ_init_mcbsp0() and TPX64XXQ_init_mcbsp1()
//			extended API function.


void TPX64XXQ_init_dsp()
{
 	unsigned x;

	// EMIF-B initialization.
	// This must be done prior EMIF-A initialization since all on-board ID registers are read out from
	// this EMIF via IOX control registers.
	*C64XX_EMIFB_GCR 		= TPX64XXQ_EMIFB_GCR_DATA;
	*C64XX_EMIFB_CE0 		= TPX64XXQ_EMIFB_CE0_DATA;
	*C64XX_EMIFB_CE1 		= TPX64XXQ_EMIFB_CE1_DATA;
	*C64XX_EMIFB_CE2 		= TPX64XXQ_EMIFB_CE2_DATA;
	*C64XX_EMIFB_CE3 		= TPX64XXQ_EMIFB_CE3_DATA;

	// wait a little to ensure that EMIF-B has been configured.
	for(x=0;x<100;x++);


	// EMIF-A initialization.
	// Note, that TPX64xxQ boards may differ in SSRAM/SDRAM capacity and DSP speed grade,
	// which result in different SDRAM control register initialization.
	*C64XX_EMIFA_GCR 		= TPX64XXQ_EMIFA_GCR_DATA;

	*C64XX_EMIFA_CE0 		= TPX64XXQ_EMIFA_CE0_DATA;
	*C64XX_EMIFA_CE0_SEC 	= TPX64XXQ_EMIFA_CE0_SEC_DATA;

	*C64XX_EMIFA_CE1 		= TPX64XXQ_EMIFA_CE1_DATA;
	*C64XX_EMIFA_CE0_SEC 	= TPX64XXQ_EMIFA_CE1_SEC_DATA;

	if(TPX64XXQ_SDRAM_IS_INSTALLED)
	{
		// if SDRAM is installed, then configure EMIF-A CE2 area as SDRAM
		*C64XX_EMIFA_CE2 		= TPX64XXQ_EMIFA_CE2_DATA;
		*C64XX_EMIFA_CE2_SEC 	= TPX64XXQ_EMIFA_CE2_SEC_DATA;

		// set SDRAM control registers upon the DSP speed grade and SDRAM capacity
		if(TPX64XXQ_SDRAM_IS_INSTALLED && TPX64XXQ_DSP_SPEED_ID_IS_600MHZ)
		{
			// perform SDRAM setting for 600MHz DSP
			if(TPX64XXQ_SDRAM_LEN_IS_64MB)
			{
				// configure for 16Mx32 SDRAM and DSP speed 600MHz
				*C64XX_EMIFA_SDRAM 		= TPX64XXQ_EMIFA_SDRAM16M_DATA_600MHZ;
				*C64XX_EMIFA_SDRAM_TIMER= TPX64XXQ_EMIFA_SDRAM16M_TIMER_DATA_600MHZ;
			}
			else
			{
				// configure for 4Mx32 SDRAM and DSP speed 600MHz
				*C64XX_EMIFA_SDRAM 		= TPX64XXQ_EMIFA_SDRAM4M_DATA_600MHZ;
				*C64XX_EMIFA_SDRAM_TIMER= TPX64XXQ_EMIFA_SDRAM4M_TIMER_DATA_600MHZ;
			}


			*C64XX_EMIFA_SDRAM_EXT 	= TPX64XXQ_EMIFA_SDRAM_EXT_DATA_600MHZ;
		}

		if(TPX64XXQ_DSP_SPEED_ID_IS_720MHZ)
		{
			// perform SDRAM setting for 720MHz DSP
			if(TPX64XXQ_SDRAM_LEN_IS_64MB)
			{
				// configure for 16Mx32 SDRAM and DSP speed 720MHz
				*C64XX_EMIFA_SDRAM 		= TPX64XXQ_EMIFA_SDRAM16M_DATA_720MHZ;
				*C64XX_EMIFA_SDRAM_TIMER= TPX64XXQ_EMIFA_SDRAM16M_TIMER_DATA_720MHZ;
			}
			else
			{
				// configure for 4Mx32 SDRAM and DSP speed 720MHz
				*C64XX_EMIFA_SDRAM 		= TPX64XXQ_EMIFA_SDRAM4M_DATA_720MHZ;
				*C64XX_EMIFA_SDRAM_TIMER= TPX64XXQ_EMIFA_SDRAM4M_TIMER_DATA_720MHZ;
	 		}

			*C64XX_EMIFA_SDRAM_EXT 	= TPX64XXQ_EMIFA_SDRAM_EXT_DATA_720MHZ;
		}
	}
	else
	{
		// otherwise, if no SDRAM is installed, configure EMIF-A CE2 area as SBSRAM
		// (this will exclude refresh cycles)
		*C64XX_EMIFA_CE2 		= TPX64XXQ_EMIFA_CE0_DATA;
		*C64XX_EMIFA_CE2_SEC 	= TPX64XXQ_EMIFA_CE0_SEC_DATA;
	}

	*C64XX_EMIFA_CE3 		= TPX64XXQ_EMIFA_CE3_DATA;
	*C64XX_EMIFA_CE3_SEC 	= TPX64XXQ_EMIFA_CE3_SEC_DATA;


	// read from dedicated SSRAM location, which corresponds to DSP-to-DSP request
	// to this node (in order to clear this request) and write to list SSRAM location
	// in order to clear transferred token
	if(TPX64XXQ_SSRAM_IS_INSTALLED)
	{
		x=TPX64XXQ_GET_W32_SSRAM_RQ_TO_ME;
		TPX64XXQ_SET_W32_SSRAM_RQ_TO_ME(x&0);	// use X&0==0 in order to exclude compiler warning re never usage of X
 	}

	// Initialize GPIO control registers in order to control MXSIOX RESET (GPIO-0)
	// and TM/XIO-0/1 OE control (GPIO-14/15) at the run-time using extended API
	// functions below.
	// All GPIO-0/14/15 pins shall be configured as OUTPUTs.
	// GPIO-0 pin must be set to deault '0' output vale in order to set SX-RESET
	// into the 'ON' state (active low). GPIO-14/15 outputs shall be set to the '1'
	// state in order to configure SIOX TM/XIO pins as INPUTs.
	// Note, that GPEN-4..7 bits of GPEN register shall be hold in the '1' state (0xf0) in order to have
	// DSP external interrupts EXT_INT4..7 enabled.
	(*C6X_GPEN)	= (0xf0 | TPX64XXQ_GPIO_SX_RESET_BITMASK | TPX64XXQ_GPIO_SX_XIO0_OE_BITMASK | TPX64XXQ_GPIO_SX_XIO1_OE_BITMASK);
	(*C6X_GPDIR)= (TPX64XXQ_GPIO_SX_RESET_BITMASK | TPX64XXQ_GPIO_SX_XIO0_OE_BITMASK | TPX64XXQ_GPIO_SX_XIO1_OE_BITMASK);
	(*C6X_GPVAL)= (TPX64XXQ_GPIO_SX_XIO0_OE_INPUT | TPX64XXQ_GPIO_SX_XIO1_OE_INPUT);


	// Configure DSP on-chip McBSP-0/1/2 serial ports, which are used for DSP-to-DSP communication
	// as serial links. McBSP ports are configured via corresponding extended API functions, which
	// are using default McBSP ports configuration.
		// set default configuration for McBSP-0 port in case DIAGONAL serial link is enabled
	if(TPX64XXQ_DIAG_LINK_ENABLE_IS_ON)
		TPX64XXQ_init_mcbsp0();

		// set default configuration for McBSP-1 port in case VERTICAL serial link is enabled
	if(TPX64XXQ_VERT_LINK_ENABLE_IS_ON)
		TPX64XXQ_init_mcbsp1();

		// set default configuration for McBSP-2 port for always enabled HORIZONTAL serial link
	TPX64XXQ_init_mcbsp2();
	
	
	// finally, clear DSPINT interrupt request from host if it has occured
	TPX64XXQ_CLR_HPIC_DSPINT;
}





// MX-SIOX RESET control
// ~~~~~~~~~~~~~~~~~~~~~~
//  MX-SIOX RESET output signal is generated as GPIO-0 output pin of C64xx GPIO-DATA register.
//	MX-SIOX RESET is active low and is set to 'ON' if GPIO-0 is set to the '0' state.
//	Input parameter is V={TPX64XXQ_SX_RESET_OFF/ON} or V={TPX64XXQ_OFF/ON}

void TPX64XXQ_set_sx_reset(int v)
// set MX-SIOX RESET line:
//		v=TPX64XXQ_SX_RESET_OFF/TPX64XXQ_OFF  - set SIOX RESET to 'off', i.e. enables MX-SIOX (i.e. deactivates MX-SIOX reset signal)
//		v=TPX64XXQ_SX_RESET_ON/TPX64XXQ_ON - set SIOX RESET to 'on', i.e. disables MX-SIOX (i.e. activates MX-SIOX reset signal)
{
 	if((v&1)==TPX64XXQ_OFF)
 		(*C6X_GPVAL)	|= TPX64XXQ_GPIO_SX_RESET_BITMASK;
 	else
 		(*C6X_GPVAL)	&= ~TPX64XXQ_GPIO_SX_RESET_BITMASK;
}


int TPX64XXQ_get_sx_reset()
// returns current MX-SIOX RESET line status:
//		TPX64XXQ_SX_RESET_ON/TPX64XXQ_ON  - if MX-SIOX RESET is in the 'ON' state, i.e. MX-SIOX is disabled
//		TPX64XXQ_SX_RESET_OFF/TPX64XXQ_OFF- if MX-SIOX RESET is in the 'OFF' state, i.e. MX-SIOX is enabled
{
 return(((*C6X_GPVAL) & TPX64XXQ_GPIO_SX_RESET_BITMASK) ? TPX64XXQ_OFF:TPX64XXQ_ON);
}



// MX-SIOX TM/XIO-0/1 I/O control
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	MX-SIOX TM/XIO-0/1 I/O pins are routed to C64xx DSP on-chip TOUT0/1 and TINP0/1 timer I/O pins correspondingly
//	and can	be configured as either general purpose I/O or timer I/O via bits FUNC and CLKSRC of the
//	corresponding C64xx DSP on-chip timer control register.
//
// 	For drection control of MX-SIOX TM/XIO-0/1 I/O pins, extended API functions are using C64xx GPIO-14/15 bits,
//	which shall be initially configured as OUTPUTs in TPX64XXQ_init_dsp() extended APIO function.
//
//		Notes:
//		------
//			1. 	In order to exclude any confusions with use DSP application, TPX64xxQ extended API functions
//				provide TPX64XXQ_set_xio0/1_cnf() functions, whcih allow to configre MX-SIOX TM/XIO-0/1 pins
//				as either general INPUT or OUTPUT (via C64xx GPIO-14/15 output pins) with particular pin
//				function designation (either general purpose I/O or timer I/O) defined by user DSP application.
//			2.	TPX64XXQ_set_xio0/1_data() extended API functions, if invoked will configure the corresponding
//				DSP on-chip	timer output pins (TOUT0 or TOUT1) as general purpose OUTPUT by setting bit FUNC=0
//				of the corresponding C64xx DSP on-chip timer control register. So, as soon this(these) function(s)
//				is(are) invoked, the corresponding DSP on-chip	timer output pin(s) (TOUT0 or TOUT1) will remain
//				as general purpose OUTPUT until it(they) will be re-configured back as timer OUTPUT by user DSP
//				applications.
//			3.	TPX64XXQ_get_xio0/1_data() extended API functions always return current state of the corresponding
//				MX-SIOX TM/XIO-0/1 pins via C64xx DSP on-chip TINP0 or TINP1 timer input pin correspondingly



void TPX64XXQ_set_sx_xio0_cnf(int cnf)
// 	Configures MX-SIOX TM/XIO-0 pin to appear as general INPUT or OUTPUT pin routed to C64xx DSP on-chip
//	TINP0 or TOUT0 timer I/O pin correspondingly. Direction is controlled  via C64xx DSP on=chip GPIO-14 bit.
// 	Input parameter is CNF={TPX64XXQ_SX_XIO_CNF_OUTPUT/INPUT}
{

	if((cnf&1)==TPX64XXQ_SX_XIO_CNF_OUTPUT)
 		(*C6X_GPVAL)	&= ~TPX64XXQ_GPIO_SX_XIO0_OE_BITMASK;
 	else
 		(*C6X_GPVAL)	|= TPX64XXQ_GPIO_SX_XIO0_OE_BITMASK;
}


void TPX64XXQ_set_sx_xio1_cnf(int cnf)
// 	Configures MX-SIOX TM/XIO-1 pin to appear as general INPUT or OUTPUT pin routed to C64xx DSP on-chip
//	TINP1 or TOUT1 timer I/O pin correspondingly. Direction is controlled  via C64xx DSP on=chip GPIO-15 bit.
// 	Input parameter is CNF={TPX64XXQ_SX_XIO_CNF_OUTPUT/INPUT}
{

	if((cnf&1)==TPX64XXQ_SX_XIO_CNF_OUTPUT)
 		(*C6X_GPVAL)	&= ~TPX64XXQ_GPIO_SX_XIO1_OE_BITMASK;
 	else
 		(*C6X_GPVAL)	|= TPX64XXQ_GPIO_SX_XIO1_OE_BITMASK;
}


int TPX64XXQ_get_sx_xio0_cnf()
// 	Returns current configuration of MX-SIOX TM/XIO-0 pin as either general INPUT or OUTPUT pin routed
//	to C64xx DSP on-chip TINP0 or TOUT0 timer I/O pin correspondingly. Direction is controlled  via C64xx DSP on=chip GPIO-14 bit.
// 	Returned value is CNF={TPX64XXQ_SX_XIO_CNF_OUTPUT/INPUT}
{
	return(((*C6X_GPVAL) & TPX64XXQ_GPIO_SX_XIO0_OE_BITMASK) ?  TPX64XXQ_SX_XIO_CNF_INPUT:TPX64XXQ_SX_XIO_CNF_OUTPUT);
}


int TPX64XXQ_get_sx_xio1_cnf()
// 	Returns current configuration of MX-SIOX TM/XIO-1 pin as either general INPUT or OUTPUT pin routed
//	to C64xx DSP on-chip TINP1 or TOUT1 timer I/O pin correspondingly. Direction is controlled  via C64xx DSP on=chip GPIO-15 bit.
// 	Returned value is CNF={TPX64XXQ_SX_XIO_CNF_OUTPUT/INPUT}
{
	return(((*C6X_GPVAL) & TPX64XXQ_GPIO_SX_XIO1_OE_BITMASK) ?  TPX64XXQ_SX_XIO_CNF_INPUT:TPX64XXQ_SX_XIO_CNF_OUTPUT);
}

// local bit defs for C64xx DSP on-chip TIMER CONTROL register
#define	__C6X_TM_CONTROL_FUNC_BITMASK__			0x1
#define	__C6X_TM_CONTROL_DATOUT_BITMASK__		0x4
#define	__C6X_TM_CONTROL_DATIN_BITMASK__		0x8


void TPX64XXQ_set_sx_xio0_data(int v)
// 	This function must be invoked in case MX-SIOX TM/XIO-0 pin has been configured as general OUTPUT pin only.
//	Configures C64xx DSP on-chip TOUT0 timer-0 output pin to appear as general purpose OUTPUT pin by
//	means of setting bit FUNC=0 of C64xx DSP on-chip timer-0 control register. Data output is performed via
//	bit DATOUT of C64xx DSP on-chip timer-0 control register. Pin TOUT0 will remain as general purpose output
//	(FUNC=0) until it will re-configured as timer-0 output by user DSP applications.
// 	Input parameter is V={TPX64XXQ_ON/OFF} or  {TPX64XXQ_SX_XIO_DATA_ON/OFF}.
{

	// configure TOUT0 pin as general purpose output with state control via bit DATOUT of TIMER-0 CONTROL register
	// This settng will remain until re-configured by user DSP application.
	*C6X_TM0_CONTROL &= ~__C6X_TM_CONTROL_FUNC_BITMASK__;

	if((v&1)==TPX64XXQ_ON)
		(*C6X_TM0_CONTROL) |= __C6X_TM_CONTROL_DATOUT_BITMASK__;
 	else
		(*C6X_TM0_CONTROL) &= ~__C6X_TM_CONTROL_DATOUT_BITMASK__;
}


void TPX64XXQ_set_sx_xio1_data(int v)
// 	This function must be invoked in case MX-SIOX TM/XIO-1 pin has been configured as general OUTPUT pin only.
//	Configures C64xx DSP on-chip TOUT1 timer-1 output pin to appear as general purpose OUTPUT pin by
//	means of setting bit FUNC=0 of C64xx DSP on-chip timer-1 control register. Data output is performed via
//	bit DATOUT of C64xx DSP on-chip timer-1 control register. Pin TOUT0 will remain as general purpose output
//	(FUNC=0) until it will re-configured as timer-0 output by user DSP applications.
// 	Input parameter is V={TPX64XXQ_ON/OFF} or  {TPX64XXQ_SX_XIO_DATA_ON/OFF}.
{

	// configure TOUT1 pin as general purpose output with state control via bit DATOUT of TIMER-1 CONTROL register
	// This settng will remain until re-configured by user DSP application.
	*C6X_TM1_CONTROL &= ~__C6X_TM_CONTROL_FUNC_BITMASK__;

	if((v&1)==TPX64XXQ_ON)
		(*C6X_TM1_CONTROL) |= __C6X_TM_CONTROL_DATOUT_BITMASK__;
 	else
		(*C6X_TM1_CONTROL) &= ~__C6X_TM_CONTROL_DATOUT_BITMASK__;
}



int TPX64XXQ_get_sx_xio0_data()
// 	This function can be invoked in case MX-SIOX TM/XIO-0 pin has been configre as either general INPUT or
//	OUTPUT pin.	Returns current state of C64xx DSP on-chip TINP0 timer-0 input pin, which is always routed
//	to MX-SIOX TM/XIO-0 I/O pin. Current pin state is defined by bit DATIN of C64xx DSP on-chip timer-0
//	control register.
// 	Returned value is V={TPX64XXQ_ON/OFF} or  {TPX64XXQ_SX_XIO_DATA_ON/OFF}.
{
	return(((*C6X_TM0_CONTROL) & __C6X_TM_CONTROL_DATIN_BITMASK__) ? TPX64XXQ_ON:TPX64XXQ_OFF);
}


int TPX64XXQ_get_sx_xio1_data()
// 	This function can be invoked in case MX-SIOX TM/XIO-1 pin has been configre as either general INPUT or
//	OUTPUT pin.	Returns current state of C64xx DSP on-chip TINP1 timer-1 input pin, which is always routed
//	to MX-SIOX TM/XIO-1 I/O pin. Current pin state is defined by bit DATIN of C64xx DSP on-chip timer-1
//	control register.
// 	Returned value is V={TPX64XXQ_ON/OFF} or  {TPX64XXQ_SX_XIO_DATA_ON/OFF}.
{
	return(((*C6X_TM1_CONTROL) & __C6X_TM_CONTROL_DATIN_BITMASK__) ? TPX64XXQ_ON:TPX64XXQ_OFF);
}




// DSP-to-DSP communication functions via C64xx DSP on-chip McBSP-0/1/2 serial ports
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//	TPX64xxQ uses C64xx DSP on-chip McBSP-0/1/2 serial ports for DSP-to-DSP communication
//	as the following:
//		- McBSP-0 port can be either used for on-board DIAGONAL DSP-to-DSP communication, or
//		  can be routed to on-board MX-SIOX SIO-0 port as it is defined via DIAGONAL link
//		  enable bit of SYS STATUS register. McBSP-0 port, if not used as on-board DIAGONAL
//		  DSP-to-DSP communication, can be used for either connection to external SIOX h/w
//		  (AD/DA, DIO, application specific coprocessors, etc), or can be used for board-to-board
//		  communication, that is similar to on-board DSP-to-DSP communication.
//		- McBSP-1 port can be either used for on-board VERTICAL DSP-to-DSP communication, or
//		  can be routed to on-board MX-SIOX SIO-1 port as it is defined via VERTICAL link
//		  enable bit of SYS STATUS register. McBSP-1 port, if not used as on-board VERTICAL
//		  DSP-to-DSP communication, can be used for either connection to external SIOX h/w
//		  (AD/DA, DIO, application specific coprocessors, etc), or can be used for board-to-board
//		  communication, that is similar to on-board DSP-to-DSP communication.
//		- McBSP-2 port is always used for on-board HORIZONTAL DSP-to-DSP communication.
//
//	Provided TPX64XXQ_init_mcbsp0/1/2() extended API functions perform default intialization of
//	corresponding C64xx DSP on-chip McBSP ports as the following:
//		- both transmitter and receiver are configured with active low FSX/FSR, active high
//		  CLKX/CLKR and active high DX/DR
//		- receiver is configured with externally drivern FSR and CLKR
//		- transmitter CLKX is driven by McBSP sample rate generator equal to 75MBit/s1 for 600MHz
//		  C64xx DSP and 72Mbit/s for 720MHz C64xx DSP in case this McBSP port is used for on-board
//		  DSP-to-DSP communication, or is set to 60MBit/s for in case this McBSP port is not used
//		  for on-board DSP-to-DSP communication (60MBit/s is good for board-to-board DSP-to-DSP
//		  communication via MX-SIOX connector)
//		- transmitter FSX is driven by transmitter internal circuitry on each write to transmitter
//		  data register (DXR)
//		- both transmitter and receiver are configured with FSX/FSR 1-bit delay (XDATLY=1),
//		  32-bit element length, and single phase data frame with one data word.
//
//	In case provided default McBSP-0/1/2 ports configuration used in TPX64XXQ_init_mcbsp0/1/2()
//	extended API function does not meet user application requirements, then these functions
//	either shall be modified or these function calls shall be commented out from TPX64XXQ_init_dsp()
//	function.
//
//	Provided TPX64XXQ_set/get_mcbsp0/1_data() extended API function provide s/w polling for
//	corresponding XRDY and RRDY bits of McBSP port control register (SPCR) in order to determine
//	data ready condition.
//
//	TPX64XXQ_set/get_mcbsp0/1_data() extended API function do not check whether corresponding
//	McBSP-0/1 serial ports are enabled for on-board DSP-to-DSP communication or used for
//	off-board communication.


	// default McBSP port register settings defs:
		// SPCR register:
		//		- all bits are in default power-on settings
		//
		// 	SPCR: 0000 0000 0000 0000 0000 0000 0000 0000
#define	__TPX64XXQ_MCBSP_SPCR_DATA						0x00000000
#define	__TPX64XXQ_MCBSP_SPCR_GO_DATA					0x00410001		// must be ORed with SPCR in order to
																		// enable SRG, and X/R
		// PCR register:
		//		- positive X/R clock polarity (CLKRP/CLKXP=0)
		//		- inverse X/R frame sync polarity (FSXP/FSRP=1)
		//      - CLKR is driven by external clock (CLKRM=0)
		//		- CLKX is driven by internal sample rate generator (CLKXM=1)
		//		- FSR is driven externally (FSRM=0)
		//		- FSX is defined by FSX mode bit of SRGR register (actually on DXR-to-XSR transfer)
		//	PCR: 0000 1010 0000 1100
#define	__TPX64XXQ_MCBSP_PCR_DATA						0x0A0C

		// RCR/XCR register:
		//		- single-phase frame (RPHASE/XPHASE=0)
		//		- 1 word per phase (RFRLEN1/XFRLEN1='0000000')
		//		- 32-bit words (RWDLEN1/XWDLEN1='101')
		//		- no companding (RCOMPAND/XCOMPAND='00')
		//		- unexpecetd R/X frame restrats the transfer (RFIG/XFIG=0)
		//		- 1-bit FSX/FSR delay (RDATLY/XDATLY='00')
		//	RCR/XCR: 0000 0000 0000 0001 0000 0000 1010 0000
#define	__TPX64XXQ_MCBSP_RCR_DATA						0x000100a0
#define	__TPX64XXQ_MCBSP_XCR_DATA						0x000100a0

		// SRGR register defines bit rate and FSX generation mode
		//		- CPU clock is used (CLKSM=1)
		//		- FSX is generated on DXR-to-XSR transfer (FSGM=0)
		//		- CLKGDV is set upon the McBSP port usage and DSP/4 clock (on/off-board communication, 600/720MHZ DSP)
		//	SRGR: 0010 0000 0000 0000 0000 0000 <CLKGDV-1>
#define	__TPX64XXQ_MCBSP_SRGR_DATA_ONBOARD_600MHZ		0x20000001		// on-board 75MHz bit rate from 600MHz DSP
#define	__TPX64XXQ_MCBSP_SRGR_DATA_ONBOARD_720MHZ		0x20000002		// on-board 60MHz bit rate from 720MHz DSP
#define	__TPX64XXQ_MCBSP_SRGR_DATA_EXTERNAL_600MHZ		0x20000002		// external 50MHz bit rate from 600MHz DSP
#define	__TPX64XXQ_MCBSP_SRGR_DATA_EXTERNAL_720MHZ		0x20000003		// external 45MHz bit rate from 720MHz DSP




		// McBSP-0 port configuration function
		// McBSP-0 port can be either used for on-board DIAGONAL DSP-to-DSP communication if enabled via
		// corresponding bit of SYS STATUS register, or is used for external communication via MX-SIOX SIO-0 port.
void		TPX64XXQ_init_mcbsp0()
{
 	int i;

	*C6X_MCBSP0_SPCR 	= __TPX64XXQ_MCBSP_SPCR_DATA;
	*C6X_MCBSP0_PCR    	= __TPX64XXQ_MCBSP_PCR_DATA;
	*C6X_MCBSP0_RCR		= __TPX64XXQ_MCBSP_RCR_DATA;
	*C6X_MCBSP0_XCR		= __TPX64XXQ_MCBSP_XCR_DATA;

	if(TPX64XXQ_DIAG_LINK_ENABLE_IS_ON)
	{
		// use these sttings in case McBSP port is used for on-board DSP-to-DSP communication
		*C6X_MCBSP0_SRGR	= (TPX64XXQ_DSP_SPEED_ID_IS_600MHZ ? __TPX64XXQ_MCBSP_SRGR_DATA_ONBOARD_600MHZ:
																	__TPX64XXQ_MCBSP_SRGR_DATA_ONBOARD_720MHZ);
 	}
	else
	{
		// use these sttings in case McBSP port is used for on-board DSP-to-DSP communication
		*C6X_MCBSP0_SRGR	= (TPX64XXQ_DSP_SPEED_ID_IS_600MHZ ? __TPX64XXQ_MCBSP_SRGR_DATA_EXTERNAL_600MHZ:
																	__TPX64XXQ_MCBSP_SRGR_DATA_EXTERNAL_720MHZ);
	}

	// wait for some time to allow McBSP to configure at configured clock and
	// enable McBSP port operation afterthat, and wait again for some time
	for(i=0;i<50;i++);
	*C6X_MCBSP0_SPCR	|= __TPX64XXQ_MCBSP_SPCR_GO_DATA;
	for(i=0;i<50;i++);
}


		// data transmission function over McBSP-0 serial port
		// This function perform polling for XRDY condition.
void		TPX64XXQ_set_mcbsp0_data(unsigned d)
{
 	TPX64XXQ_WAIT_MCBSP0_XRDY;
 	TPX64XXQ_SET_MCBSP0_DXR_DATA(d);
}


		// data reception function over McBSP-0 serial port
		// This function perform polling for RRDY condition and may idle infinitely if
		// no input data is received.
unsigned	TPX64XXQ_get_mcbsp0_data()
{
 	TPX64XXQ_WAIT_MCBSP0_RRDY;
	return(TPX64XXQ_GET_MCBSP0_DRR_DATA);
}





		// McBSP-1 port configuration function
		// McBSP-1 port can be either used for on-board VERTICAL DSP-to-DSP communication if enabled via
		// corresponding bit of SYS STATUS register, or is ised for external communication via MX-SIOX SIO-1 port.
void		TPX64XXQ_init_mcbsp1()
{
 	int i;

	*C6X_MCBSP1_SPCR 	= __TPX64XXQ_MCBSP_SPCR_DATA;
	*C6X_MCBSP1_PCR    	= __TPX64XXQ_MCBSP_PCR_DATA;
	*C6X_MCBSP1_RCR		= __TPX64XXQ_MCBSP_RCR_DATA;
	*C6X_MCBSP1_XCR		= __TPX64XXQ_MCBSP_XCR_DATA;

	if(TPX64XXQ_VERT_LINK_ENABLE_IS_ON)
	{
		// use these sttings in case McBSP port is used for on-board DSP-to-DSP communication
		*C6X_MCBSP1_SRGR	= (TPX64XXQ_DSP_SPEED_ID_IS_600MHZ ? __TPX64XXQ_MCBSP_SRGR_DATA_ONBOARD_600MHZ:
																	__TPX64XXQ_MCBSP_SRGR_DATA_ONBOARD_720MHZ);
 	}
	else
	{
		// use these sttings in case McBSP port is used for on-board DSP-to-DSP communication
		*C6X_MCBSP1_SRGR	= (TPX64XXQ_DSP_SPEED_ID_IS_600MHZ ? __TPX64XXQ_MCBSP_SRGR_DATA_EXTERNAL_600MHZ:
																	__TPX64XXQ_MCBSP_SRGR_DATA_EXTERNAL_720MHZ);
	}

	// wait for some time to allow McBSP to configure at configured clock and
	// enable McBSP port operation afterthat, and wait again for some time
	for(i=0;i<50;i++);
	*C6X_MCBSP1_SPCR	|= __TPX64XXQ_MCBSP_SPCR_GO_DATA;
	for(i=0;i<50;i++);
}


		// data transmission function over McBSP-1 serial port
		// This function perform polling for XRDY condition.
void		TPX64XXQ_set_mcbsp1_data(unsigned d)
{
 	TPX64XXQ_WAIT_MCBSP1_XRDY;
 	TPX64XXQ_SET_MCBSP1_DXR_DATA(d);
}


		// data reception function over McBSP-1 serial port
		// This function perform polling for RRDY condition and may idle infinitely if
		// no input data is received.
unsigned	TPX64XXQ_get_mcbsp1_data()
{
 	TPX64XXQ_WAIT_MCBSP1_RRDY;
	return(TPX64XXQ_GET_MCBSP1_DRR_DATA);
}




		// McBSP-2 port configuration function
		// McBSP-2 port is always used for on-board HORIZONTAL DSP-to-DSP communication.
void		TPX64XXQ_init_mcbsp2()
{
 	int i;

	*C6X_MCBSP2_SPCR 	= __TPX64XXQ_MCBSP_SPCR_DATA;
	*C6X_MCBSP2_PCR    	= __TPX64XXQ_MCBSP_PCR_DATA;
	*C6X_MCBSP2_RCR		= __TPX64XXQ_MCBSP_RCR_DATA;
	*C6X_MCBSP2_XCR		= __TPX64XXQ_MCBSP_XCR_DATA;
	*C6X_MCBSP2_SRGR	= (TPX64XXQ_DSP_SPEED_ID_IS_600MHZ ? __TPX64XXQ_MCBSP_SRGR_DATA_ONBOARD_600MHZ:
																__TPX64XXQ_MCBSP_SRGR_DATA_ONBOARD_720MHZ);

	// wait for some time to allow McBSP to configure at configured clock and
	// enable McBSP port operation afterthat, and wait again for some time
	for(i=0;i<50;i++);
	*C6X_MCBSP2_SPCR	|= __TPX64XXQ_MCBSP_SPCR_GO_DATA;
	for(i=0;i<50;i++);
}


		// data transmission function over McBSP-2 serial port
		// This function perform polling for XRDY condition.
void		TPX64XXQ_set_mcbsp2_data(unsigned d)
{
 	TPX64XXQ_WAIT_MCBSP2_XRDY;
 	TPX64XXQ_SET_MCBSP2_DXR_DATA(d);
}


		// data reception function over McBSP-2 serial port
		// This function perform polling for RRDY condition and may idle infinitely if
		// no input data is received.
unsigned	TPX64XXQ_get_mcbsp2_data()
{
 	TPX64XXQ_WAIT_MCBSP2_RRDY;
	return(TPX64XXQ_GET_MCBSP2_DRR_DATA);
}

#endif
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

//=========================================================================

#endif
