/*******************************************************************************************
  TORNADO-PX64xxQ Quad 'C64xx PIOX/PIOX-16 DSP Coprocessor Utilities. Rev.1A-1
  Host DSP definitions for TORNADO DSP systems/controllers with C3x/C6000 DSP.

  (C) MicroLAB Systems Ltd, 2003
  WEB: http://www.mlabsys.com
  FTP: ftp://ftp.mlabsys.com


  Revision history:
  -----------------
	rev.1A	- March-2003, initial release

	rev.1A-1 - July-2003
		- updated COFF loader function in order to support upload os COFF sections
		  allocated at non-x4 address boundary and with non-x4 bytes length


  Notes:
  ------
    1. 	This file must be used as .H header file for C3x/C6x DSP environment of
	   	host TORNADO-3x/P3x/E3x/6x/P6x/E6x DSP systems and controllers.

    2. 	This file can be best viewed with the TAB setting equal to '4'.

	3.	This header file is externally controlled from user C-code by run-time
		compiler definitions (keys), which has to be defined prior inclusion of this
		header file into user code. Run-time compiler keys shall be used in order to
		match DSP type, TORNADO board types, to define PIOX/PIOX-16 base address, and to
		include/exclude function bodies of extended API functions.

		run-time compiler defs (mutually exclusive) for selection of host TORNADO DSP platform:
		~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
			__HOST_C6XXX_DSP_PX16_AINC4__    (** default)
				- if defined to any value in user host DSP .C file for, then this applications
				  is assumed to be compiled for TORNADO DSP systems and controllers with
				  C6xxx DSP, which feature 16-bit PIOX-16 I/F data word allocated to 16-bit LSW of
				  32-bit DSP data word, i.e. address increment for PIOX-16 data words is '+4'.
				  This definition is valid for TORNADO-6x/P62/P67/P6202/P6203/P64xx DSP systems
				  and TORNADO-E62/E67/E6202/E6203 DSP controllers, and cannot be used with TORNADO-E64x
				  DSP controllers.

				  Data type for access to control registers of PIOX/PIOX-16 interface in both PIOX and
				  PIOX-16 modes, and to 16-bit C64xx DSP HPI port registers in 16-bit PIOX-16
				  mode is defined via __HOST_C6XXX_DSP_CNTR_RG_W32__ and __HOST_C6XXX_DSP_CNTR_RG_W16__
				  run-time compiler keys. However, 32-bit C64xx DSP HPI port registers in 32-bit
				  PIOX mode are accessed using 32-bit UNSIGNED data type.

				  In case __PIOX_SADDR__ definition is omitted in host DSP .C source file,
				  then it is defined in accordance with default setting for TORNADO-P64xx DSP
				  systems.

				  This is default setting if none of __HOST_C6XXX_DSP_PX16_AINC2__ and
				  __HOST_C3X_DSP__ definitions is provided in user host DSP .C file.


			__HOST_C6XXX_DSP_PX16_AINC2__
				- if defined to any value in user host DSP .C file for, then this application
				  is assumed to be compiled for TORNADO-E64xx DSP controllers only with 16-bit
				  PIOX-16 data word allocated as a continous array of 16-bit words with
				  address increment '+2'. This definition corresponds to 16-bit PIOX-16 mode.
				  All control registers of  PIOX-16 interface and 16-bit C64xx DSP HPI
				  port registers in 16-bit PIOX-16 mode are accessed using UNSIGNED SHORT data type.
				  This is the only ultimate setting for TORNADO-E64xx DSP controllers.

				  __HOST_C6XXX_DSP_CNTR_RG_W32__ and __HOST_C6XXX_DSP_CNTR_RG_W16__
				  run-time compiler keys are ignored.

				  If  __PIOX_SADDR__ definition is omitted in host DSP .C file, then it will be
				  set to default setting for TORNADO-E64xx DSP controllers.


			__HOST_C3X_DSP__
				- if defined to any value in host DSP .C file for, then this applications
				  is assumed to be compiled for C3x DSP for all TORNADO-3x/TP3x/E3x DSP systems
				  and controllers.

				  If __PIOX_SADDR__ definition is omitted in host .C file, then it is set to
				  default 0xff0000 value, which is a common setting for all TORNADO-3x/TP3x/E3x
				  DSP systems/controllers.


		run-time compiler keys (mutually exclusive) for selection of data type for access to CONTROL REGISTERS of host PIOX I/F:
		~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
			__HOST_C6XXX_DSP_CNTR_RG_W32__    (** default)
				- if defined to any value in user host DSP .C file for, then CONTROL
				  registers and 16-bit C64xx HPI port registers of host PIOX/PIOX-16 interface
				  are accessed as 32-bit UNSIGNED data type variables. This setting is
				  valid only in case __HOST_C6XXX_DSP_PX16_AINC4__ compiler key is defined.
				  This setting is an ultimate setting for TORNADO-P64xx DSP systems for applications,
				  which require 32-bit HPI port configuration for TORNADO-PX64xxQ on-board DSP.
				  Since PIOX/PIOX-16 interface of TORNADO-P64xx DSP systems is mapped into 16-bit
				  C64x DSP EMIF-B data bus with data access format controlled via IOX control register,
				  then it is not possible for host DSP to perform 32-bit access to C64xx DSP on-chip HPI
				  registers and simultaneously access PIOX I/F CONTROL REGISTERS using 16-bit data type
				  (refer to TORNADO-P6xxx User's Guide for more details, section 2.2) without
				  modification of TORNADO-P64xx PIOX I/F DATA ACCESS FORMAT IOX register before
				  host access to 32-bit C64xx DSP on-chip HPI port and to 16-bit PIOX CONTROL registers of
				  TORNADO-PX64xxQ. This setting will select host 32-bit access to both C64xx DSP on-chip
				  HPI port and PIOX I/F control registers in order to resolve this confusion and will
				  allow to support both 16-bit and 32-bit C64xx DSP HPI port configuration, although
				  this setting will deliver one exta cycle for access to 16-bit PIOX I/F data.

				  TORNADO-6x/P62/P67/P6202/P6203/E62/E67/E6202/E6203 DSP boards are not critical for
				  these run-time compiler keys, however the recommended setting is default
				  __HOST_C6XXX_DSP_CNTR_RG_W32__.


			__HOST_C6XXX_DSP_CNTR_RG_W16__
				- if defined to any value in user host DSP .C file for, then CONTROL
				  registers and 16-bit C64xx HPI port registers of host PIOX/PIOX-16 interface
				  are accessed as 16-bit UNSIGNED SHORT data type variables. This setting is
				  valid only in case __HOST_C6XXX_DSP_PX16_AINC4__ compiler key is defined.
				  This setting is an ultimate setting for TORNADO-P64xx DSP systems for applications,
				  which require 16-bit HPI port configuration for TORNADO-PX64xxQ on-board DSP.
				  Since PIOX/PIOX-16 interface of TORNADO-P64xx DSP systems is mapped into 16-bit
				  C64x DSP EMIF-B data bus with data access format controlled via IOX control register
				  (refer to TORNADO-P6xxx User's Guide for more details, section 2.2), then it is
				  possible to save one host DSP access cycle to 16-bit PIOX I/F CONTROL registers
				  and 16-bit C64xx DSP on-chip HPI port. This setting can be used only in case
				  16-bit HPI data format is used for C64xx DSP of TORNADO-PX64xxQ and cannot be used
				  with 32-bit HPI data format.

				  TORNADO-6x/P62/P67/P6202/P6203/E62/E67/E6202/E6203 DSP boards are not critical for
				  these run-time compiler keys, however the recommended setting is default
				  __HOST_C6XXX_DSP_CNTR_RG_W32__.




		run-time compiler key for setting host PIOX I/F base address
		~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
			__PIOX_SADDR__
				- must be defined along with __HOST_C6XXX_DSP_PX16_AINC4__, __HOST_C6XXX_DSP_PX16_AINC2__,
				  and __HOST_C3X_DSP__ definitions in order to set PIOX/PIOX-16 I/O base address
				  in host DSP memory map.

				  For some TORNADO DSP systems and controllers, __PIOX_SADDR__ definition can be
				  omitted with setting to default value (refer to the above description for
				  __HOST_C6XXX_DSP_PX16_AINC4__, __HOST_C6XXX_DSP_PX16_AINC2__, and __HOST_C3X_DSP__
				  definitions.


		run-time compiler keys for excluding/inclusion of extendion API functions bodies:
		~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
			__HTPX64XXQ_EXCLUDE_AUX_FUNCTIONS__
				- if defined in user code (assigned to any value), then the function bodies for local
	      		  aux functions for HPI access, HPI init, HIF init, etc are excluded from the user
				  source code. Default is INCLUDED. Note, that in case  user application comprises
				  of several .C linked together modules, then this run-time compiler key must be
				  included in all modules except for only one root module in order to exclude multiple
				  function body appearance.


			__HTPX64XXQ_EXCLUDE_CBSF_PROGRAM_LOADER__
				- if defined in user code (assigned to any value), then the function body for
				  C6xxx C-BSF loader function (HPTX64XXQ_load_program()) is excluded from the
				  user source code. Default is INCLUDED. Note, that in case user application comprises
				  of several .C linked together modules, then this run-time compiler key must be
				  included in all modules except for only one root module in order to exclude multiple
				  function body appearance.


    4. 	TI C3x/C4x Code Composer C/Assembler compiler tools are required to compile this header file
	    for C3x DSP, and TI C6000 Code Composer Studio v.2.0 or later C/Assembler compiler tools are
		required to compile this header file for C6xxx DSP.


*******************************************************************************************/

#ifndef __HPX64XXQ_C3X_C6XXX__
#define __HPX64XXQ_C3X_C6XXX__		1

#include	"stdlib.h"


//==================== DSP platform and host PIOX-16 address defs ========================
//	Host TORNADO DSP platform def (__HOST_C6XXX_DSP_PX16_AINC4__, or __HOST_C6XXX_DSP_PX16_AINC2__,
//	or __HOST_C3X_DSP__) and	PIOX/PIOX-16 start address def (__PIOX_SADDR__) shall be
//	defined in user host DSP C-code prior inclusion of this header file in order to
//	provide	correct operation of TORNADO-PX64xxQ utilities.
//
//	__HOST_C6XXX_DSP_PX16_AINC4__, or __HOST_C6XXX_DSP_PX16_AINC2__, and __HOST_C3X_DSP__
//	can be set to any value prior inclusion of this header file into user C-code.
//
//	Refer to the corresponding .H header file or user's guide for your TORNADO DSP
//	system/controller in order to define the value for __PIOX_SADDR__ definintion,
//	which corresponds to the PIOX/PIOX-16 base address value or your TORNADO DSP
//	system/controller. Note, that host TORNADO-P64xx/E64xx/3x/P3x/E3x DSP systems and
//	controllers do not require setting of __PIOX_SADDR__ run-time compiler key, since
//	the corresponding default values are used upon defined __HOST_C6XXX_DSP_PX16_AINC4__,
//	__HOST_C6XXX_DSP_PX16_AINC2__ and _HOST_C3X_DSP_ run-time compiler keys.
//
//  In case none of __HOST_C6XXX_DSP_PX16_AINC4__, __HOST_C6XXX_DSP_PX16_AINC2__
//	and __HOST_C3X_DSP__ run-time compiler keys is defined in user host DSP C-code prior
//	inclusion of this header file, then default  __HOST_C6XXX_DSP_PX16_AINC4__ is
//	selected with __PIOX_SADDR__ run-time compiler key set to default value for
//	TORNADO-P64xx DSP systems.


	// check for host C6xxx DSP platform with 16-bit PIOX-16 data words allocated as
	// LSW at 32-bit words boundaries with address increment '+4'. This is the most
	// common settings for TORNADO DSP systems/controllers except for TORNADO-E64xx
	// DSP controllers. Use this setting as default if none of compiler keys has been defined.
#if ((defined __HOST_C6XXX_DSP_PX16_AINC4__) ||  !((defined __HOST_C6XXX_DSP_PX16_AINC4__) || (defined __HOST_C6XXX_DSP_PX16_AINC2__) || (defined __HOST_C3X__)))

	#ifndef __HOST_C6XXX_DSP_PX16_AINC4__
		#define __HOST_C6XXX_DSP_PX16_AINC4__			1
	#endif

		// treat alternate defs as erroneous defs
	#ifdef __HOST_C6XXX_DSP_PX16_AINC2__
		#undef __HOST_C6XXX_DSP_PX16_AINC2__
	#endif

	#ifdef __HOST_C3X_DSP__
		#undef __HOST_C3X_DSP__
	#endif
		// word address step definition
	#define	_WAS_										4


		// check for __HOST_C6XXX_DSP_CNTR_RG_W32__ and __HOST_C6XXX_DSP_CNTR_RG_W16__ definitions.
		// If not defined, set __HOST_C6XXX_DSP_CNTR_RG_W32__ as default, otherwise leave it as it is.
	#if ((defined __HOST_C6XXX_DSP_CNTR_RG_W32__) || (!(defined __HOST_C6XXX_DSP_CNTR_RG_W32__) && !(defined __HOST_C6XXX_DSP_CNTR_RG_W16__)))

		#ifndef __HOST_C6XXX_DSP_CNTR_RG_W32__
			#define __HOST_C6XXX_DSP_CNTR_RG_W32__		1
		#endif

		// treat alternate def as erroneous
		#ifdef  __HOST_C6XXX_DSP_CNTR_RG_W16__
			#undef __HOST_C6XXX_DSP_CNTR_RG_W16__
		#endif
	#endif


		// check for __PIOX_SADDR__ definition.
		// If not defined, set to PIOX/PIOX-16 base address for TORNADO-P64xx DSP systems.
		// If defined, then leave it as it is.
	#ifndef __PIOX_SADDR__
		#define	__PIOX_SADDR__							0x6C000000
	#endif
#endif



	// check for host C6xxx DSP platform with 16-bit PIOX-16 data words allocated as
	// continuous array of 16-bit data words with address increment '+2'. This is the only
	// ultimate setting for TORNADO-E64xx DSP controllers.
#if ((defined __HOST_C6XXX_DSP_PX16_AINC2__))
		// treat alternate defs as erroneous defs
	#ifdef __HOST_C6XXX_DSP_PX16_AINC4__
		#undef __HOST_C6XXX_DSP_PX16_AINC4__
	#endif

	#ifdef __HOST_C3X_DSP__
		#undef __HOST_C3X_DSP__
	#endif
		// word address step definition
	#define	_WAS_										2

		// check for __PIOX_SADDR__ definition.
		// In case __PIOX_SADDR__ key has been defined, then leave it as it is if defined,
		// otherwise, set it to PIOX-16 base address	for TORNADO-E64xx DSP controller. .
	#ifndef __PIOX_SADDR__
		#define	__PIOX_SADDR__						0x60100000
	#endif
#endif




	// Finally, check for host C3x DSP platform.
	// We can exclude to undefine all other defs, since we would never get there if they are defined.
#ifdef __HOST_C3X_DSP__
		// word address step definition
	#define	_WAS_								1

		// If not defined, define PIOX/PIOX-16 address to common value for all
		// T3x/TP3x/E3x DSP systems/controllers. If defined, then leave it as it is.
	#ifndef  __PIOX_SADDR__
		#define	__PIOX_SADDR__					0xff0000
	#endif
#endif



	// Finally we must define C-code data type for accessing PIOX I/F control registers
	// and C64xx HPI registers upon the DSP platform used.
#if ((defined __HOST_C6XXX_DSP_PX16_AINC4__) ||  (defined __HOST_C6XXX_DSP_PX16_AINC2__))
		// these defs are for all C6xx DSP platforms

	#if ((defined __HOST_C6XXX_DSP_PX16_AINC4__) && (defined __HOST_C6XXX_DSP_CNTR_RG_W32__))
		// __HOST_C6XXX_DSP_CNTR_RG_W32__ run-time compiler key is valid along with
		// __HOST_C6XXX_DSP_PX16_AINC4__ run-time compiler key only
		typedef	unsigned 						__HTPX64XXQ_CNTR_RG_DATA_TYPE__;
		typedef	unsigned 						__HTPX64XXQ_HPI16_RG_DATA_TYPE__;
	#else
		typedef	unsigned short					__HTPX64XXQ_CNTR_RG_DATA_TYPE__;
		typedef	unsigned short					__HTPX64XXQ_HPI16_RG_DATA_TYPE__;
	#endif
	typedef	unsigned 							__HTPX64XXQ_HPI32_RG_DATA_TYPE__;
#else
		// these defs are for all C3x DSP platforms
	typedef	unsigned 							__HTPX64XXQ_CNTR_RG_DATA_TYPE__;
	typedef	unsigned 							__HTPX64XXQ_HPI16_RG_DATA_TYPE__;
	typedef	unsigned 							__HTPX64XXQ_HPI32_RG_DATA_TYPE__;
#endif
//==========================================================================================



//========================== general defs ==================================================

#define	HTPX64XXQ_OFF							0
#define	HTPX64XXQ_ON							1


	// TPX64XxQ internal DSP clock defs
#define	HTPX64XXQ_DSP_CLK_FREQ_600MHZ			6.0e+8	// 600 MHz DSP clock with C64xx silicon 1.03
#define	HTPX64XXQ_DSP_CLK_FREQ_720MHZ			7.2e+8	// 720 MHz DSP clock with C64xx silicon 1.1
//==========================================================================================



//===================== host PIOX/PIOX-16 ports address definitions ========================

	// control registers addresses
	// ----------------------------
	//	Address for PIOX control registers do not depend upon the PIOX data format mode.
#define HTPX64XXQ_CNTR1_RG_ADDR					(__PIOX_SADDR__+(0x0000*_WAS_))	// r/w: CONTROL #1 register
#define HTPX64XXQ_CNTR2_RG_ADDR					(__PIOX_SADDR__+(0x0001*_WAS_))	// r/w: CONTROL #2 register
#define HTPX64XXQ_DSP_CNF_STAT_RG_ADDR			(__PIOX_SADDR__+(0x0004*_WAS_))	// r: BootMode status and HPI data format mode for each DSP
#define HTPX64XXQ_DSP_LINK_CNF_RG_ADDR			(__PIOX_SADDR__+(0x0007*_WAS_))	// r/w: DSP serial LINK configuration
#define HTPX64XXQ_DEV_ID_RG_ADDR				(__PIOX_SADDR__+(0x0008*_WAS_))	// r: device/revision ID
#define HTPX64XXQ_DSP_XMEM_LEN_ID_RG_ADDR		(__PIOX_SADDR__+(0x0009*_WAS_))	// r: DSP external SSRAM/SDRAM memories length IDs
#define HTPX64XXQ_INT_STAT_RG_ADDR				(__PIOX_SADDR__+(0x000b*_WAS_))	// r: interrupt status
#define HTPX64XXQ_HIRQ0_SEL_RG_ADDR				(__PIOX_SADDR__+(0x000c*_WAS_))	// r/w: Host IRQ-0 source selector
#define HTPX64XXQ_HIRQ1_SEL_RG_ADDR				(__PIOX_SADDR__+(0x000d*_WAS_))	// r/w: Host IRQ-1 source selector
#define HTPX64XXQ_HIRQ2_SEL_RG_ADDR				(__PIOX_SADDR__+(0x000e*_WAS_))	// r/w: Host IRQ-2 source selector
#define HTPX64XXQ_HIRQ3_SEL_RG_ADDR				(__PIOX_SADDR__+(0x000f*_WAS_))	// r/w: Host IRQ-3 source selector
#define HTPX64XXQ_CLR_DSPA_HPI_ERR_ADDR			(__PIOX_SADDR__+(0x0010*_WAS_))	// w: clear HPI timeout flag for DSP#A
#define HTPX64XXQ_CLR_DSPB_HPI_ERR_ADDR			(__PIOX_SADDR__+(0x0011*_WAS_))	// w: clear HPI timeout flag for DSP#B
#define HTPX64XXQ_CLR_DSPC_HPI_ERR_ADDR			(__PIOX_SADDR__+(0x0012*_WAS_))	// w: clear HPI timeout flag for DSP#C
#define HTPX64XXQ_CLR_DSPD_HPI_ERR_ADDR			(__PIOX_SADDR__+(0x0013*_WAS_))	// w: clear HPI timeout flag for DSP#D
#define HTPX64XXQ_DSP_BROADCAST_IRQ_ADDR		(__PIOX_SADDR__+(0x0018*_WAS_))	// w: generate common broadcast IRQ to all DSPs


	// C64xx DSP on-chip HPI ports addresses
	// -------------------------------------
	//	Provided are address definitions for 32-bt PIOX data format mode (with _W32_ suffix)
	//	and for 16-bit PIOX-16 mode (with _LSW_/_MSW_ suffixes).
#define HTPX64XXQ_DSPA_HPIC_RG_W32_ADDR			(__PIOX_SADDR__+(0x0080*_WAS_))	// r/w: DSP#A 32-bit HPIC reg
#define HTPX64XXQ_DSPA_HPIC_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x0080*_WAS_))	// r/w: DSP#A 16-bit LSW of HPIC reg
#define HTPX64XXQ_DSPA_HPIC_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x0081*_WAS_))	// r/w: DSP#A 16-bit MSW of HPIC reg
#define HTPX64XXQ_DSPA_HPIA_RG_W32_ADDR			(__PIOX_SADDR__+(0x0081*_WAS_))	// r/w: DSP#A 32-bit HPIA reg
#define HTPX64XXQ_DSPA_HPIA_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x0082*_WAS_))	// r/w: DSP#A 16-bit LSW of HPIA reg
#define HTPX64XXQ_DSPA_HPIA_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x0083*_WAS_))	// r/w: DSP#A 16-bit MSW of HPIA reg
#define HTPX64XXQ_DSPA_HPID_AINC_RG_W32_ADDR	(__PIOX_SADDR__+(0x0082*_WAS_))	// r/w: DSP#A 32-bit HPID-AINC reg
#define HTPX64XXQ_DSPA_HPID_AINC_RG_W16LSW_ADDR	(__PIOX_SADDR__+(0x0084*_WAS_))	// r/w: DSP#A 16-bit LSW of HPID-AINC reg
#define HTPX64XXQ_DSPA_HPID_AINC_RG_W16MSW_ADDR	(__PIOX_SADDR__+(0x0085*_WAS_))	// r/w: DSP#A 16-bit MSW of HPID-AINC reg
#define HTPX64XXQ_DSPA_HPID_RG_W32_ADDR			(__PIOX_SADDR__+(0x0083*_WAS_))	// r/w: DSP#A 32-bit HPID reg
#define HTPX64XXQ_DSPA_HPID_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x0086*_WAS_))	// r/w: DSP#A 16-bit LSW of HPID reg
#define HTPX64XXQ_DSPA_HPID_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x0087*_WAS_))	// r/w: DSP#A 16-bit MSW of HPID reg

#define HTPX64XXQ_DSPB_HPIC_RG_W32_ADDR			(__PIOX_SADDR__+(0x0088*_WAS_))	// r/w: DSP#B 32-bit HPIC reg
#define HTPX64XXQ_DSPB_HPIC_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x0088*_WAS_))	// r/w: DSP#B 16-bit LSW of HPIC reg
#define HTPX64XXQ_DSPB_HPIC_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x0089*_WAS_))	// r/w: DSP#B 16-bit MSW of HPIC reg
#define HTPX64XXQ_DSPB_HPIA_RG_W32_ADDR			(__PIOX_SADDR__+(0x0089*_WAS_))	// r/w: DSP#B 32-bit HPIA reg
#define HTPX64XXQ_DSPB_HPIA_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x008A*_WAS_))	// r/w: DSP#B 16-bit LSW of HPIA reg
#define HTPX64XXQ_DSPB_HPIA_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x008B*_WAS_))	// r/w: DSP#B 16-bit MSW of HPIA reg
#define HTPX64XXQ_DSPB_HPID_AINC_RG_W32_ADDR	(__PIOX_SADDR__+(0x008A*_WAS_))	// r/w: DSP#B 32-bit HPID-AINC reg
#define HTPX64XXQ_DSPB_HPID_AINC_RG_W16LSW_ADDR	(__PIOX_SADDR__+(0x008C*_WAS_))	// r/w: DSP#B 16-bit LSW of HPID-AINC reg
#define HTPX64XXQ_DSPB_HPID_AINC_RG_W16MSW_ADDR	(__PIOX_SADDR__+(0x008D*_WAS_))	// r/w: DSP#B 16-bit MSW of HPID-AINC reg
#define HTPX64XXQ_DSPB_HPID_RG_W32_ADDR			(__PIOX_SADDR__+(0x008B*_WAS_))	// r/w: DSP#B 32-bit HPID reg
#define HTPX64XXQ_DSPB_HPID_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x008E*_WAS_))	// r/w: DSP#B 16-bit LSW of HPID reg
#define HTPX64XXQ_DSPB_HPID_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x008F*_WAS_))	// r/w: DSP#B 16-bit MSW of HPID reg

#define HTPX64XXQ_DSPC_HPIC_RG_W32_ADDR			(__PIOX_SADDR__+(0x0090*_WAS_))	// r/w: DSP#C 32-bit HPIC reg
#define HTPX64XXQ_DSPC_HPIC_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x0090*_WAS_))	// r/w: DSP#C 16-bit LSW of HPIC reg
#define HTPX64XXQ_DSPC_HPIC_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x0091*_WAS_))	// r/w: DSP#C 16-bit MSW of HPIC reg
#define HTPX64XXQ_DSPC_HPIA_RG_W32_ADDR			(__PIOX_SADDR__+(0x0091*_WAS_))	// r/w: DSP#C 32-bit HPIA reg
#define HTPX64XXQ_DSPC_HPIA_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x0092*_WAS_))	// r/w: DSP#C 16-bit LSW of HPIA reg
#define HTPX64XXQ_DSPC_HPIA_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x0093*_WAS_))	// r/w: DSP#C 16-bit MSW of HPIA reg
#define HTPX64XXQ_DSPC_HPID_AINC_RG_W32_ADDR	(__PIOX_SADDR__+(0x0092*_WAS_))	// r/w: DSP#C 32-bit HPID-AINC reg
#define HTPX64XXQ_DSPC_HPID_AINC_RG_W16LSW_ADDR	(__PIOX_SADDR__+(0x0094*_WAS_))	// r/w: DSP#C 16-bit LSW of HPID-AINC reg
#define HTPX64XXQ_DSPC_HPID_AINC_RG_W16MSW_ADDR	(__PIOX_SADDR__+(0x0095*_WAS_))	// r/w: DSP#C 16-bit MSW of HPID-AINC reg
#define HTPX64XXQ_DSPC_HPID_RG_W32_ADDR			(__PIOX_SADDR__+(0x0093*_WAS_))	// r/w: DSP#C 32-bit HPID reg
#define HTPX64XXQ_DSPC_HPID_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x0096*_WAS_))	// r/w: DSP#C 16-bit LSW of HPID reg
#define HTPX64XXQ_DSPC_HPID_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x0097*_WAS_))	// r/w: DSP#C 16-bit MSW of HPID reg

#define HTPX64XXQ_DSPD_HPIC_RG_W32_ADDR			(__PIOX_SADDR__+(0x0098*_WAS_))	// r/w: DSP#D 32-bit HPIC reg
#define HTPX64XXQ_DSPD_HPIC_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x0098*_WAS_))	// r/w: DSP#D 16-bit LSW of HPIC reg
#define HTPX64XXQ_DSPD_HPIC_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x0099*_WAS_))	// r/w: DSP#D 16-bit MSW of HPIC reg
#define HTPX64XXQ_DSPD_HPIA_RG_W32_ADDR			(__PIOX_SADDR__+(0x0099*_WAS_))	// r/w: DSP#D 32-bit HPIA reg
#define HTPX64XXQ_DSPD_HPIA_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x009A*_WAS_))	// r/w: DSP#D 16-bit LSW of HPIA reg
#define HTPX64XXQ_DSPD_HPIA_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x009B*_WAS_))	// r/w: DSP#D 16-bit MSW of HPIA reg
#define HTPX64XXQ_DSPD_HPID_AINC_RG_W32_ADDR	(__PIOX_SADDR__+(0x009A*_WAS_))	// r/w: DSP#D 32-bit HPID-AINC reg
#define HTPX64XXQ_DSPD_HPID_AINC_RG_W16LSW_ADDR	(__PIOX_SADDR__+(0x009C*_WAS_))	// r/w: DSP#D 16-bit LSW of HPID-AINC reg
#define HTPX64XXQ_DSPD_HPID_AINC_RG_W16MSW_ADDR	(__PIOX_SADDR__+(0x009D*_WAS_))	// r/w: DSP#D 16-bit MSW of HPID-AINC reg
#define HTPX64XXQ_DSPD_HPID_RG_W32_ADDR			(__PIOX_SADDR__+(0x009B*_WAS_))	// r/w: DSP#D 32-bit HPID reg
#define HTPX64XXQ_DSPD_HPID_RG_W16LSW_ADDR		(__PIOX_SADDR__+(0x009E*_WAS_))	// r/w: DSP#D 16-bit LSW of HPID reg
#define HTPX64XXQ_DSPD_HPID_RG_W16MSW_ADDR		(__PIOX_SADDR__+(0x009F*_WAS_))	// r/w: DSP#D 16-bit MSW of HPID reg

//==========================================================================================


//======================== host PIOX-16 register data definitions ==========================

#define HTPX64XXQ_PX_RG_DATA_BITMASK		0xff	// D0..D7 only are valid for PIOX control registers

	// Control #1 Register bits
	// ------------------------
	// Includes DSP-A..D reset control.
		// Bitmasks
#define HTPX64XXQ_DSPA_RESET_BITMASK			0x1
#define HTPX64XXQ_DSPB_RESET_BITMASK			0x2
#define HTPX64XXQ_DSPC_RESET_BITMASK			0x4
#define HTPX64XXQ_DSPD_RESET_BITMASK			0x8

		// direct RESET ON/OFF values
#define HTPX64XXQ_DSPA_RESET_ON 				0x0		// DSP#A RESET control
#define HTPX64XXQ_DSPA_RESET_OFF				0x1

#define HTPX64XXQ_DSPB_RESET_ON 				0x0		// DSP#B RESET control
#define HTPX64XXQ_DSPB_RESET_OFF				0x2

#define HTPX64XXQ_DSPC_RESET_ON 				0x0		// DSP#C RESET control
#define HTPX64XXQ_DSPC_RESET_OFF				0x4

#define HTPX64XXQ_DSPD_RESET_ON 				0x0		// DSP#D RESET control
#define HTPX64XXQ_DSPD_RESET_OFF				0x8

#define HTPX64XXQ_DSP_RESET_ON_ALL 				0x0		// value to put all DSP to the RESET state
#define HTPX64XXQ_DSP_RESET_OFF_ALL 			0xf		// value to put all DSP to the RUN state

		// defs for universal _USET_/_UGET_ macros
#define HTPX64XXQ_DSPA_RESET_BITSHIFT			0
#define HTPX64XXQ_DSPB_RESET_BITSHIFT			1
#define HTPX64XXQ_DSPC_RESET_BITSHIFT			2
#define HTPX64XXQ_DSPD_RESET_BITSHIFT			3

			// parameter def for _USET_/_UGET_ macros for DSP resets control
#define HTPX64XXQ_DSP_RESET_ON 					0		// DSP RESET is ON
#define HTPX64XXQ_DSP_RESET_OFF					1		// DSP RESET is OFF



	// Control #2 Register bits
	// ------------------------
	//  Includes HPI timeout error enable, DSP HPI data format and bootmode control.
		// bitmasks
#define HTPX64XXQ_DSP_BMODE_CNTR_BITMASK		0x01    // DSP bootmode control
#define HTPX64XXQ_DSP_HPI_FMT_CNTR_BITMASK		0x02	// DSP HPI data format control
#define HTPX64XXQ_PX_FMT_BITMASK				0x40	// r-only: PIOX data format (PX16/PX32)
#define HTPX64XXQ_DSP_HPI_ERR_EN_BITMASK		0x80	// timeout enable for DSP HPI access to all DSPs

		// bit values defs
#define HTPX64XXQ_DSP_BMODE_CNTR_NONE			0x00	// no C64xx DSP boot
#define HTPX64XXQ_DSP_BMODE_CNTR_HPI			0x01	// C64xx DSP boot via HPI

#define HTPX64XXQ_DSP_HPI_FMT_CNTR_HPI16		0x00	// C64xx DSP HPI data format HPI-16
#define HTPX64XXQ_DSP_HPI_FMT_CNTR_HPI32		0x02	// C64xx DSP HPI data format HPI-32

#define HTPX64XXQ_PX_FMT_PX16					0x00	// r-only: PIOX-16 data format
#define HTPX64XXQ_PX_FMT_PX32					0x40	// r-only: PIOX-32 data format

#define HTPX64XXQ_DSP_HPI_ERR_EN_ON				0x80	// enable HPI TIMEOUT error control
#define HTPX64XXQ_DSP_HPI_ERR_EN_OFF			0x00

		// defs for universal _USET_/_UGET_ macros for HPI ERROR enable control
		// _USET_/_UGET_ macros for HPI ERROR enable control are using HTPX64XXQ_ON/OFF parameters values
#define HTPX64XXQ_DSP_HPI_ERR_EN_BITSHIFT		7



	// DSP Configuration Status register bits (read-only)
	// ---------------------------------------------------
	//  Includes read-only latched valued DSP Bootmode and HPI data format for each DSP, which are
	//	latched on release of RESET signal for each DSP individually.
		// bitmasks
#define HTPX64XXQ_DSPA_BMODE_STAT_BITMASK		0x01    // DSP-A bootmode status
#define HTPX64XXQ_DSPB_BMODE_STAT_BITMASK		0x02    // DSP-B bootmode status
#define HTPX64XXQ_DSPC_BMODE_STAT_BITMASK		0x04    // DSP-C bootmode status
#define HTPX64XXQ_DSPD_BMODE_STAT_BITMASK		0x08    // DSP-D bootmode status
#define HTPX64XXQ_DSPA_HPI_FMT_STAT_BITMASK		0x10	// DSP-A HPI data format status
#define HTPX64XXQ_DSPB_HPI_FMT_STAT_BITMASK		0x20	// DSP-B HPI data format status
#define HTPX64XXQ_DSPC_HPI_FMT_STAT_BITMASK		0x40	// DSP-C HPI data format status
#define HTPX64XXQ_DSPD_HPI_FMT_STAT_BITMASK		0x80	// DSP-D HPI data format status

		// bit values defs
#define HTPX64XXQ_DSPA_BMODE_STAT_NONE			0x00	// DSP-A has been initialized without boot
#define HTPX64XXQ_DSPA_BMODE_STAT_HPI			0x01	// DSP-A has been initialized with HPI boot

#define HTPX64XXQ_DSPB_BMODE_STAT_NONE			0x00	// DSP-B has been initialized without boot
#define HTPX64XXQ_DSPB_BMODE_STAT_HPI			0x02	// DSP-B has been initialized with HPI boot

#define HTPX64XXQ_DSPC_BMODE_STAT_NONE			0x00	// DSP-C has been initialized without boot
#define HTPX64XXQ_DSPC_BMODE_STAT_HPI			0x04	// DSP-C has been initialized with HPI boot

#define HTPX64XXQ_DSPD_BMODE_STAT_NONE			0x00	// DSP-D has been initialized without boot
#define HTPX64XXQ_DSPD_BMODE_STAT_HPI			0x08	// DSP-D has been initialized with HPI boot

#define HTPX64XXQ_DSPA_HPI_FMT_STAT_HPI16		0x00	// DSP-A has been initialized with HPI-16 data format
#define HTPX64XXQ_DSPA_HPI_FMT_STAT_HPI32		0x10	// DSP-A has been initialized with HPI-32 data format

#define HTPX64XXQ_DSPB_HPI_FMT_STAT_HPI16		0x00	// DSP-B has been initialized with HPI-16 data format
#define HTPX64XXQ_DSPB_HPI_FMT_STAT_HPI32		0x20	// DSP-B has been initialized with HPI-32 data format

#define HTPX64XXQ_DSPC_HPI_FMT_STAT_HPI16		0x00	// DSP-C has been initialized with HPI-16 data format
#define HTPX64XXQ_DSPC_HPI_FMT_STAT_HPI32		0x40	// DSP-C has been initialized with HPI-32 data format

#define HTPX64XXQ_DSPD_HPI_FMT_STAT_HPI16		0x00	// DSP-D has been initialized with HPI-16 data format
#define HTPX64XXQ_DSPD_HPI_FMT_STAT_HPI32		0x80	// DSP-D has been initialized with HPI-32 data format

		// defs for universal _USET_/_UGET_ macros
#define HTPX64XXQ_DSPA_BMODE_STAT_BITSHIFT		0
#define HTPX64XXQ_DSPB_BMODE_STAT_BITSHIFT		1
#define HTPX64XXQ_DSPC_BMODE_STAT_BITSHIFT		2
#define HTPX64XXQ_DSPD_BMODE_STAT_BITSHIFT		3
#define HTPX64XXQ_DSPA_HPI_FMT_STAT_BITSHIFT	4
#define HTPX64XXQ_DSPB_HPI_FMT_STAT_BITSHIFT	5
#define HTPX64XXQ_DSPC_HPI_FMT_STAT_BITSHIFT	6
#define HTPX64XXQ_DSPD_HPI_FMT_STAT_BITSHIFT	7

			// parameter def for all _UGET_ macros returning DSP bootmode status
#define HTPX64XXQ_DSP_BMODE_STAT_NONE			0		// denotes that DSP has been initialized without boot
#define HTPX64XXQ_DSP_BMODE_STAT_HPI			1		// denotes that DSP has been initialized with HPI boot

			// parameter def for all _UGET_ macros returning DSP HPI data format status
#define HTPX64XXQ_DSP_HPI_FMT_STAT_HPI16		0		// denotes that DSP has been initialized with HPI-16 data format
#define HTPX64XXQ_DSP_HPI_FMT_STAT_HPI32		1		// denotes that DSP has been initialized with HPI-32 data format



	// VERT/DIAG DSP-to-DSP LINKs Configuration register
	// --------------------------------------------------
	//  Includes enable control for DSP-to-DSP Vertical and Diagonal Links
		// bitmasks
#define HTPX64XXQ_DSPAC_LINK_EN_BITMASK			0x01    // DSP-A to DSP-C VERTICAL Link enable control
#define HTPX64XXQ_DSPBD_LINK_EN_BITMASK			0x02    // DSP-B to DSP-D VERTICAL Link enable control
#define HTPX64XXQ_DSPAD_LINK_EN_BITMASK			0x04    // DSP-A to DSP-D DIAGONAL Link enable control
#define HTPX64XXQ_DSPBC_LINK_EN_BITMASK			0x08    // DSP-B to DSP-C DIAGONAL Link enable control

		// bit values defs
#define HTPX64XXQ_DSPAC_LINK_EN_OFF				0x00
#define HTPX64XXQ_DSPAC_LINK_EN_ON				0x01

#define HTPX64XXQ_DSPBD_LINK_EN_OFF				0x00
#define HTPX64XXQ_DSPBD_LINK_EN_ON				0x02

#define HTPX64XXQ_DSPAD_LINK_EN_OFF				0x00
#define HTPX64XXQ_DSPAD_LINK_EN_ON				0x04

#define HTPX64XXQ_DSPBC_LINK_EN_OFF				0x00
#define HTPX64XXQ_DSPBC_LINK_EN_ON				0x08

		// defs for universal _USET_/_UGET_ macros for DSP-to-DSP links enable control
		// _USET_/_UGET_ macros for DSP-to-DSP links enable are using HTPX64XXQ_ON/OFF parameters values
#define HTPX64XXQ_DSPAC_LINK_EN_BITSHIFT		0
#define HTPX64XXQ_DSPBD_LINK_EN_BITSHIFT		1
#define HTPX64XXQ_DSPAD_LINK_EN_BITSHIFT		2
#define HTPX64XXQ_DSPBC_LINK_EN_BITSHIFT		3



	// Device/Revision ID register bits (read-only)
	// --------------------------------------------
	//  Includes device ID info, revision ID info and DSP speed ID info.
		// bitmasks
#define HTPX64XXQ_DEV_ID_BITMASK				0x0f    // Device ID
#define HTPX64XXQ_REV_ID_BITMASK				0x30    // revision ID
#define HTPX64XXQ_DSP_SPEED_ID_BITMASK			0xc0    // DSP speed ID

		// bit values defs
#define	HTPX6414Q_DEV_ID						0x00	// TPX6414Q device ID
#define	HTPX6415Q_DEV_ID						0x01	// TPX6415Q device ID
#define	HTPX6416Q_DEV_ID						0x02	// TPX6416Q device ID

#define	HTPX64XXQ_REV_ID_REV1					0x10	// ID for board f/w revision 1 (so far the only f/w revision)

#define	HTPX64XXQ_DSP_SPEED_ID_600MHZ			0x00	// 600 MHz C64xx DSP speed grade ID
#define	HTPX64XXQ_DSP_SPEED_ID_720MHZ			0x40	// 720 MHz C64xx DSP speed grade ID



	// DSP external SSRAM/SDRAM memorry length ID register bits (read-only)
	// --------------------------------------------------------------------
	//  Includes installed SSRAM length ID and SDRAM length ID.
		// bitmasks
#define HTPX64XXQ_DSP_SSRAM_LEN_ID_BITMASK		0x03    // SSRAM length
#define HTPX64XXQ_DSP_SDRAM_LEN_ID_BITMASK		0x0c    // SDRAM length

		// bit values defs
#define	HTPX64XXQ_DSP_SSRAM_LEN_ID_0K			0	// SSRAM is not installed
#define	HTPX64XXQ_DSP_SSRAM_LEN_ID_32K			1	// SSRAM capacity is 32Kx32
#define	HTPX64XXQ_DSP_SSRAM_LEN_ID_64K			2	// SSRAM capacity is 64Kx32
		// aliases (for length in bytes)
#define	HTPX64XXQ_DSP_SSRAM_LEN_ID_0KB			HTPX64XXQ_DSP_SSRAM_LEN_ID_0K
#define	HTPX64XXQ_DSP_SSRAM_LEN_ID_128KB		HTPX64XXQ_DSP_SSRAM_LEN_ID_32K
#define	HTPX64XXQ_DSP_SSRAM_LEN_ID_256KB		HTPX64XXQ_DSP_SSRAM_LEN_ID_64K

#define	HTPX64XXQ_DSP_SDRAM_LEN_ID_0M			0	// SDRAM is not installed
#define	HTPX64XXQ_DSP_SDRAM_LEN_ID_4M			4	// SDRAM capacity is 4Mx32 per DSP
#define	HTPX64XXQ_DSP_SDRAM_LEN_ID_16M			8	// SDRAM capacity is 16Mx32 per DSP
		// aliases (for length in bytes)
#define	HTPX64XXQ_DSP_SDRAM_LEN_ID_0MB			HTPX64XXQ_DSP_SDRAM_LEN_ID_0M
#define	HTPX64XXQ_DSP_SDRAM_LEN_ID_16MB			HTPX64XXQ_DSP_SDRAM_LEN_ID_4M
#define	HTPX64XXQ_DSP_SDRAM_LEN_ID_64MB			HTPX64XXQ_DSP_SDRAM_LEN_ID_16M



	// Interrupts status register (read-only)
	// ---------------------------------------
	//	Include HPI HINT and HPI access timeout error status for each DSP.
		// bitmasks
#define HTPX64XXQ_DSPA_HINT_BITMASK				0x01	// DSP-A HPI HINT
#define HTPX64XXQ_DSPB_HINT_BITMASK				0x02	// DSP-B HPI HINT
#define HTPX64XXQ_DSPC_HINT_BITMASK				0x04	// DSP-C HPI HINT
#define HTPX64XXQ_DSPD_HINT_BITMASK				0x08	// DSP-D HPI HINT
#define HTPX64XXQ_DSPA_HPI_ERR_BITMASK			0x10	// timeout error for DSP-A HPI access
#define HTPX64XXQ_DSPB_HPI_ERR_BITMASK			0x20	// timeout error for DSP-B HPI access
#define HTPX64XXQ_DSPC_HPI_ERR_BITMASK			0x40	// timeout error for DSP-C HPI access
#define HTPX64XXQ_DSPD_HPI_ERR_BITMASK			0x80	// timeout error for DSP-D HPI access

		// bit values defs
#define HTPX64XXQ_DSPA_HINT_OFF					0x00
#define HTPX64XXQ_DSPA_HINT_ON					0x01

#define HTPX64XXQ_DSPB_HINT_OFF					0x00
#define HTPX64XXQ_DSPB_HINT_ON					0x02

#define HTPX64XXQ_DSPC_HINT_OFF					0x00
#define HTPX64XXQ_DSPC_HINT_ON					0x04

#define HTPX64XXQ_DSPD_HINT_OFF					0x00
#define HTPX64XXQ_DSPD_HINT_ON					0x08

#define HTPX64XXQ_DSPA_HPI_ERR_OFF				0x00
#define HTPX64XXQ_DSPA_HPI_ERR_ON				0x10

#define HTPX64XXQ_DSPB_HPI_ERR_OFF				0x00
#define HTPX64XXQ_DSPB_HPI_ERR_ON				0x20

#define HTPX64XXQ_DSPC_HPI_ERR_OFF				0x00
#define HTPX64XXQ_DSPC_HPI_ERR_ON				0x40

#define HTPX64XXQ_DSPD_HPI_ERR_OFF				0x00
#define HTPX64XXQ_DSPD_HPI_ERR_ON				0x80

		// defs for universal _UGET_ macros for reading DSP HINT status and HPI ERROR status
		// _UGET_ macros for reading DSP HINT status and HPI ERROR status are using HTPX64XXQ_ON/OFF parameters values
#define HTPX64XXQ_DSPA_HINT_BITSHIFT			0
#define HTPX64XXQ_DSPB_HINT_BITSHIFT			1
#define HTPX64XXQ_DSPC_HINT_BITSHIFT			2
#define HTPX64XXQ_DSPD_HINT_BITSHIFT			3
#define HTPX64XXQ_DSPA_HPI_ERR_BITSHIFT			4
#define HTPX64XXQ_DSPB_HPI_ERR_BITSHIFT			5
#define HTPX64XXQ_DSPC_HPI_ERR_BITSHIFT			6
#define HTPX64XXQ_DSPD_HPI_ERR_BITSHIFT			7



	// PIOX HIRQ-0..3 selector registers
	// ----------------------------------
	//	These defs are common for all PIOX HIRQ-0..3 host IRQ selector registers.
		// bitmasks
#define HTPX64XXQ_HIRQ_SEL_BITMASK				0x07	// HIRQ source selector
#define HTPX64XXQ_HIRQ_EN_BITMASK				0x80	// HIRQ enable bit

		// bit values defs
#define HTPX64XXQ_HIRQ_EN_OFF					0x00	// HIRQ line is disabled (Z-state)
#define HTPX64XXQ_HIRQ_EN_ON					0x80	// HIRQ line is active and source is selected by HIRQ selector bits

#define HTPX64XXQ_HIRQ_SEL_OFF					0x00	// no HIRQ source
#define HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR			0x01	// Logical OR of all HPI timeout errors is the HIRQ source
#define HTPX64XXQ_HIRQ_SEL_DSPA_HINT			0x04	// DSP-A HINT is the HIRQ source
#define HTPX64XXQ_HIRQ_SEL_DSPB_HINT			0x05	// DSP-B HINT is the HIRQ source
#define HTPX64XXQ_HIRQ_SEL_DSPC_HINT			0x06	// DSP-C HINT is the HIRQ source
#define HTPX64XXQ_HIRQ_SEL_DSPD_HINT			0x07	// DSP-D HINT is the HIRQ source
#define HTPX64XXQ_HIRQ_SEL_RSRV1				0x02	// reserved state, corresponds to no HIRQ source
#define HTPX64XXQ_HIRQ_SEL_RSRV2				0x03	// reserved state, corresponds to no HIRQ source

		// defs for universal _USET_/_UGET_ macros for HIRQ lines enable control
		// _USET_/_UGET_ macros for HIRQ lines enable are using HTPX64XXQ_ON/OFF parameters values
#define HTPX64XXQ_HIRQ_EN_BITSHIFT				7
//==========================================================================================



//======================== C64xx DSP HPI port defs =========================================
//	C64xx DSP HPI defs are generally not required for user.
//	Use provide extended API functions for HPI access instead.

	// HPI port data access bitmasks
#define HTPX64XXQ_DSP_HPI16_DATA_BITMASK  		0xffff		// 16-bit bitmask for C64xx DSP HPIC/HPIA/HPID regs for HPI16 data format mode
#define HTPX64XXQ_DSP_HPI32_DATA_BITMASK  		0xffffffff	// 32-bit bitmask for C64xx DSP HPIC/HPIA/HPID regs for HPI32 data format mode


	// C64xx DSP HPIC register bits
	// ----------------------------
	// HPIC register asctually appears as two identical 16-bit LSW/MSW, which have to be set identically
	// for HPI-32 data format mode. For HPI16 mode, only LSW can be written.
		// bitmasks
#define	HTPX64XXQ_DSP_HPIC_HWOB_BITMASK			0x1		// HWOB bit is used for HPI-16 data format only
#define	HTPX64XXQ_DSP_HPIC_DSPINT_BITMASK		0x2
#define	HTPX64XXQ_DSP_HPIC_HINT_BITMASK			0x4
#define	HTPX64XXQ_DSP_HPIC_HRDY_BITMASK			0x8
#define	HTPX64XXQ_DSP_HPIC_FETCH_BITMASK		0x10

		// bit values defs (shal be used for read-only operation)
#define	HTPX64XXQ_DSP_HPIC_HWOB_OFF				0
#define	HTPX64XXQ_DSP_HPIC_HWOB_ON				0x1

#define	HTPX64XXQ_DSP_HPIC_DSPINT_OFF			0
#define	HTPX64XXQ_DSP_HPIC_DSPINT_ON			0x2

#define	HTPX64XXQ_DSP_HPIC_HINT_OFF				0
#define	HTPX64XXQ_DSP_HPIC_HINT_ON				0x4

#define	HTPX64XXQ_DSP_HPIC_HRDY_OFF				0
#define	HTPX64XXQ_DSP_HPIC_HRDY_ON				0x8

#define	HTPX64XXQ_DSP_HPIC_FETCH_ON				0x10

		// write-only HPIC command codes (can be ORed) (HWB=1 for HPI-16 mode, which has  effect for HPI32 mode)
#define	HTPX64XXQ_DSP_HPIC_WR_HINT_OFF			0x5		// HINT:   write '1' in order to clear HINT
#define	HTPX64XXQ_DSP_HPIC_WR_DSPINT_ON			0x3		// DSPINT: write '1' in order to set DSPINT
#define	HTPX64XXQ_DSP_HPIC_WR_FETCH_ON			0x11	// FETCH:  w-only (write '1' in order to set FETCH)
//==========================================================================================



//============= PX64xxQ DSP memory map and defs for host access via HPI port================
// valid HPI address areas for C64xx DSP at TORNADO-PX64xxQ
//
//  Notes:
//	------
//		1. 	Provided for host access are HPI areas addresses for C64xx DSP on-chip memory,
//			on-board SSRAM and SDRAM memories. For system and software environments safety/integrity,
//			other C64xx DSP	environment areas are not recommended for access from host DSP via
//			C64xx DSP on-chip HPI ports.
//
// 		2. 	All host defs, functions and macros, which are referred to DSP environment,
// 			have HTPX64XXQ_DSPE_ prefix.

	// C64xx DSP address range
	// ----------------------------------
#define	HTPX64XXQ_DSPE_ADDR_MIN					0x00000000
#define	HTPX64XXQ_DSPE_ADDR_MAX					0xffffffff


	// C64xx DSP on-chip memory area
	// -----------------------------
#define	HTPX64XXQ_DSPE_ONCHIP_RAM_SADDR			0x00000000
#define	HTPX64XXQ_DSPE_ONCHIP_RAM_EADDR			0x000fffff
#define	HTPX64XXQ_DSPE_ONCHIP_RAM_LEN			0x00100000


	// SSRAM area defs
	// ---------------
		// SDRAM length defs in x32 words
#define	HTPX64XXQ_DSPE_SSRAM_LEN_0K				0x00000000
#define	HTPX64XXQ_DSPE_SSRAM_LEN_32K			0x00020000
#define	HTPX64XXQ_DSPE_SSRAM_LEN_64K			0x00040000
#define	HTPX64XXQ_DSPE_SSRAM_LEN_MAX			HTPX64XXQ_DSPE_SSRAM_LEN_64K	// max available SSRAM length
			// SDRAM length aliases (in bytes)
#define	HTPX64XXQ_DSPE_SSRAM_LEN_0KB			HTPX64XXQ_DSPE_SSRAM_LEN_0K
#define	HTPX64XXQ_DSPE_SSRAM_LEN_128KB			HTPX64XXQ_DSPE_SSRAM_LEN_32K
#define	HTPX64XXQ_DSPE_SSRAM_LEN_256KB			HTPX64XXQ_DSPE_SSRAM_LEN_64K

		// SDRAM area mapping (start/end addresses)
		// Notes:
		// 		 1. Particular SSRAM length ID can be read by host DSP from HTPX64XXQ_DSP_XMEM_LEN_ID_RG
		//		    read-only register.
		// 		 2. Care must be taken, since HTPX64XXQ_DSPE_SSRAM_EADDR macro returns (HTPX64XXQ_DSPE_SSRAM_SADDR-1)
		// 		    value in case SSRAM is not installed!!!
#define	HTPX64XXQ_DSPE_SSRAM_SADDR				0x80000000														// SSRAM start address
#define	HTPX64XXQ_DSPE_SSRAM_EADDR				(HTPX64XXQ_DSPE_SSRAM_SADDR+HTPX64XXQ_get_dsp_ssram_len()-1)  	// last SSRAM address
#define	HTPX64XXQ_DSPE_SSRAM_EADDR_MAX			(HTPX64XXQ_DSPE_SSRAM_SADDR+HTPX64XXQ_DSPE_SSRAM_LEN_MAX-1) 	// maximum SSRAM last address

      	// SSRAM addresses for direct inter-DSP interrupt/message passing via SSRAM
		// (valid for installed SSRAM only)
#define	HTPX64XXQ_DSPE_SSRAM_DSPA_RQ_ADDR		(HTPX64XXQ_DSPE_SSRAM_EADDR-3) 	 // SSRAM address for IRQ to DSP-A
#define	HTPX64XXQ_DSPE_SSRAM_DSPB_RQ_ADDR		(HTPX64XXQ_DSPE_SSRAM_EADDR-7)	 // SSRAM address for IRQ to DSP-B
#define	HTPX64XXQ_DSPE_SSRAM_DSPC_RQ_ADDR		(HTPX64XXQ_DSPE_SSRAM_EADDR-11)	 // SSRAM address for IRQ to DSP-C
#define	HTPX64XXQ_DSPE_SSRAM_DSPD_RQ_ADDR		(HTPX64XXQ_DSPE_SSRAM_EADDR-15)	 // SSRAM address for IRQ to DSP-D



	// SDRAM area defs
	// ---------------
		// SDRAM length defs in x32 words
#define	HTPX64XXQ_DSPE_SDRAM_LEN_0M				0
#define	HTPX64XXQ_DSPE_SDRAM_LEN_4M				0x01000000
#define	HTPX64XXQ_DSPE_SDRAM_LEN_16M			0x04000000
#define	HTPX64XXQ_DSPE_SDRAM_LEN_MAX			HTPX64XXQ_DSPE_SDRAM_LEN_16M    	// max available so far SDRAM length
			// SDRAM length aliases (in bytes)
#define	HTPX64XXQ_DSPE_SDRAM_LEN_0MB			HTPX64XXQ_DSPE_SDRAM_LEN_0M
#define	HTPX64XXQ_DSPE_SDRAM_LEN_16MB			HTPX64XXQ_DSPE_SDRAM_LEN_4M
#define	HTPX64XXQ_DSPE_SDRAM_LEN_64MB			HTPX64XXQ_DSPE_SDRAM_LEN_16M

		// SDRAM area mapping (start/end addresses)
		// Notes:
		// 		 1. Particular SDRAM length ID can be read by host DSP from HTPX64XXQ_DSP_XMEM_LEN_ID_RG
		//		    read-only register.
		// 		 2. Care must be taken, since HTPX64XXQ_DSPE_SDRAM_EADDR macro returns (HTPX64XXQ_DSPE_SDRAM_SADDR-1)
		// 		    value in case SDRAM is not installed!!!
#define	HTPX64XXQ_DSPE_SDRAM_SADDR				0xA0000000														// SDRAM start address
#define	HTPX64XXQ_DSPE_SDRAM_EADDR				(HTPX64XXQ_DSPE_SDRAM_SADDR+HTPX64XXQ_get_dsp_sdram_len()-1)   	// last SDRAM address
#define	HTPX64XXQ_DSPE_SDRAM_EADDR_MAX			(HTPX64XXQ_DSPE_SDRAM_SADDR+HTPX64XXQ_DSPE_SDRAM_LEN_MAX-1) 	// maximum SDRAM last address

//==========================================================================================



//======== optional parameters for universal macros for accessing PX/HPI registers =========

	// these IDs for DSP cores shall be used as 'N' parameter for DSP RESET control macros
	// and for universal HPI access macros
#define	HTPX64XXQ_DSPA							0
#define	HTPX64XXQ_DSPB							1
#define	HTPX64XXQ_DSPC							2
#define	HTPX64XXQ_DSPD							3
#define	HTPX64XXQ_MAX_DSP_NUMBER				3

	// these IDs for DSP HINTs shall be used as 'N' parameter for universal GET_HINT access macros
#define	HTPX64XXQ_DSPA_HINT						0
#define	HTPX64XXQ_DSPB_HINT						1
#define	HTPX64XXQ_DSPC_HINT						2
#define	HTPX64XXQ_DSPD_HINT						3

	// these IDs for HPI#n error shall be used as 'N' parameter for universal GET_HPI_ERR and CLR_HPI_ERR macros
#define	HTPX64XXQ_DSPA_HPI_ERR					0
#define	HTPX64XXQ_DSPB_HPI_ERR					1
#define	HTPX64XXQ_DSPC_HPI_ERR					2
#define	HTPX64XXQ_DSPD_HPI_ERR					3


// these IDs for HIRQ shall be used as the 'N' parameter for accessing HIRQ SET/GET macros
#define	HTPX64XXQ_HIRQ0							0
#define	HTPX64XXQ_HIRQ1							1
#define	HTPX64XXQ_HIRQ2							2
#define	HTPX64XXQ_HIRQ3							3
//==========================================================================================



//====================== macros for access to PIOX Control Registers =======================
//
//	Host TP64xxQ utilities provide unified utility macros/functions synopsis:
//	  	- HTPX64XXQ_set_..._rg(..)    - sets the control register contents
//		- HTPX64XXQ_get_..._rg()      - returns contents of control register
//  	- HTPX64XXQ_set_...(..)       - sets bit of control register (using particular bit set value)
//	  	- HTPX64XXQ_get_...()         - returns bit state of control register (using particular bit set value)
//  	- HTPX64XXQ_uset_...(ON/OFF)  - sets bit of control register using unified ON/OFF values (may be not applicable for all bits)
//	  	- HTPX64XXQ_uget_...()        - returns bit state of control register using unified ON/OFF values (may be not applicable for all bits)
//
// 	Notes:
// 	------
// 		None.


	// macros for access to PIOX CONTROL#1 register
	// --------------------------------------------
#define HTPX64XXQ_get_cntr1_rg()				((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_CNTR1_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)
#define HTPX64XXQ_set_cntr1_rg(v)				(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_CNTR1_RG_ADDR) = ((__HTPX64XXQ_CNTR_RG_DATA_TYPE__)(v))

		// dedicated bits macros
#define HTPX64XXQ_get_dspa_reset()              (HTPX64XXQ_get_cntr1_rg() & HTPX64XXQ_DSPA_RESET_BITMASK)
#define HTPX64XXQ_set_dspa_reset(v)             HTPX64XXQ_set_cntr1_rg((HTPX64XXQ_get_cntr1_rg() & ~HTPX64XXQ_DSPA_RESET_BITMASK) | ((v) & HTPX64XXQ_DSPA_RESET_BITMASK))

#define HTPX64XXQ_get_dspb_reset()              (HTPX64XXQ_get_cntr1_rg() & HTPX64XXQ_DSPB_RESET_BITMASK)
#define HTPX64XXQ_set_dspb_reset(v)             HTPX64XXQ_set_cntr1_rg((HTPX64XXQ_get_cntr1_rg() & ~HTPX64XXQ_DSPB_RESET_BITMASK) | ((v) & HTPX64XXQ_DSPB_RESET_BITMASK))

#define HTPX64XXQ_get_dspc_reset()              (HTPX64XXQ_get_cntr1_rg() & HTPX64XXQ_DSPC_RESET_BITMASK)
#define HTPX64XXQ_set_dspc_reset(v)             HTPX64XXQ_set_cntr1_rg((HTPX64XXQ_get_cntr1_rg() & ~HTPX64XXQ_DSPC_RESET_BITMASK) | ((v) & HTPX64XXQ_DSPC_RESET_BITMASK))

#define HTPX64XXQ_get_dspd_reset()              (HTPX64XXQ_get_cntr1_rg() & HTPX64XXQ_DSPD_RESET_BITMASK)
#define HTPX64XXQ_set_dspd_reset(v)             HTPX64XXQ_set_cntr1_rg((HTPX64XXQ_get_cntr1_rg() & ~HTPX64XXQ_DSPD_RESET_BITMASK) | ((v) & HTPX64XXQ_DSPD_RESET_BITMASK))
			// aliases
#define HTPX64XXQ_dspa_reset()              	HTPX64XXQ_get_dspa_reset()
#define HTPX64XXQ_dspb_reset()              	HTPX64XXQ_get_dspb_reset()
#define HTPX64XXQ_dspc_reset()              	HTPX64XXQ_get_dspc_reset()
#define HTPX64XXQ_dspd_reset()              	HTPX64XXQ_get_dspd_reset()

		// direct bit set macros
#define HTPX64XXQ_SET_DSPA_RESET_OFF			HTPX64XXQ_set_dspa_reset(HTPX64XXQ_DSPA_RESET_OFF)
#define HTPX64XXQ_SET_DSPA_RESET_ON				HTPX64XXQ_set_dspa_reset(HTPX64XXQ_DSPA_RESET_ON)

#define HTPX64XXQ_SET_DSPB_RESET_OFF			HTPX64XXQ_set_dspb_reset(HTPX64XXQ_DSPB_RESET_OFF)
#define HTPX64XXQ_SET_DSPB_RESET_ON				HTPX64XXQ_set_dspb_reset(HTPX64XXQ_DSPB_RESET_ON)

#define HTPX64XXQ_SET_DSPC_RESET_OFF			HTPX64XXQ_set_dspc_reset(HTPX64XXQ_DSPC_RESET_OFF)
#define HTPX64XXQ_SET_DSPC_RESET_ON				HTPX64XXQ_set_dspc_reset(HTPX64XXQ_DSPC_RESET_ON)

#define HTPX64XXQ_SET_DSPD_RESET_OFF			HTPX64XXQ_set_dspd_reset(HTPX64XXQ_DSPD_RESET_OFF)
#define HTPX64XXQ_SET_DSPD_RESET_ON				HTPX64XXQ_set_dspd_reset(HTPX64XXQ_DSPD_RESET_ON)

#define HTPX64XXQ_SET_DSP_RESET_ON_ALL			HTPX64XXQ_set_cntr1_rg(HTPX64XXQ_DSP_RESET_ON_ALL)
#define HTPX64XXQ_SET_DSP_RESET_OFF_ALL			HTPX64XXQ_set_cntr1_rg(HTPX64XXQ_DSP_RESET_OFF_ALL)

		// conditional macros (for use in IF() and other conditional operators)
#define HTPX64XXQ_DSPA_RESET_IS_OFF				(HTPX64XXQ_get_dspa_reset()==HTPX64XXQ_DSPA_RESET_OFF)
#define HTPX64XXQ_DSPA_RESET_IS_ON				(HTPX64XXQ_get_dspa_reset()==HTPX64XXQ_DSPA_RESET_ON)

#define HTPX64XXQ_DSPB_RESET_IS_OFF				(HTPX64XXQ_get_dspb_reset()==HTPX64XXQ_DSPB_RESET_OFF)
#define HTPX64XXQ_DSPB_RESET_IS_ON				(HTPX64XXQ_get_dspb_reset()==HTPX64XXQ_DSPB_RESET_ON)

#define HTPX64XXQ_DSPC_RESET_IS_OFF				(HTPX64XXQ_get_dspc_reset()==HTPX64XXQ_DSPC_RESET_OFF)
#define HTPX64XXQ_DSPC_RESET_IS_ON				(HTPX64XXQ_get_dspc_reset()==HTPX64XXQ_DSPC_RESET_ON)

#define HTPX64XXQ_DSPD_RESET_IS_OFF				(HTPX64XXQ_get_dspd_reset()==HTPX64XXQ_DSPD_RESET_OFF)
#define HTPX64XXQ_DSPD_RESET_IS_ON				(HTPX64XXQ_get_dspd_reset()==HTPX64XXQ_DSPD_RESET_ON)

		// unified bit macros with V={HTPX64XXQ_DSP_RESET_ON/OFF}
#define HTPX64XXQ_uget_dspa_reset()				HTPX64XXQ_get_dspa_reset()
#define HTPX64XXQ_uset_dspa_reset(v)			HTPX64XXQ_set_dspa_reset(v)

#define HTPX64XXQ_uget_dspb_reset()				(HTPX64XXQ_get_dspb_reset() >> HTPX64XXQ_DSPB_RESET_BITSHIFT)
#define HTPX64XXQ_uset_dspb_reset(v)			HTPX64XXQ_set_dspb_reset((v) << HTPX64XXQ_DSPB_RESET_BITSHIFT)

#define HTPX64XXQ_uget_dspc_reset()				(HTPX64XXQ_get_dspc_reset() >> HTPX64XXQ_DSPC_RESET_BITSHIFT)
#define HTPX64XXQ_uset_dspc_reset(v)			HTPX64XXQ_set_dspc_reset((v) << HTPX64XXQ_DSPC_RESET_BITSHIFT)

#define HTPX64XXQ_uget_dspd_reset()				(HTPX64XXQ_get_dspd_reset() >> HTPX64XXQ_DSPD_RESET_BITSHIFT)
#define HTPX64XXQ_uset_dspd_reset(v)			HTPX64XXQ_set_dspd_reset((v) << HTPX64XXQ_DSPD_RESET_BITSHIFT)

		// universal DSP RESET control macros with parameters DSP#={HTPX64XXQ_DSPA..DSPD} and  DSP RESET value V={HTPX64XXQ_DSP_RESET_ON/OFF}
#define HTPX64XXQ_uget_dsp_reset(n)				((HTPX64XXQ_get_cntr1_rg() >> ((n)&0x3)) &1)
#define HTPX64XXQ_uset_dsp_reset(n,v)			HTPX64XXQ_set_cntr1_rg((HTPX64XXQ_get_cntr1_rg() & ~(1<<((n)&0x3))) | (((v)&1) << ((n)&0x3)))
			// aliases
#define HTPX64XXQ_dsp_reset(n)					HTPX64XXQ_uget_dsp_reset(n)

			// universal direct set/get macros
#define HTPX64XXQ_SET_DSP_RESET_OFF(n)          HTPX64XXQ_uset_dsp_reset(n,HTPX64XXQ_DSP_RESET_OFF)
#define HTPX64XXQ_SET_DSP_RESET_ON(n)          	HTPX64XXQ_uset_dsp_reset(n,HTPX64XXQ_DSP_RESET_ON)

#define HTPX64XXQ_DSP_RESET_IS_OFF(n)          	(HTPX64XXQ_uget_dsp_reset(n)==HTPX64XXQ_DSP_RESET_OFF)
#define HTPX64XXQ_DSP_RESET_IS_ON(n)          	(HTPX64XXQ_uget_dsp_reset(n)==HTPX64XXQ_DSP_RESET_ON)



	// macros for access to PIOX CONTROL#2 register
	// --------------------------------------------
#define HTPX64XXQ_get_cntr2_rg()				((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_CNTR2_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)
#define HTPX64XXQ_set_cntr2_rg(v)				(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_CNTR2_RG_ADDR) = ((__HTPX64XXQ_CNTR_RG_DATA_TYPE__)(v))

		// dedicated bits macros
#define HTPX64XXQ_get_dsp_bmode_cntr()         	(HTPX64XXQ_get_cntr2_rg() & HTPX64XXQ_DSP_BMODE_CNTR_BITMASK)
#define HTPX64XXQ_set_dsp_bmode_cntr(v)        	HTPX64XXQ_set_cntr2_rg((HTPX64XXQ_get_cntr2_rg() & ~HTPX64XXQ_DSP_BMODE_CNTR_BITMASK) | ((v) & HTPX64XXQ_DSP_BMODE_CNTR_BITMASK) )

#define HTPX64XXQ_get_dsp_hpi_fmt_cntr()       	(HTPX64XXQ_get_cntr2_rg() & HTPX64XXQ_DSP_HPI_FMT_CNTR_BITMASK)
#define HTPX64XXQ_set_dsp_hpi_fmt_cntr(v)		HTPX64XXQ_set_cntr2_rg((HTPX64XXQ_get_cntr2_rg() & ~HTPX64XXQ_DSP_HPI_FMT_CNTR_BITMASK) | ((v) & HTPX64XXQ_DSP_HPI_FMT_CNTR_BITMASK) )

#define HTPX64XXQ_get_px_fmt()            		(HTPX64XXQ_get_cntr2_rg() & HTPX64XXQ_PX_FMT_BITMASK)

#define HTPX64XXQ_get_dsp_hpi_err_en()          (HTPX64XXQ_get_cntr2_rg() & HTPX64XXQ_DSP_HPI_ERR_EN_BITMASK)
#define HTPX64XXQ_set_dsp_hpi_err_en(v)			HTPX64XXQ_set_cntr2_rg((HTPX64XXQ_get_cntr2_rg() & ~HTPX64XXQ_DSP_HPI_ERR_EN_BITMASK) | ((v) & HTPX64XXQ_DSP_HPI_ERR_EN_BITMASK) )
			// aliases
#define HTPX64XXQ_dsp_bmode_cntr()				HTPX64XXQ_get_dsp_bmode_cntr()
#define HTPX64XXQ_dsp_hpi_fmt_cntr()			HTPX64XXQ_get_dsp_hpi_fmt_cntr()
#define HTPX64XXQ_px_fmt()						HTPX64XXQ_get_px_fmt()
#define HTPX64XXQ_dsp_hpi_err_en()				HTPX64XXQ_get_dsp_hpi_err_en()

		// direct bit set macros
#define HTPX64XXQ_SET_DSP_BMODE_CNTR_NONE		HTPX64XXQ_set_dsp_bmode_cntr(HTPX64XXQ_DSP_BMODE_CNTR_NONE)
#define HTPX64XXQ_SET_DSP_BMODE_CNTR_HPI		HTPX64XXQ_set_dsp_bmode_cntr(HTPX64XXQ_DSP_BMODE_CNTR_HPI)

#define HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI16	HTPX64XXQ_set_dsp_hpi_fmt_cntr(HTPX64XXQ_DSP_HPI_FMT_CNTR_HPI16)
#define HTPX64XXQ_SET_DSP_HPI_FMT_CNTR_HPI32	HTPX64XXQ_set_dsp_hpi_fmt_cntr(HTPX64XXQ_DSP_HPI_FMT_CNTR_HPI32)

#define HTPX64XXQ_SET_DSP_HPI_ERR_EN_OFF		HTPX64XXQ_set_dsp_hpi_err_en(HTPX64XXQ_DSP_HPI_ERR_EN_OFF)
#define HTPX64XXQ_SET_DSP_HPI_ERR_EN_ON			HTPX64XXQ_set_dsp_hpi_err_en(HTPX64XXQ_DSP_HPI_ERR_EN_ON)

		// conditional macros (for use in IF() and other conditional operators)
#define HTPX64XXQ_DSP_BMODE_CNTR_IS_NONE		(HTPX64XXQ_get_dsp_bmode_cntr()==HTPX64XXQ_DSP_BMODE_CNTR_NONE)
#define HTPX64XXQ_DSP_BMODE_CNTR_IS_HPI			(HTPX64XXQ_get_dsp_bmode_cntr()==HTPX64XXQ_DSP_BMODE_CNTR_HPI)

#define HTPX64XXQ_DSP_HPI_FMT_CNTR_IS_HPI16		(HTPX64XXQ_get_dsp_hpi_fmt_cntr()==HTPX64XXQ_DSP_HPI_FMT_CNTR_HPI16)
#define HTPX64XXQ_DSP_HPI_FMT_CNTR_IS_HPI32		(HTPX64XXQ_get_dsp_hpi_fmt_cntr()==HTPX64XXQ_DSP_HPI_FMT_CNTR_HPI32)

#define HTPX64XXQ_PX_FMT_IS_PX16				(HTPX64XXQ_get_px_fmt()==HTPX64XXQ_PX_FMT_PX16)
#define HTPX64XXQ_PX_FMT_IS_PX32				(HTPX64XXQ_get_px_fmt()==HTPX64XXQ_PX_FMT_PX32)

#define HTPX64XXQ_DSP_HPI_ERR_EN_IS_OFF			(HTPX64XXQ_get_dsp_hpi_err_en()==HTPX64XXQ_DSP_HPI_ERR_EN_OFF)
#define HTPX64XXQ_DSP_HPI_ERR_EN_IS_ON			(HTPX64XXQ_get_dsp_hpi_err_en()==HTPX64XXQ_DSP_HPI_ERR_EN_ON)

		// unified bit macros for HPI ERROR ENABLE with V = {HTPX64XXQ_ON/OFF}
#define HTPX64XXQ_uget_dsp_hpi_err_en()			(HTPX64XXQ_get_dsp_hpi_err_en() >> HTPX64XXQ_DSP_HPI_ERR_EN_BITSHIFT)
#define HTPX64XXQ_uset_dsp_hpi_err_en(v)		HTPX64XXQ_set_dsp_hpi_err_en((v) << HTPX64XXQ_DSP_HPI_ERR_EN_BITSHIFT)



	// macros for access to PIOX DSP CONFIGURATION STATUS register (read-only)
	// -----------------------------------------------------------------------
#define HTPX64XXQ_get_dsp_cnf_stat_rg()			((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_DSP_CNF_STAT_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)

		// dedicated bits macros
#define HTPX64XXQ_get_dspa_bmode_stat()         (HTPX64XXQ_get_dsp_cnf_stat_rg() & HTPX64XXQ_DSPA_BMODE_STAT_BITMASK)
#define HTPX64XXQ_get_dspb_bmode_stat()         (HTPX64XXQ_get_dsp_cnf_stat_rg() & HTPX64XXQ_DSPB_BMODE_STAT_BITMASK)
#define HTPX64XXQ_get_dspc_bmode_stat()         (HTPX64XXQ_get_dsp_cnf_stat_rg() & HTPX64XXQ_DSPC_BMODE_STAT_BITMASK)
#define HTPX64XXQ_get_dspd_bmode_stat()         (HTPX64XXQ_get_dsp_cnf_stat_rg() & HTPX64XXQ_DSPD_BMODE_STAT_BITMASK)

#define HTPX64XXQ_get_dspa_hpi_fmt_stat()       (HTPX64XXQ_get_dsp_cnf_stat_rg() & HTPX64XXQ_DSPA_HPI_FMT_STAT_BITMASK)
#define HTPX64XXQ_get_dspb_hpi_fmt_stat()       (HTPX64XXQ_get_dsp_cnf_stat_rg() & HTPX64XXQ_DSPB_HPI_FMT_STAT_BITMASK)
#define HTPX64XXQ_get_dspc_hpi_fmt_stat()       (HTPX64XXQ_get_dsp_cnf_stat_rg() & HTPX64XXQ_DSPC_HPI_FMT_STAT_BITMASK)
#define HTPX64XXQ_get_dspd_hpi_fmt_stat()       (HTPX64XXQ_get_dsp_cnf_stat_rg() & HTPX64XXQ_DSPD_HPI_FMT_STAT_BITMASK)

			//aliases
#define HTPX64XXQ_dspa_bmode_stat()				HTPX64XXQ_get_dspa_bmode_stat()
#define HTPX64XXQ_dspb_bmode_stat()				HTPX64XXQ_get_dspb_bmode_stat()
#define HTPX64XXQ_dspc_bmode_stat()				HTPX64XXQ_get_dspc_bmode_stat()
#define HTPX64XXQ_dspd_bmode_stat()				HTPX64XXQ_get_dspd_bmode_stat()

#define HTPX64XXQ_dspa_hpi_fmt_stat()			HTPX64XXQ_get_dspa_hpi_fmt_stat()
#define HTPX64XXQ_dspb_hpi_fmt_stat()			HTPX64XXQ_get_dspb_hpi_fmt_stat()
#define HTPX64XXQ_dspc_hpi_fmt_stat()			HTPX64XXQ_get_dspc_hpi_fmt_stat()
#define HTPX64XXQ_dspd_hpi_fmt_stat()			HTPX64XXQ_get_dspd_hpi_fmt_stat()


		// conditional macros (for use in IF() and other conditional operators)
#define HTPX64XXQ_DSPA_BMODE_STAT_IS_NONE		(HTPX64XXQ_get_dspa_bmode_stat()==HTPX64XXQ_DSPA_BMODE_STAT_NONE)
#define HTPX64XXQ_DSPA_BMODE_STAT_IS_HPI		(HTPX64XXQ_get_dspa_bmode_stat()==HTPX64XXQ_DSPA_BMODE_STAT_HPI)

#define HTPX64XXQ_DSPB_BMODE_STAT_IS_NONE		(HTPX64XXQ_get_dspb_bmode_stat()==HTPX64XXQ_DSPB_BMODE_STAT_NONE)
#define HTPX64XXQ_DSPB_BMODE_STAT_IS_HPI		(HTPX64XXQ_get_dspb_bmode_stat()==HTPX64XXQ_DSPB_BMODE_STAT_HPI)

#define HTPX64XXQ_DSPC_BMODE_STAT_IS_NONE		(HTPX64XXQ_get_dspc_bmode_stat()==HTPX64XXQ_DSPC_BMODE_STAT_NONE)
#define HTPX64XXQ_DSPC_BMODE_STAT_IS_HPI		(HTPX64XXQ_get_dspc_bmode_stat()==HTPX64XXQ_DSPC_BMODE_STAT_HPI)

#define HTPX64XXQ_DSPD_BMODE_STAT_IS_NONE		(HTPX64XXQ_get_dspd_bmode_stat()==HTPX64XXQ_DSPD_BMODE_STAT_NONE)
#define HTPX64XXQ_DSPD_BMODE_STAT_IS_HPI		(HTPX64XXQ_get_dspd_bmode_stat()==HTPX64XXQ_DSPD_BMODE_STAT_HPI)

#define HTPX64XXQ_DSPA_HPI_FMT_STAT_IS_HPI16	(HTPX64XXQ_get_dspa_hpi_fmt_stat()==HTPX64XXQ_DSPA_HPI_FMT_STAT_HPI16)
#define HTPX64XXQ_DSPA_HPI_FMT_STAT_IS_HPI32	(HTPX64XXQ_get_dspa_hpi_fmt_stat()==HTPX64XXQ_DSPA_HPI_FMT_STAT_HPI32)

#define HTPX64XXQ_DSPB_HPI_FMT_STAT_IS_HPI16	(HTPX64XXQ_get_dspb_hpi_fmt_stat()==HTPX64XXQ_DSPB_HPI_FMT_STAT_HPI16)
#define HTPX64XXQ_DSPB_HPI_FMT_STAT_IS_HPI32	(HTPX64XXQ_get_dspb_hpi_fmt_stat()==HTPX64XXQ_DSPB_HPI_FMT_STAT_HPI32)

#define HTPX64XXQ_DSPC_HPI_FMT_STAT_IS_HPI16	(HTPX64XXQ_get_dspc_hpi_fmt_stat()==HTPX64XXQ_DSPC_HPI_FMT_STAT_HPI16)
#define HTPX64XXQ_DSPC_HPI_FMT_STAT_IS_HPI32	(HTPX64XXQ_get_dspc_hpi_fmt_stat()==HTPX64XXQ_DSPC_HPI_FMT_STAT_HPI32)

#define HTPX64XXQ_DSPD_HPI_FMT_STAT_IS_HPI16	(HTPX64XXQ_get_dspd_hpi_fmt_stat()==HTPX64XXQ_DSPD_HPI_FMT_STAT_HPI16)
#define HTPX64XXQ_DSPD_HPI_FMT_STAT_IS_HPI32	(HTPX64XXQ_get_dspd_hpi_fmt_stat()==HTPX64XXQ_DSPD_HPI_FMT_STAT_HPI32)


		// unified bit macros with V={HTPX64XXQ_DSP_BMODE_STAT_NONE/HPI}
#define HTPX64XXQ_uget_dspa_bmode_stat()		HTPX64XXQ_get_dspa_bmode_stat()
#define HTPX64XXQ_uget_dspb_bmode_stat()		(HTPX64XXQ_get_dspb_bmode_stat() >> HTPX64XXQ_DSPB_BMODE_STAT_BITSHIFT)
#define HTPX64XXQ_uget_dspc_bmode_stat()		(HTPX64XXQ_get_dspc_bmode_stat() >> HTPX64XXQ_DSPC_BMODE_STAT_BITSHIFT)
#define HTPX64XXQ_uget_dspd_bmode_stat()		(HTPX64XXQ_get_dspd_bmode_stat() >> HTPX64XXQ_DSPD_BMODE_STAT_BITSHIFT)

		// unified bit macros with V={HTPX64XXQ_DSP_HPI_FMT_STAT_HPI16/HPI32}
#define HTPX64XXQ_uget_dspa_hpi_fmt_stat()		(HTPX64XXQ_get_dspa_hpi_fmt_stat() >> HTPX64XXQ_DSPA_HPI_FMT_STAT_BITSHIFT)
#define HTPX64XXQ_uget_dspb_hpi_fmt_stat()		(HTPX64XXQ_get_dspb_hpi_fmt_stat() >> HTPX64XXQ_DSPB_HPI_FMT_STAT_BITSHIFT)
#define HTPX64XXQ_uget_dspc_hpi_fmt_stat()		(HTPX64XXQ_get_dspc_hpi_fmt_stat() >> HTPX64XXQ_DSPC_HPI_FMT_STAT_BITSHIFT)
#define HTPX64XXQ_uget_dspd_hpi_fmt_stat()		(HTPX64XXQ_get_dspd_hpi_fmt_stat() >> HTPX64XXQ_DSPD_HPI_FMT_STAT_BITSHIFT)


		// universal DSP BMODE return macros with parameters DSP#={HTPX64XXQ_DSPA..DSPD} and  BMODE value V={HTPX64XXQ_DSP_BMODE_STAT_NONE/HPI}
#define HTPX64XXQ_uget_dsp_bmode_stat(n)		((HTPX64XXQ_get_dsp_cnf_stat_rg() >> ((n)&0x3) ) & 1)
			//alias
#define HTPX64XXQ_dsp_bmode_stat(n)				HTPX64XXQ_uget_dsp_bmode_stat(n)

			// universal conditional macros (for use in IF() and other conditional operators)
#define HTPX64XXQ_DSP_BMODE_STAT_IS_NONE(n)		(HTPX64XXQ_uget_dsp_bmode_stat(n)==HTPX64XXQ_DSP_BMODE_STAT_NONE)
#define HTPX64XXQ_DSP_BMODE_STAT_IS_HPI(n)		(HTPX64XXQ_uget_dsp_bmode_stat(n)==HTPX64XXQ_DSP_BMODE_STAT_HPI)


		// universal DSP HPI FORMAT return macros with parameters DSP# ={HTPX64XXQ_DSPA..DSPD} and HPI FMT value V={HTPX64XXQ_DSP_HPI_FMT_STAT_HPI16/HPI32}
#define HTPX64XXQ_uget_dsp_hpi_fmt_stat(n)		((HTPX64XXQ_get_dsp_cnf_stat_rg() >> (((n)&0x3)+4) ) & 1)
			//alias
#define HTPX64XXQ_dsp_hpi_fmt_stat(n)			HTPX64XXQ_uget_dsp_hpi_fmt_stat(n)

			// universal conditional macros (for use in IF() and other conditional operators)
#define HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI16(n)	(HTPX64XXQ_uget_dsp_hpi_fmt_stat(n)==HTPX64XXQ_DSP_HPI_FMT_STAT_HPI16)
#define HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI32(n)	(HTPX64XXQ_uget_dsp_hpi_fmt_stat(n)==HTPX64XXQ_DSP_HPI_FMT_STAT_HPI32)



	// macros for access to PIOX DSP LINKs Enable register
	// ---------------------------------------------------
#define HTPX64XXQ_get_dsp_link_cnf_rg()			((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_DSP_LINK_CNF_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)
#define HTPX64XXQ_set_dsp_link_cnf_rg(v)		(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_DSP_LINK_CNF_RG_ADDR) = ((__HTPX64XXQ_CNTR_RG_DATA_TYPE__)(v))

		// dedicated bits macros
#define HTPX64XXQ_get_dspac_link_en()			(HTPX64XXQ_get_dsp_link_cnf_rg() & HTPX64XXQ_DSPAC_LINK_EN_BITMASK)
#define HTPX64XXQ_set_dspac_link_en(v)			HTPX64XXQ_set_dsp_link_cnf_rg((HTPX64XXQ_get_dsp_link_cnf_rg() & ~HTPX64XXQ_DSPAC_LINK_EN_BITMASK) | ((v) & HTPX64XXQ_DSPAC_LINK_EN_BITMASK))

#define HTPX64XXQ_get_dspbd_link_en()			(HTPX64XXQ_get_dsp_link_cnf_rg() & HTPX64XXQ_DSPBD_LINK_EN_BITMASK)
#define HTPX64XXQ_set_dspbd_link_en(v)			HTPX64XXQ_set_dsp_link_cnf_rg((HTPX64XXQ_get_dsp_link_cnf_rg() & ~HTPX64XXQ_DSPBD_LINK_EN_BITMASK) | ((v) & HTPX64XXQ_DSPBD_LINK_EN_BITMASK))

#define HTPX64XXQ_get_dspad_link_en()			(HTPX64XXQ_get_dsp_link_cnf_rg() & HTPX64XXQ_DSPAD_LINK_EN_BITMASK)
#define HTPX64XXQ_set_dspad_link_en(v)			HTPX64XXQ_set_dsp_link_cnf_rg((HTPX64XXQ_get_dsp_link_cnf_rg() & ~HTPX64XXQ_DSPAD_LINK_EN_BITMASK) | ((v) & HTPX64XXQ_DSPAD_LINK_EN_BITMASK))

#define HTPX64XXQ_get_dspbc_link_en()			(HTPX64XXQ_get_dsp_link_cnf_rg() & HTPX64XXQ_DSPBC_LINK_EN_BITMASK)
#define HTPX64XXQ_set_dspbc_link_en(v)			HTPX64XXQ_set_dsp_link_cnf_rg((HTPX64XXQ_get_dsp_link_cnf_rg() & ~HTPX64XXQ_DSPBC_LINK_EN_BITMASK) | ((v) & HTPX64XXQ_DSPBC_LINK_EN_BITMASK))

			//aliases
#define HTPX64XXQ_dspac_link_en()				HTPX64XXQ_get_dspac_link_en()
#define HTPX64XXQ_dspbd_link_en()				HTPX64XXQ_get_dspbd_link_en()
#define HTPX64XXQ_dspad_link_en()				HTPX64XXQ_get_dspad_link_en()
#define HTPX64XXQ_dspbc_link_en()				HTPX64XXQ_get_dspbc_link_en()

		// direct bit set macros
#define HTPX64XXQ_SET_DSPAC_LINK_EN_OFF			HTPX64XXQ_set_dspac_link_en(HTPX64XXQ_DSPAC_LINK_EN_OFF)
#define HTPX64XXQ_SET_DSPAC_LINK_EN_ON			HTPX64XXQ_set_dspac_link_en(HTPX64XXQ_DSPAC_LINK_EN_ON)

#define HTPX64XXQ_SET_DSPBD_LINK_EN_OFF			HTPX64XXQ_set_dspbd_link_en(HTPX64XXQ_DSPBD_LINK_EN_OFF)
#define HTPX64XXQ_SET_DSPBD_LINK_EN_ON			HTPX64XXQ_set_dspbd_link_en(HTPX64XXQ_DSPBD_LINK_EN_ON)

#define HTPX64XXQ_SET_DSPAD_LINK_EN_OFF			HTPX64XXQ_set_dspad_link_en(HTPX64XXQ_DSPAD_LINK_EN_OFF)
#define HTPX64XXQ_SET_DSPAD_LINK_EN_ON			HTPX64XXQ_set_dspad_link_en(HTPX64XXQ_DSPAD_LINK_EN_ON)

#define HTPX64XXQ_SET_DSPBC_LINK_EN_OFF			HTPX64XXQ_set_dspbc_link_en(HTPX64XXQ_DSPBC_LINK_EN_OFF)
#define HTPX64XXQ_SET_DSPBC_LINK_EN_ON			HTPX64XXQ_set_dspbc_link_en(HTPX64XXQ_DSPBC_LINK_EN_ON)

		// conditional macros (for use in IF() and other conditional operators)
#define HTPX64XXQ_DSPAC_LINK_EN_IS_OFF			(HTPX64XXQ_get_dspac_link_en()==HTPX64XXQ_DSPAC_LINK_EN_OFF)
#define HTPX64XXQ_DSPAC_LINK_EN_IS_ON			(HTPX64XXQ_get_dspac_link_en()==HTPX64XXQ_DSPAC_LINK_EN_ON)

#define HTPX64XXQ_DSPBD_LINK_EN_IS_OFF			(HTPX64XXQ_get_dspbd_link_en()==HTPX64XXQ_DSPBD_LINK_EN_OFF)
#define HTPX64XXQ_DSPBD_LINK_EN_IS_ON			(HTPX64XXQ_get_dspbd_link_en()==HTPX64XXQ_DSPBD_LINK_EN_ON)

#define HTPX64XXQ_DSPAD_LINK_EN_IS_OFF			(HTPX64XXQ_get_dspad_link_en()==HTPX64XXQ_DSPAD_LINK_EN_OFF)
#define HTPX64XXQ_DSPAD_LINK_EN_IS_ON			(HTPX64XXQ_get_dspad_link_en()==HTPX64XXQ_DSPAD_LINK_EN_ON)

#define HTPX64XXQ_DSPBC_LINK_EN_IS_OFF			(HTPX64XXQ_get_dspbc_link_en()==HTPX64XXQ_DSPBC_LINK_EN_OFF)
#define HTPX64XXQ_DSPBC_LINK_EN_IS_ON			(HTPX64XXQ_get_dspbc_link_en()==HTPX64XXQ_DSPBC_LINK_EN_ON)

		// unified bit macros with V={HTPX64XXQ_ON/OFF}
#define HTPX64XXQ_uget_dspac_link_en()			HTPX64XXQ_get_dspac_link_en()
#define HTPX64XXQ_uset_dspac_link_en(v)			HTPX64XXQ_set_dspac_link_en(v)

#define HTPX64XXQ_uget_dspbd_link_en()			(HTPX64XXQ_get_dspbd_link_en() >> HTPX64XXQ_DSPBD_LINK_EN_BITSHIFT)
#define HTPX64XXQ_uset_dspbd_link_en(v)			HTPX64XXQ_set_dspbd_link_en((v) <<HTPX64XXQ_DSPBD_LINK_EN_BITSHIFT)

#define HTPX64XXQ_uget_dspad_link_en()			(HTPX64XXQ_get_dspad_link_en() >> HTPX64XXQ_DSPAD_LINK_EN_BITSHIFT)
#define HTPX64XXQ_uset_dspad_link_en(v)			HTPX64XXQ_set_dspad_link_en((v) <<HTPX64XXQ_DSPAD_LINK_EN_BITSHIFT)

#define HTPX64XXQ_uget_dspbc_link_en()			(HTPX64XXQ_get_dspbc_link_en() >> HTPX64XXQ_DSPBC_LINK_EN_BITSHIFT)
#define HTPX64XXQ_uset_dspbc_link_en(v)			HTPX64XXQ_set_dspbc_link_en((v) <<HTPX64XXQ_DSPBC_LINK_EN_BITSHIFT)



	// macros for access to PIOX Device ID register (read-only)
	// ---------------------------------------------------------
#define HTPX64XXQ_get_dev_id_rg()				((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_DEV_ID_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)

		// dedicated bit specific macros
#define HTPX64XXQ_get_dev_id()					(HTPX64XXQ_get_dev_id_rg() & HTPX64XXQ_DEV_ID_BITMASK)
#define HTPX64XXQ_get_rev_id()					(HTPX64XXQ_get_dev_id_rg() & HTPX64XXQ_REV_ID_BITMASK)
#define HTPX64XXQ_get_dsp_speed_id()			(HTPX64XXQ_get_dev_id_rg() & HTPX64XXQ_DSP_SPEED_ID_BITMASK)
			// aliases
#define HTPX64XXQ_dev_id()						HTPX64XXQ_get_dev_id()
#define HTPX64XXQ_rev_id()						HTPX64XXQ_get_rev_id()
#define HTPX64XXQ_dsp_speed_id()				HTPX64XXQ_get_dsp_speed_id()

		// conditional macros (for use in IF() and other conditional operators)
		// (a variety of board ID combinations are provided in order to detect boards family)
#define HTPX6414Q_BOARD							(HTPX64XXQ_get_dev_id()==HTPX6414Q_DEV_ID)
#define HTPX6415Q_BOARD							(HTPX64XXQ_get_dev_id()==HTPX6415Q_DEV_ID)
#define HTPX6416Q_BOARD							(HTPX64XXQ_get_dev_id()==HTPX6416Q_DEV_ID)

#define HTPX6414Q_HTPX6415Q_BOARD				(HTXP6414Q_BOARD || HTPX6415Q_BOARD)
#define HTPX64XXQ_BOARD							(HTPX6414Q_BOARD || HTPX6415Q_BOARD || HTPX6416Q_BOARD)

#define HTPX64XXQ_REV_ID_IS_REV1				(HTPX64XXQ_get_rev_id()==HTPX64XXQ_REV_ID_REV1)

#define HTPX64XXQ_DSP_SPEED_ID_IS_600MHZ		(HTPX64XXQ_get_dsp_speed_id()==HTPX64XXQ_DSP_SPEED_ID_600MHZ)
#define HTPX64XXQ_DSP_SPEED_ID_IS_720MHZ		(HTPX64XXQ_get_dsp_speed_id()==HTPX64XXQ_DSP_SPEED_ID_720MHZ)



	// macros for access to DSP external memory (SSRAM/SDRAM) length ID register (read-only)
	// --------------------------------------------------------------------------------------
#define HTPX64XXQ_get_dsp_xmem_len_id_rg()		((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_DSP_XMEM_LEN_ID_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)

		// dedicated bit specific macros
#define HTPX64XXQ_get_dsp_ssram_len_id()		(HTPX64XXQ_get_dsp_xmem_len_id_rg() & HTPX64XXQ_DSP_SSRAM_LEN_ID_BITMASK)
#define HTPX64XXQ_get_dsp_sdram_len_id()		(HTPX64XXQ_get_dsp_xmem_len_id_rg() & HTPX64XXQ_DSP_SDRAM_LEN_ID_BITMASK)
			// aliases
#define TPX64XXQ_dsp_ssram_len_id()				HTPX64XXQ_get_dsp_ssram_len_id()
#define TPX64XXQ_dsp_sdram_len_id()				HTPX64XXQ_get_dsp_sdram_len_id()

		// condition check macros (for use in IF() and other conditional operators) for SSRAM/SDRAM length in x32 words
#define HTPX64XXQ_DSP_SSRAM_LEN_IS_0K			(HTPX64XXQ_get_dsp_ssram_len_id()==HTPX64XXQ_DSP_SSRAM_LEN_ID_0K)
#define HTPX64XXQ_DSP_SSRAM_LEN_IS_32K			(HTPX64XXQ_get_dsp_ssram_len_id()==HTPX64XXQ_DSP_SSRAM_LEN_ID_32K)
#define HTPX64XXQ_DSP_SSRAM_LEN_IS_64K			(HTPX64XXQ_get_dsp_ssram_len_id()==HTPX64XXQ_DSP_SSRAM_LEN_ID_64K)
#define HTPX64XXQ_DSP_SSRAM_IS_INSTALLED		(HTPX64XXQ_DSP_SSRAM_LEN_IS_32K || HTPX64XXQ_DSP_SSRAM_LEN_IS_64K)
#define HTPX64XXQ_DSP_SSRAM_IS_NOT_INSTALLED	!HTPX64XXQ_DSP_SSRAM_IS_INSTALLED

#define HTPX64XXQ_DSP_SDRAM_LEN_IS_0M			(HTPX64XXQ_get_dsp_sdram_len_id()==HTPX64XXQ_DSP_SDRAM_LEN_ID_0M)
#define HTPX64XXQ_DSP_SDRAM_LEN_IS_4M			(HTPX64XXQ_get_dsp_sdram_len_id()==HTPX64XXQ_DSP_SDRAM_LEN_ID_4M)
#define HTPX64XXQ_DSP_SDRAM_LEN_IS_16M			(HTPX64XXQ_get_dsp_sdram_len_id()==HTPX64XXQ_DSP_SDRAM_LEN_ID_16M)
#define HTPX64XXQ_DSP_SDRAM_IS_INSTALLED		(HTPX64XXQ_DSP_SDRAM_LEN_IS_4M || HTPX64XXQ_DSP_SDRAM_LEN_IS_16M)
#define HTPX64XXQ_DSP_SDRAM_IS_NOT_INSTALLED	!HTPX64XXQ_DSP_SDRAM_IS_INSTALLED

			// aliases for length in bytes
#define HTPX64XXQ_DSP_SSRAM_LEN_IS_0KB			HTPX64XXQ_DSP_SSRAM_LEN_IS_0K
#define HTPX64XXQ_DSP_SSRAM_LEN_IS_128KB		HTPX64XXQ_DSP_SSRAM_LEN_IS_32K
#define HTPX64XXQ_DSP_SSRAM_LEN_IS_256KB		HTPX64XXQ_DSP_SSRAM_LEN_IS_64K

#define HTPX64XXQ_DSP_SDRAM_LEN_IS_0MB			HTPX64XXQ_DSP_SDRAM_LEN_IS_0M
#define HTPX64XXQ_DSP_SDRAM_LEN_IS_16MB			HTPX64XXQ_DSP_SDRAM_LEN_IS_4M
#define HTPX64XXQ_DSP_SDRAM_LEN_IS_64MB			HTPX64XXQ_DSP_SDRAM_LEN_IS_16M

		// macros returning true SSRAM/SDRAM length (in bytes)
#define HTPX64XXQ_get_dsp_ssram_len()     		(HTPX64XXQ_DSP_SSRAM_LEN_IS_0K ? HTPX64XXQ_DSPE_SSRAM_LEN_0K:\
													(HTPX64XXQ_DSP_SSRAM_LEN_IS_32K ? HTPX64XXQ_DSPE_SSRAM_LEN_32K:\
														HTPX64XXQ_DSPE_SSRAM_LEN_64K ))

#define HTPX64XXQ_get_dsp_sdram_len()     		(HTPX64XXQ_DSP_SDRAM_LEN_IS_0M ? HTPX64XXQ_DSPE_SDRAM_LEN_0M:\
													(HTPX64XXQ_DSP_SDRAM_LEN_IS_4M ? HTPX64XXQ_DSPE_SDRAM_LEN_4M:\
														HTPX64XXQ_DSPE_SDRAM_LEN_16M ))
			//aliases
#define HTPX64XXQ_dsp_ssram_len()				HTPX64XXQ_get_dsp_ssram_len()
#define HTPX64XXQ_dsp_sdram_len()				HTPX64XXQ_get_dsp_sdram_len()




	// macros for access to PIOX INTERRUPT STATUS register (read-only)
	// ---------------------------------------------------------------
#define HTPX64XXQ_get_int_stat_rg()				((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_INT_STAT_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)

		// dedicated bits macros
#define HTPX64XXQ_get_dspa_hint()         		(HTPX64XXQ_get_int_stat_rg() & HTPX64XXQ_DSPA_HINT_BITMASK)
#define HTPX64XXQ_get_dspb_hint()         		(HTPX64XXQ_get_int_stat_rg() & HTPX64XXQ_DSPB_HINT_BITMASK)
#define HTPX64XXQ_get_dspc_hint()         		(HTPX64XXQ_get_int_stat_rg() & HTPX64XXQ_DSPC_HINT_BITMASK)
#define HTPX64XXQ_get_dspd_hint()         		(HTPX64XXQ_get_int_stat_rg() & HTPX64XXQ_DSPD_HINT_BITMASK)

#define HTPX64XXQ_get_dspa_hpi_err()       		(HTPX64XXQ_get_int_stat_rg() & HTPX64XXQ_DSPA_HPI_ERR_BITMASK)
#define HTPX64XXQ_get_dspb_hpi_err()       		(HTPX64XXQ_get_int_stat_rg() & HTPX64XXQ_DSPB_HPI_ERR_BITMASK)
#define HTPX64XXQ_get_dspc_hpi_err()       		(HTPX64XXQ_get_int_stat_rg() & HTPX64XXQ_DSPC_HPI_ERR_BITMASK)
#define HTPX64XXQ_get_dspd_hpi_err()       		(HTPX64XXQ_get_int_stat_rg() & HTPX64XXQ_DSPD_HPI_ERR_BITMASK)

			//aliases
#define HTPX64XXQ_dspa_hint()					HTPX64XXQ_get_dspa_hint()
#define HTPX64XXQ_dspb_hint()					HTPX64XXQ_get_dspb_hint()
#define HTPX64XXQ_dspc_hint()					HTPX64XXQ_get_dspc_hint()
#define HTPX64XXQ_dspd_hint()					HTPX64XXQ_get_dspd_hint()

#define HTPX64XXQ_dspa_hpi_err()				HTPX64XXQ_get_dspa_hpi_err()
#define HTPX64XXQ_dspb_hpi_err()				HTPX64XXQ_get_dspb_hpi_err()
#define HTPX64XXQ_dspc_hpi_err()				HTPX64XXQ_get_dspc_hpi_err()
#define HTPX64XXQ_dspd_hpi_err()				HTPX64XXQ_get_dspd_hpi_err()


		// conditional macros (for use in IF() and other conditional operators)
#define HTPX64XXQ_DSPA_HINT_IS_OFF				(HTPX64XXQ_get_dspa_hint()==HTPX64XXQ_DSPA_HINT_OFF)
#define HTPX64XXQ_DSPA_HINT_IS_ON				(HTPX64XXQ_get_dspa_hint()==HTPX64XXQ_DSPA_HINT_ON)

#define HTPX64XXQ_DSPB_HINT_IS_OFF				(HTPX64XXQ_get_dspb_hint()==HTPX64XXQ_DSPB_HINT_OFF)
#define HTPX64XXQ_DSPB_HINT_IS_ON				(HTPX64XXQ_get_dspb_hint()==HTPX64XXQ_DSPB_HINT_ON)

#define HTPX64XXQ_DSPC_HINT_IS_OFF				(HTPX64XXQ_get_dspc_hint()==HTPX64XXQ_DSPC_HINT_OFF)
#define HTPX64XXQ_DSPC_HINT_IS_ON				(HTPX64XXQ_get_dspc_hint()==HTPX64XXQ_DSPC_HINT_ON)

#define HTPX64XXQ_DSPD_HINT_IS_OFF				(HTPX64XXQ_get_dspd_hint()==HTPX64XXQ_DSPD_HINT_OFF)
#define HTPX64XXQ_DSPD_HINT_IS_ON				(HTPX64XXQ_get_dspd_hint()==HTPX64XXQ_DSPD_HINT_ON)

#define HTPX64XXQ_DSPA_HPI_ERR_IS_OFF			(HTPX64XXQ_get_dspa_hpi_err()==HTPX64XXQ_DSPA_HPI_ERR_OFF)
#define HTPX64XXQ_DSPA_HPI_ERR_IS_ON			(HTPX64XXQ_get_dspa_hpi_err()==HTPX64XXQ_DSPA_HPI_ERR_ON)

#define HTPX64XXQ_DSPB_HPI_ERR_IS_OFF			(HTPX64XXQ_get_dspb_hpi_err()==HTPX64XXQ_DSPB_HPI_ERR_OFF)
#define HTPX64XXQ_DSPB_HPI_ERR_IS_ON			(HTPX64XXQ_get_dspb_hpi_err()==HTPX64XXQ_DSPB_HPI_ERR_ON)

#define HTPX64XXQ_DSPC_HPI_ERR_IS_OFF			(HTPX64XXQ_get_dspc_hpi_err()==HTPX64XXQ_DSPC_HPI_ERR_OFF)
#define HTPX64XXQ_DSPC_HPI_ERR_IS_ON			(HTPX64XXQ_get_dspc_hpi_err()==HTPX64XXQ_DSPC_HPI_ERR_ON)

#define HTPX64XXQ_DSPD_HPI_ERR_IS_OFF			(HTPX64XXQ_get_dspd_hpi_err()==HTPX64XXQ_DSPD_HPI_ERR_OFF)
#define HTPX64XXQ_DSPD_HPI_ERR_IS_ON			(HTPX64XXQ_get_dspd_hpi_err()==HTPX64XXQ_DSPD_HPI_ERR_ON)

		// unified bit macros with V={HTPX64XXQ_ON/OFF}
#define HTPX64XXQ_uget_dspa_hint()				HTPX64XXQ_get_dspa_hint()
#define HTPX64XXQ_uget_dspb_hint()				(HTPX64XXQ_get_dspb_hint() >> HTPX64XXQ_DSPB_HINT_BITSHIFT)
#define HTPX64XXQ_uget_dspc_hint()				(HTPX64XXQ_get_dspc_hint() >> HTPX64XXQ_DSPC_HINT_BITSHIFT)
#define HTPX64XXQ_uget_dspd_hint()				(HTPX64XXQ_get_dspd_hint() >> HTPX64XXQ_DSPD_HINT_BITSHIFT)

#define HTPX64XXQ_uget_dspa_hpi_err()			(HTPX64XXQ_get_dspa_hpi_err() >> HTPX64XXQ_DSPA_HPI_ERR_BITSHIFT)
#define HTPX64XXQ_uget_dspb_hpi_err()			(HTPX64XXQ_get_dspb_hpi_err() >> HTPX64XXQ_DSPB_HPI_ERR_BITSHIFT)
#define HTPX64XXQ_uget_dspc_hpi_err()			(HTPX64XXQ_get_dspc_hpi_err() >> HTPX64XXQ_DSPC_HPI_ERR_BITSHIFT)
#define HTPX64XXQ_uget_dspd_hpi_err()			(HTPX64XXQ_get_dspd_hpi_err() >> HTPX64XXQ_DSPD_HPI_ERR_BITSHIFT)

		// universal HPI HINT return macros with parameters DSP#={HTPX64XXQ_DSPA..DSPD} and  value V={HTPX64XXQ_ON/OFF}
#define HTPX64XXQ_uget_dsp_hint(n)				((HTPX64XXQ_get_int_stat_rg() >> ((n)&0x3) ) & 1)
			// alias
#define HTPX64XXQ_dsp_hint(n)					HTPX64XXQ_uget_dsp_hint(n)

			// universal direct conditional macros
#define HTPX64XXQ_DSP_HINT_IS_OFF(n)			(HTPX64XXQ_uget_dsp_hint(n)==HTPX64XXQ_OFF)
#define HTPX64XXQ_DSP_HINT_IS_ON(n)				(HTPX64XXQ_uget_dsp_hint(n)==HTPX64XXQ_ON)


		// universal HPI ERROR return macros with parameters DSP#={HTPX64XXQ_DSPA..DSPD} and  value V={HTPX64XXQ_ON/OFF}
#define HTPX64XXQ_uget_dsp_hpi_err(n)			((HTPX64XXQ_get_int_stat_rg() >> (((n)&0x3)+4) ) & 1)
			// alias
#define HTPX64XXQ_dsp_hpi_err(n)				HTPX64XXQ_uget_dsp_hpi_err(n)

			// universal direct conditional macros
#define HTPX64XXQ_DSP_HPI_ERR_IS_OFF(n)			(HTPX64XXQ_uget_dsp_hpi_err(n)==HTPX64XXQ_OFF)
#define HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)			(HTPX64XXQ_uget_dsp_hpi_err(n)==HTPX64XXQ_ON)


	// macros for HPI ERRORs clear
	// ----------------------------
#define HTPX64XXQ_clr_dspa_hpi_err()			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_CLR_DSPA_HPI_ERR_ADDR) = 0
#define HTPX64XXQ_clr_dspb_hpi_err()			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_CLR_DSPB_HPI_ERR_ADDR) = 0
#define HTPX64XXQ_clr_dspc_hpi_err()			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_CLR_DSPC_HPI_ERR_ADDR) = 0
#define HTPX64XXQ_clr_dspd_hpi_err()			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_CLR_DSPD_HPI_ERR_ADDR) = 0

		// direct macros
#define HTPX64XXQ_CLR_DSPA_HPI_ERR				HTPX64XXQ_clr_dspa_hpi_err()
#define HTPX64XXQ_CLR_DSPB_HPI_ERR				HTPX64XXQ_clr_dspb_hpi_err()
#define HTPX64XXQ_CLR_DSPC_HPI_ERR				HTPX64XXQ_clr_dspc_hpi_err()
#define HTPX64XXQ_CLR_DSPD_HPI_ERR				HTPX64XXQ_clr_dspd_hpi_err()

		// macros to clear all HPI errors
#define HTPX64XXQ_CLR_DSP_HPI_ERR_ALL			{HTPX64XXQ_CLR_DSPA_HPI_ERR;HTPX64XXQ_CLR_DSPB_HPI_ERR;HTPX64XXQ_CLR_DSPC_HPI_ERR;HTPX64XXQ_CLR_DSPD_HPI_ERR;}

		// universal HPI ERROR clear macros with parameter DSP#={HTPX64XXQ_DSPA..DSPD}
#define HTPX64XXQ_clr_dsp_hpi_err(n)			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) (HTPX64XXQ_CLR_DSPA_HPI_ERR_ADDR+((n&0x3)*_WAS_))) = 0
			// direct macros
#define HTPX64XXQ_CLR_DSP_HPI_ERR(n)			HTPX64XXQ_clr_dsp_hpi_err(n)



	// macros for DSP BROADCAST INTERRUPT generation
	// ----------------------------------------------
#define HTPX64XXQ_set_dsp_broadcast_irq()		(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_DSP_BROADCAST_IRQ_ADDR) = 0

			// direct macros
#define HTPX64XXQ_SET_DSP_BROADCAST_IRQ			HTPX64XXQ_set_dsp_broadcast_irq()




	// universal macros for access to HIRQ-0..3 registers with parameter HIRQ# N={HTPX64XXQ_HIRQ0..3}
	// -----------------------------------------------------------------------------------------------
#define HTPX64XXQ_get_hirq_sel_rg(n)			((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) (HTPX64XXQ_HIRQ0_SEL_RG_ADDR+ (((n)&0x3)*_WAS_))) & HTPX64XXQ_PX_RG_DATA_BITMASK)
#define HTPX64XXQ_set_hirq_sel_rg(n,v)			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) (HTPX64XXQ_HIRQ0_SEL_RG_ADDR+(((n)&0x3)*_WAS_))) = ((__HTPX64XXQ_CNTR_RG_DATA_TYPE__)(v))

		// universal dedicated bit macros with parameter HIRQ# N={HTPX64XXQ_HIRQ0..3}
#define HTPX64XXQ_get_hirq_sel(n)				(HTPX64XXQ_get_hirq_sel_rg(n) & HTPX64XXQ_HIRQ_SEL_BITMASK)
#define HTPX64XXQ_get_hirq_en(n)				(HTPX64XXQ_get_hirq_sel_rg(n) & HTPX64XXQ_HIRQ_EN_BITMASK)
#define HTPX64XXQ_set_hirq_sel(n,v)				HTPX64XXQ_set_hirq_sel_rg(n,(HTPX64XXQ_get_hirq_sel_rg(n) & ~HTPX64XXQ_HIRQ_SEL_BITMASK) | ((v) & HTPX64XXQ_HIRQ_SEL_BITMASK))
#define HTPX64XXQ_set_hirq_en(n,v)				HTPX64XXQ_set_hirq_sel_rg(n,(HTPX64XXQ_get_hirq_sel_rg(n) & ~HTPX64XXQ_HIRQ_EN_BITMASK) | ((v) & HTPX64XXQ_HIRQ_EN_BITMASK))
			// aliases
#define HTPX64XXQ_hirq_sel(n)					HTPX64XXQ_get_hirq_sel(n)
#define HTPX64XXQ_hirq_en(n)					HTPX64XXQ_get_hirq_en(n)

		// universal direct bit set macros with parameter HIRQ# N={HTPX64XXQ_HIRQ0..3}
#define HTPX64XXQ_SET_HIRQ_SEL_OFF(n)			HTPX64XXQ_set_hirq_sel(n,HTPX64XXQ_HIRQ_SEL_OFF)
#define HTPX64XXQ_SET_HIRQ_SEL_DSP_HPI_ERR(n)	HTPX64XXQ_set_hirq_sel(n,HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_SET_HIRQ_SEL_DSPA_HINT(n)		HTPX64XXQ_set_hirq_sel(n,HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_SET_HIRQ_SEL_DSPB_HINT(n)		HTPX64XXQ_set_hirq_sel(n,HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_SET_HIRQ_SEL_DSPC_HINT(n)		HTPX64XXQ_set_hirq_sel(n,HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_SET_HIRQ_SEL_DSPD_HINT(n)		HTPX64XXQ_set_hirq_sel(n,HTPX64XXQ_HIRQ_SEL_DSPD_HINT)
			// universal macros with selection of HINT of particular DSP #k (k=HTPX64XXQ_DSPA..DSPD_HINT)
#define HTPX64XXQ_SET_HIRQ_SEL_DSP_HINT(n,k)	HTPX64XXQ_set_hirq_sel(n,(HTPX64XXQ_HIRQ_SEL_DSPA_HINT+(k%4)))

#define HTPX64XXQ_SET_HIRQ_EN_OFF(n)			HTPX64XXQ_set_hirq_en(n,HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_SET_HIRQ_EN_ON(n)				HTPX64XXQ_set_hirq_en(n,HTPX64XXQ_HIRQ_EN_ON)

#define HTPX64XXQ_DISABLE_HIRQ(n)				HTPX64XXQ_set_hirq_sel_rg(n,0)
#define HTPX64XXQ_DISABLE_HIRQ_ALL				{HTPX64XXQ_set_hirq_sel_rg(0,0);HTPX64XXQ_set_hirq_sel_rg(1,0);HTPX64XXQ_set_hirq_sel_rg(2,0);HTPX64XXQ_set_hirq_sel_rg(3,0);}



		// universal conditional macros with parameter HIRQ# N={HTPX64XXQ_HIRQ0..3} (for use in IF() and other conditional operators)
#define HTPX64XXQ_HIRQ_SEL_IS_OFF(n)			((HTPX64XXQ_get_hirq_sel(n)==HTPX64XXQ_HIRQ_SEL_OFF) || (HTPX64XXQ_get_hirq_sel(n)==HTPX64XXQ_HIRQ_SEL_RSRV1) || (HTPX64XXQ_get_hirq_sel(n)==HTPX64XXQ_HIRQ_SEL_RSRV2))
#define HTPX64XXQ_HIRQ_SEL_IS_DSP_HPI_ERR(n)	(HTPX64XXQ_get_hirq_sel(n)==HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_HIRQ_SEL_IS_DSPA_HINT(n)		(HTPX64XXQ_get_hirq_sel(n)==HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_HIRQ_SEL_IS_DSPB_HINT(n)		(HTPX64XXQ_get_hirq_sel(n)==HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_HIRQ_SEL_IS_DSPC_HINT(n)		(HTPX64XXQ_get_hirq_sel(n)==HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_HIRQ_SEL_IS_DSPD_HINT(n)		(HTPX64XXQ_get_hirq_sel(n)==HTPX64XXQ_HIRQ_SEL_DSPD_HINT)

#define HTPX64XXQ_HIRQ_SEL_IS_DSP_HINT(n,k)		(HTPX64XXQ_get_hirq_sel(n)==(HTPX64XXQ_HIRQ_SEL_DSPA_HINT+(k%4)))

#define HTPX64XXQ_HIRQ_EN_IS_OFF(n)				(HTPX64XXQ_get_hirq_en(n)==HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_HIRQ_EN_IS_ON(n)				(HTPX64XXQ_get_hirq_en(n)==HTPX64XXQ_HIRQ_EN_ON)

		// universal unified HIRQ-0..3 ENABLE macros with parameter HIRQ# N={HTPX64XXQ_HIRQ0..3}  and with V = {HTPX64XXQ_ON/OFF}
#define HTPX64XXQ_uget_hirq_en(n)				(HTPX64XXQ_get_hirq_en(n) >> HTPX64XXQ_HIRQ_EN_BITSHIFT)
#define HTPX64XXQ_uset_hirq_en(n,v)				HTPX64XXQ_set_hirq_en((n,v) << HTPX64XXQ_HIRQ_EN_BITSHIFT)



	// macros for access to HIRQ-0..3 SELECTOR registers
	// --------------------------------------------------
#define HTPX64XXQ_get_hirq0_sel_rg()			((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_HIRQ0_SEL_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)
#define HTPX64XXQ_set_hirq0_sel_rg(v)			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_HIRQ0_SEL_RG_ADDR) = ((__HTPX64XXQ_CNTR_RG_DATA_TYPE__)(v))

#define HTPX64XXQ_get_hirq1_sel_rg()			((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_HIRQ1_SEL_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)
#define HTPX64XXQ_set_hirq1_sel_rg(v)			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_HIRQ1_SEL_RG_ADDR) = ((__HTPX64XXQ_CNTR_RG_DATA_TYPE__)(v))

#define HTPX64XXQ_get_hirq2_sel_rg()			((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_HIRQ2_SEL_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)
#define HTPX64XXQ_set_hirq2_sel_rg(v)			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_HIRQ2_SEL_RG_ADDR) = ((__HTPX64XXQ_CNTR_RG_DATA_TYPE__)(v))

#define HTPX64XXQ_get_hirq3_sel_rg()			((*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_HIRQ3_SEL_RG_ADDR) & HTPX64XXQ_PX_RG_DATA_BITMASK)
#define HTPX64XXQ_set_hirq3_sel_rg(v)			(*(__HTPX64XXQ_CNTR_RG_DATA_TYPE__ *) HTPX64XXQ_HIRQ3_SEL_RG_ADDR) = ((__HTPX64XXQ_CNTR_RG_DATA_TYPE__)(v))

		// dedicated bits macros
#define HTPX64XXQ_get_hirq0_sel()				(HTPX64XXQ_get_hirq0_sel_rg() & HTPX64XXQ_HIRQ_SEL_BITMASK)
#define HTPX64XXQ_get_hirq0_en()				(HTPX64XXQ_get_hirq0_sel_rg() & HTPX64XXQ_HIRQ_EN_BITMASK)
#define HTPX64XXQ_set_hirq0_sel(v)				HTPX64XXQ_set_hirq0_sel_rg((HTPX64XXQ_get_hirq0_sel_rg() & ~HTPX64XXQ_HIRQ_SEL_BITMASK) | ((v) & HTPX64XXQ_HIRQ_SEL_BITMASK))
#define HTPX64XXQ_set_hirq0_en(v)				HTPX64XXQ_set_hirq0_sel_rg((HTPX64XXQ_get_hirq0_sel_rg() & ~HTPX64XXQ_HIRQ_EN_BITMASK) | ((v) & HTPX64XXQ_HIRQ_EN_BITMASK))

#define HTPX64XXQ_get_hirq1_sel()				(HTPX64XXQ_get_hirq1_sel_rg() & HTPX64XXQ_HIRQ_SEL_BITMASK)
#define HTPX64XXQ_get_hirq1_en()				(HTPX64XXQ_get_hirq1_sel_rg() & HTPX64XXQ_HIRQ_EN_BITMASK)
#define HTPX64XXQ_set_hirq1_sel(v)				HTPX64XXQ_set_hirq1_sel_rg((HTPX64XXQ_get_hirq1_sel_rg() & ~HTPX64XXQ_HIRQ_SEL_BITMASK) | ((v) & HTPX64XXQ_HIRQ_SEL_BITMASK))
#define HTPX64XXQ_set_hirq1_en(v)				HTPX64XXQ_set_hirq1_sel_rg((HTPX64XXQ_get_hirq1_sel_rg() & ~HTPX64XXQ_HIRQ_EN_BITMASK) | ((v) & HTPX64XXQ_HIRQ_EN_BITMASK))

#define HTPX64XXQ_get_hirq2_sel()				(HTPX64XXQ_get_hirq2_sel_rg() & HTPX64XXQ_HIRQ_SEL_BITMASK)
#define HTPX64XXQ_get_hirq2_en()				(HTPX64XXQ_get_hirq2_sel_rg() & HTPX64XXQ_HIRQ_EN_BITMASK)
#define HTPX64XXQ_set_hirq2_sel(v)				HTPX64XXQ_set_hirq2_sel_rg((HTPX64XXQ_get_hirq2_sel_rg() & ~HTPX64XXQ_HIRQ_SEL_BITMASK) | ((v) & HTPX64XXQ_HIRQ_SEL_BITMASK))
#define HTPX64XXQ_set_hirq2_en(v)				HTPX64XXQ_set_hirq2_sel_rg((HTPX64XXQ_get_hirq2_sel_rg() & ~HTPX64XXQ_HIRQ_EN_BITMASK) | ((v) & HTPX64XXQ_HIRQ_EN_BITMASK))

#define HTPX64XXQ_get_hirq3_sel()				(HTPX64XXQ_get_hirq3_sel_rg() & HTPX64XXQ_HIRQ_SEL_BITMASK)
#define HTPX64XXQ_get_hirq3_en()				(HTPX64XXQ_get_hirq3_sel_rg() & HTPX64XXQ_HIRQ_EN_BITMASK)
#define HTPX64XXQ_set_hirq3_sel(v)				HTPX64XXQ_set_hirq3_sel_rg((HTPX64XXQ_get_hirq3_sel_rg() & ~HTPX64XXQ_HIRQ_SEL_BITMASK) | ((v) & HTPX64XXQ_HIRQ_SEL_BITMASK))
#define HTPX64XXQ_set_hirq3_en(v)				HTPX64XXQ_set_hirq3_sel_rg((HTPX64XXQ_get_hirq3_sel_rg() & ~HTPX64XXQ_HIRQ_EN_BITMASK) | ((v) & HTPX64XXQ_HIRQ_EN_BITMASK))

			//aliases
#define HTPX64XXQ_hirq0_sel()					HTPX64XXQ_get_hirq0_sel()
#define HTPX64XXQ_hirq0_en()					HTPX64XXQ_get_hirq0_en()

#define HTPX64XXQ_hirq1_sel()					HTPX64XXQ_get_hirq1_sel()
#define HTPX64XXQ_hirq1_en()					HTPX64XXQ_get_hirq1_en()

#define HTPX64XXQ_hirq2_sel()					HTPX64XXQ_get_hirq2_sel()
#define HTPX64XXQ_hirq2_en()					HTPX64XXQ_get_hirq2_en()

#define HTPX64XXQ_hirq3_sel()					HTPX64XXQ_get_hirq3_sel()
#define HTPX64XXQ_hirq3_en()					HTPX64XXQ_get_hirq3_en()

		// direct bit set macros
#define HTPX64XXQ_SET_HIRQ0_SEL_OFF				HTPX64XXQ_set_hirq0_sel(HTPX64XXQ_HIRQ_SEL_OFF)
#define HTPX64XXQ_SET_HIRQ0_SEL_DSP_HPI_ERR		HTPX64XXQ_set_hirq0_sel(HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_SET_HIRQ0_SEL_DSPA_HINT		HTPX64XXQ_set_hirq0_sel(HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_SET_HIRQ0_SEL_DSPB_HINT		HTPX64XXQ_set_hirq0_sel(HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_SET_HIRQ0_SEL_DSPC_HINT		HTPX64XXQ_set_hirq0_sel(HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_SET_HIRQ0_SEL_DSPD_HINT		HTPX64XXQ_set_hirq0_sel(HTPX64XXQ_HIRQ_SEL_DSPD_HINT)

#define HTPX64XXQ_SET_HIRQ0_EN_OFF				HTPX64XXQ_set_hirq0_en(HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_SET_HIRQ0_EN_ON				HTPX64XXQ_set_hirq0_en(HTPX64XXQ_HIRQ_EN_ON)

#define HTPX64XXQ_SET_HIRQ1_SEL_OFF				HTPX64XXQ_set_hirq1_sel(HTPX64XXQ_HIRQ_SEL_OFF)
#define HTPX64XXQ_SET_HIRQ1_SEL_DSP_HPI_ERR		HTPX64XXQ_set_hirq1_sel(HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_SET_HIRQ1_SEL_DSPA_HINT		HTPX64XXQ_set_hirq1_sel(HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_SET_HIRQ1_SEL_DSPB_HINT		HTPX64XXQ_set_hirq1_sel(HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_SET_HIRQ1_SEL_DSPC_HINT		HTPX64XXQ_set_hirq1_sel(HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_SET_HIRQ1_SEL_DSPD_HINT		HTPX64XXQ_set_hirq1_sel(HTPX64XXQ_HIRQ_SEL_DSPD_HINT)

#define HTPX64XXQ_SET_HIRQ1_EN_OFF				HTPX64XXQ_set_hirq1_en(HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_SET_HIRQ1_EN_ON				HTPX64XXQ_set_hirq1_en(HTPX64XXQ_HIRQ_EN_ON)

#define HTPX64XXQ_SET_HIRQ2_SEL_OFF				HTPX64XXQ_set_hirq2_sel(HTPX64XXQ_HIRQ_SEL_OFF)
#define HTPX64XXQ_SET_HIRQ2_SEL_DSP_HPI_ERR		HTPX64XXQ_set_hirq2_sel(HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_SET_HIRQ2_SEL_DSPA_HINT		HTPX64XXQ_set_hirq2_sel(HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_SET_HIRQ2_SEL_DSPB_HINT		HTPX64XXQ_set_hirq2_sel(HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_SET_HIRQ2_SEL_DSPC_HINT		HTPX64XXQ_set_hirq2_sel(HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_SET_HIRQ2_SEL_DSPD_HINT		HTPX64XXQ_set_hirq2_sel(HTPX64XXQ_HIRQ_SEL_DSPD_HINT)

#define HTPX64XXQ_SET_HIRQ2_EN_OFF				HTPX64XXQ_set_hirq2_en(HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_SET_HIRQ2_EN_ON				HTPX64XXQ_set_hirq2_en(HTPX64XXQ_HIRQ_EN_ON)

#define HTPX64XXQ_SET_HIRQ3_SEL_OFF				HTPX64XXQ_set_hirq3_sel(HTPX64XXQ_HIRQ_SEL_OFF)
#define HTPX64XXQ_SET_HIRQ3_SEL_DSP_HPI_ERR		HTPX64XXQ_set_hirq3_sel(HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_SET_HIRQ3_SEL_DSPA_HINT		HTPX64XXQ_set_hirq3_sel(HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_SET_HIRQ3_SEL_DSPB_HINT		HTPX64XXQ_set_hirq3_sel(HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_SET_HIRQ3_SEL_DSPC_HINT		HTPX64XXQ_set_hirq3_sel(HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_SET_HIRQ3_SEL_DSPD_HINT		HTPX64XXQ_set_hirq3_sel(HTPX64XXQ_HIRQ_SEL_DSPD_HINT)

#define HTPX64XXQ_SET_HIRQ3_EN_OFF				HTPX64XXQ_set_hirq3_en(HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_SET_HIRQ3_EN_ON				HTPX64XXQ_set_hirq3_en(HTPX64XXQ_HIRQ_EN_ON)


		// conditional macros (for use in IF() and other conditional operators)
#define HTPX64XXQ_HIRQ0_SEL_IS_OFF				((HTPX64XXQ_get_hirq0_sel()==HTPX64XXQ_HIRQ_SEL_OFF) || (HTPX64XXQ_get_hirq0_sel()==HTPX64XXQ_HIRQ_SEL_RSRV1) || (HTPX64XXQ_get_hirq0_sel()==HTPX64XXQ_HIRQ_SEL_RSRV2))
#define HTPX64XXQ_HIRQ0_SEL_IS_DSP_HPI_ERR		(HTPX64XXQ_get_hirq0_sel()==HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_HIRQ0_SEL_IS_DSPA_HINT		(HTPX64XXQ_get_hirq0_sel()==HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_HIRQ0_SEL_IS_DSPB_HINT		(HTPX64XXQ_get_hirq0_sel()==HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_HIRQ0_SEL_IS_DSPC_HINT		(HTPX64XXQ_get_hirq0_sel()==HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_HIRQ0_SEL_IS_DSPD_HINT		(HTPX64XXQ_get_hirq0_sel()==HTPX64XXQ_HIRQ_SEL_DSPD_HINT)

#define HTPX64XXQ_HIRQ0_EN_IS_OFF				(HTPX64XXQ_get_hirq0_en()==HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_HIRQ0_EN_IS_ON				(HTPX64XXQ_get_hirq0_en()==HTPX64XXQ_HIRQ_EN_ON)

#define HTPX64XXQ_HIRQ1_SEL_IS_OFF				((HTPX64XXQ_get_hirq1_sel()==HTPX64XXQ_HIRQ_SEL_OFF) || (HTPX64XXQ_get_hirq1_sel()==HTPX64XXQ_HIRQ_SEL_RSRV1) || (HTPX64XXQ_get_hirq1_sel()==HTPX64XXQ_HIRQ_SEL_RSRV2))
#define HTPX64XXQ_HIRQ1_SEL_IS_DSP_HPI_ERR		(HTPX64XXQ_get_hirq1_sel()==HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_HIRQ1_SEL_IS_DSPA_HINT		(HTPX64XXQ_get_hirq1_sel()==HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_HIRQ1_SEL_IS_DSPB_HINT		(HTPX64XXQ_get_hirq1_sel()==HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_HIRQ1_SEL_IS_DSPC_HINT		(HTPX64XXQ_get_hirq1_sel()==HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_HIRQ1_SEL_IS_DSPD_HINT		(HTPX64XXQ_get_hirq1_sel()==HTPX64XXQ_HIRQ_SEL_DSPD_HINT)

#define HTPX64XXQ_HIRQ1_EN_IS_OFF				(HTPX64XXQ_get_hirq1_en()==HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_HIRQ1_EN_IS_ON				(HTPX64XXQ_get_hirq1_en()==HTPX64XXQ_HIRQ_EN_ON)

#define HTPX64XXQ_HIRQ2_SEL_IS_OFF				((HTPX64XXQ_get_hirq2_sel()==HTPX64XXQ_HIRQ_SEL_OFF) || (HTPX64XXQ_get_hirq2_sel()==HTPX64XXQ_HIRQ_SEL_RSRV1) || (HTPX64XXQ_get_hirq2_sel()==HTPX64XXQ_HIRQ_SEL_RSRV2))
#define HTPX64XXQ_HIRQ2_SEL_IS_DSP_HPI_ERR		(HTPX64XXQ_get_hirq2_sel()==HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_HIRQ2_SEL_IS_DSPA_HINT		(HTPX64XXQ_get_hirq2_sel()==HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_HIRQ2_SEL_IS_DSPB_HINT		(HTPX64XXQ_get_hirq2_sel()==HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_HIRQ2_SEL_IS_DSPC_HINT		(HTPX64XXQ_get_hirq2_sel()==HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_HIRQ2_SEL_IS_DSPD_HINT		(HTPX64XXQ_get_hirq2_sel()==HTPX64XXQ_HIRQ_SEL_DSPD_HINT)

#define HTPX64XXQ_HIRQ2_EN_IS_OFF				(HTPX64XXQ_get_hirq2_en()==HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_HIRQ2_EN_IS_ON				(HTPX64XXQ_get_hirq2_en()==HTPX64XXQ_HIRQ_EN_ON)

#define HTPX64XXQ_HIRQ3_SEL_IS_OFF				((HTPX64XXQ_get_hirq3_sel()==HTPX64XXQ_HIRQ_SEL_OFF) || (HTPX64XXQ_get_hirq3_sel()==HTPX64XXQ_HIRQ_SEL_RSRV1) || (HTPX64XXQ_get_hirq3_sel()==HTPX64XXQ_HIRQ_SEL_RSRV2))
#define HTPX64XXQ_HIRQ3_SEL_IS_DSP_HPI_ERR		(HTPX64XXQ_get_hirq3_sel()==HTPX64XXQ_HIRQ_SEL_DSP_HPI_ERR)
#define HTPX64XXQ_HIRQ3_SEL_IS_DSPA_HINT		(HTPX64XXQ_get_hirq3_sel()==HTPX64XXQ_HIRQ_SEL_DSPA_HINT)
#define HTPX64XXQ_HIRQ3_SEL_IS_DSPB_HINT		(HTPX64XXQ_get_hirq3_sel()==HTPX64XXQ_HIRQ_SEL_DSPB_HINT)
#define HTPX64XXQ_HIRQ3_SEL_IS_DSPC_HINT		(HTPX64XXQ_get_hirq3_sel()==HTPX64XXQ_HIRQ_SEL_DSPC_HINT)
#define HTPX64XXQ_HIRQ3_SEL_IS_DSPD_HINT		(HTPX64XXQ_get_hirq3_sel()==HTPX64XXQ_HIRQ_SEL_DSPD_HINT)

#define HTPX64XXQ_HIRQ3_EN_IS_OFF				(HTPX64XXQ_get_hirq3_en()==HTPX64XXQ_HIRQ_EN_OFF)
#define HTPX64XXQ_HIRQ3_EN_IS_ON				(HTPX64XXQ_get_hirq3_en()==HTPX64XXQ_HIRQ_EN_ON)

		// unified HIRQ-0..3 ENABLE macros with V = {HTPX64XXQ_ON/OFF}
#define HTPX64XXQ_uget_hirq0_en()				(HTPX64XXQ_get_hirq0_en() >> HTPX64XXQ_HIRQ_EN_BITSHIFT)
#define HTPX64XXQ_uset_hirq0_en(v)				HTPX64XXQ_set_hirq0_en((v) << HTPX64XXQ_HIRQ_EN_BITSHIFT)

#define HTPX64XXQ_uget_hirq1_en()				(HTPX64XXQ_get_hirq1_en() >> HTPX64XXQ_HIRQ_EN_BITSHIFT)
#define HTPX64XXQ_uset_hirq1_en(v)				HTPX64XXQ_set_hirq1_en((v) << HTPX64XXQ_HIRQ_EN_BITSHIFT)

#define HTPX64XXQ_uget_hirq2_en()				(HTPX64XXQ_get_hirq2_en() >> HTPX64XXQ_HIRQ_EN_BITSHIFT)
#define HTPX64XXQ_uset_hirq2_en(v)				HTPX64XXQ_set_hirq2_en((v) << HTPX64XXQ_HIRQ_EN_BITSHIFT)

#define HTPX64XXQ_uget_hirq3_en()				(HTPX64XXQ_get_hirq3_en() >> HTPX64XXQ_HIRQ_EN_BITSHIFT)
#define HTPX64XXQ_uset_hirq3_en(v)				HTPX64XXQ_set_hirq3_en((v) << HTPX64XXQ_HIRQ_EN_BITSHIFT)
//==========================================================================================


//=========== Extended API function declarations and error definition ======================
//	TPX64xxQ Extended API functions include functions for access to C64xx DSP on-chip HPI port,
//	PIOX I/F init, DSP program load, and more.
//
//	Extended API function bodies can be included/excluded form user host DSP source code, which is
//	controlled by __HTPX64XXQ_EXCLUDE_AUX_FUNCTIONS__ and __HTPX64XXQ_EXCLUDE_CBSF_PROGRAM_LOADER__
//	run-time compiler keys. Function bodies for extended API functions with detail function
//	description can be found below in this file.
//		__HTPX64XXQ_EXCLUDE_AUX_FUNCTIONS__
//			- if defined in user code (assigned to any value), then the local
//	      	  aux functions for HPI access, HPI init, DSP EMIF init, etc are excluded
//		  	  from the user source code. Default is INCLUDED.
//
//		__HTPX64XXQ_EXCLUDE_CBSF_PROGRAM_LOADER__
//			- if defined in user code (assigned to any value), then the C6xxx C-BSF loader
//		  	  function (HPTX64XXQ_load_dsp_program() is excluded from the user source code.
//		  	  Default is INCLUDED.
//
//
//		IMPORTANT NOTES:
//		---------------
//			1.	If user host DSP code comprises of several separate source code .OBJ modules, which
//				have to be linked together into one DSP application, then function bodies of Extended
//				API functions shall be included into only one primary source code module.
//


// Extended API AUX functions declarations
// ----------------------------------------
//	AUX API function bodies are included into user code in case __HTPX64XXQ_EXCLUDE_AUX_FUNCTIONS__
//	run-time compiler key is not defined prior inclusion of this header file into user code.

   	// functions for C64xx DSP on-chip HPI port access
		// direct HPI register access functions
void		HTPX64XXQ_set_dsp_hpid_rg(unsigned n,unsigned d);
unsigned	HTPX64XXQ_get_dsp_hpid_rg(unsigned n);
void		HTPX64XXQ_set_dsp_hpid_ainc_rg(unsigned n,unsigned d);
unsigned	HTPX64XXQ_get_dsp_hpid_ainc_rg(unsigned n);
void		HTPX64XXQ_set_dsp_hpia_rg(unsigned n,unsigned d);
unsigned	HTPX64XXQ_get_dsp_hpia_rg(unsigned n);
void		HTPX64XXQ_set_dsp_hpic_rg(unsigned n,unsigned d);
unsigned	HTPX64XXQ_get_dsp_hpic_rg(unsigned n);
		// complex HPI port data access function with preset of HPIA register
void		HTPX64XXQ_write_dsp_hpi_data(unsigned n,unsigned a,unsigned d);
unsigned	HTPX64XXQ_read_dsp_hpi_data(unsigned n,unsigned a);
void		HTPX64XXQ_write_dsp_hpi_data_ainc(unsigned n,unsigned a,unsigned d);
unsigned	HTPX64XXQ_read_dsp_hpi_data_ainc(unsigned n,unsigned a);
		// HPI port initialization
void 		HTPX64XXQ_init_dsp_hpi(unsigned n);


	// host PIOX-16 I/F software reset AUX function
int			HTPX64XXQ_init_piox();

	// C64xx DSP on-chip HPI software reset AUX function
	// This function sets DSPn RESET and loads  'B RESET' code at the DSP RESET vector address.
	// After DSP RESET will be released, this will put DSPn to the known state and will allow to
	// use JTAG emulator, which requires that all four DSP cores in JTAG path must be in the RUN state.
int			HTPX64XXQ_load_dsp_default_program(unsigned n);


	// This API function initializes C64xx DSP EMIF-A/B registers and DSP on-chip GPIO registers
	// as it is performed via TPX64XXQ_init_dsp() function from DSP environment. All access to
	// DSP environment via C64xx DSP on-chip HPI port.
int			HTPX64XXQ_init_dsp(unsigned n);


// Extended API function for upload DSP code from host DSP into C64xx DSP environment
// -----------------------------------------------------------------------------------
// 	This function loads C64xx C-BSF program fm data array to C64xx DSP on-chip environments.
//	This function body is included into user code in case __HTPX64XXQ_EXCLUDE_CBSF_PROGRAM_LOADER__
//	run-time compiler key is not defined prior inclusion of this header file into user code.
int			HTPX64XXQ_load_dsp_program(unsigned n, unsigned *p);


// Error codes
// -----------
// 	Error codes are directly returned by some extended API functions.
#define	HTPX64XXQ_OK						0		// no error
#define	HTPX64XXQ_PARAM_ERR					-1		// invalid input parameter
#define	HTPX64XXQ_HW_ERR					-10		// hardware error
#define	HTPX64XXQ_HPI_ERR					-11		// timeout error during access to C64xx DSP HPI port
#define	HTPX64XXQ_CANNOT_ACCESS_HPI_ERR		-12		// C64xx DSP HPI port cannot be accessed (due to DSP is in the RESET state, etc)
#define	HTPX64XXQ_INV_CBSF_DATA_FMT_ERR		-20		// invalid C-BSF data format
//==========================================================================================





//===================== functions/macros for C64xx DSP HPI port access =====================
// 	Access from PIOX/PIOX-16 I/F to C64xx DSP on-chip port is provided via PX64xxQ extended API
// 	AUX functions. This is the recommened solution, since extended API AUX functions perform automatic
// 	adjustment of variable types used for access to HPI registers upon the HPI data format set for
// 	addressed DSP. Extended API AUX functions for HPI access operate with 32-bit I/O data only.
// 	All HPI access functions use DSP# ('N') as an input parameter in order address HPI port of
//	particular DSP. For performance reasons, HPI access functions do not check for whether DSP is
//	in the RESET state and for HPI timeout error. Error conditions check and well as enable/disable
//	of HPI access timeout control must be done by user host DSP application.
//
//	See section EXTENDED API AUX FUNCTIONS below in this file for function's code and
//	detail function description.
//
//	The following is a list of extended API functions for access to C64xx DSP on-chip HPI port:
//  (actual function declarations is provided above in this file, and function bodies are
//	provided below in section 'EXTENDED API AUX FUNCTIONS'):
//
//		direct HPI register access functions:
//			void		HTPX64XXQ_set_dsp_hpid_rg(unsigned n,unsigned d)
//			unsigned	HTPX64XXQ_get_dsp_hpid_rg(unsigned n)
//			void		HTPX64XXQ_set_dsp_hpid_ainc_rg(unsigned n,unsigned d)
//			unsigned	HTPX64XXQ_get_dsp_hpid_ainc_rg(unsigned n)
//			void		HTPX64XXQ_set_dsp_hpia_rg(unsigned n,unsigned d)
//			unsigned	HTPX64XXQ_get_dsp_hpia_rg(unsigned n)
//			void		HTPX64XXQ_set_dsp_hpic_rg(unsigned n,unsigned d)
//			unsigned	HTPX64XXQ_get_dsp_hpic_rg(unsigned n)
//
// 		complex HPI port data access function with preset of HPIA register:
//			void		HTPX64XXQ_write_dsp_hpi_data(unsigned n,unsigned a,unsigned d);
//			unsigned	HTPX64XXQ_read_dsp_hpi_data(unsigned n,unsigned a);
//			void		HTPX64XXQ_write_dsp_hpi_data_ainc(unsigned n,unsigned a,unsigned d);
//			unsigned	HTPX64XXQ_read_dsp_hpi_data_ainc(unsigned n,unsigned a);
//
//		HPI port initialize finction:
//			int 		HTPX64XXQ_init_dsp_hpi(unsigned n)

//	Provided in this section, HPI access macros include function aliases and HPIC register
//	specific bits access.


	// C64xx DSP HPIC register access and specific bits access
	// All HPIC set/get functions operates with 16-it HPIC LSW only with 16-bit MSW being set inside
	// functions as the copy of LSW.
#define	HTPX64XXQ_get_dsp_hpic(n)				HTPX64XXQ_get_dsp_hpic_rg(n)
#define	HTPX64XXQ_dsp_hpic(n)					HTPX64XXQ_get_dsp_hpic_rg(n)
#define	HTPX64XXQ_set_dsp_hpic(n,d)		 		HTPX64XXQ_set_dsp_hpic_rg(n,d)

#define	HTPX64XXQ_GET_DSP_HPIC(n)				HTPX64XXQ_get_dsp_hpic_rg(n)
#define	HTPX64XXQ_DSP_HPIC(n)					HTPX64XXQ_get_dsp_hpic_rg(n)
#define	HTPX64XXQ_SET_DSP_HPIC(n,d)		 		HTPX64XXQ_set_dsp_hpic_rg(n,d)

		// macros for access to specific HPIC register bits
#define	HTPX64XXQ_get_dsp_hpic_hwob(n)			(HTPX64XXQ_get_dsp_hpic_rg(n) & HTPX64XXQ_DSP_HPIC_HWOB_BITMASK)
#define	HTPX64XXQ_set_dsp_hpic_hwob_on(n)		HTPX64XXQ_set_dsp_hpic(n,HTPX64XXQ_DSP_HPIC_HWOB_ON)

#define	HTPX64XXQ_get_dsp_hpic_dspint(n)		(HTPX64XXQ_get_dsp_hpic_rg(n) & HTPX64XXQ_DSP_HPIC_DSPINT_BITMASK)
#define	HTPX64XXQ_set_dsp_hpic_dspint_on(n)		HTPX64XXQ_set_dsp_hpic(n,HTPX64XXQ_DSP_HPIC_WR_DSPINT_ON)

#define	HTPX64XXQ_get_dsp_hpic_hint(n)			(HTPX64XXQ_get_dsp_hpic_rg(n) & HTPX64XXQ_DSP_HPIC_HINT_BITMASK)
#define	HTPX64XXQ_clr_dsp_hpic_hint(n)			HTPX64XXQ_set_dsp_hpic(n,HTPX64XXQ_DSP_HPIC_WR_HINT_OFF)

#define	HTPX64XXQ_get_dsp_hpic_hrdy(n)			(HTPX64XXQ_get_dsp_hpic_rg(n) & HTPX64XXQ_DSP_HPIC_HRDY_BITMASK)

#define	HTPX64XXQ_set_dsp_hpic_fetch_on(n)		HTPX64XXQ_set_dsp_hpic(n,HTPX64XXQ_DSP_HPIC_WR_FETCH_ON)

			// aliases
#define	HTPX64XXQ_dsp_hpic_hwob(n)				HTPX64XXQ_get_dsp_hpic_hwob(n)
#define	HTPX64XXQ_dsp_hpic_dspint(n)			HTPX64XXQ_get_dsp_hpic_dspint(n)
#define	HTPX64XXQ_dsp_hpic_hint(n)				HTPX64XXQ_get_dsp_hpic_hint(n)
#define	HTPX64XXQ_dsp_hpic_hrdy(n)				HTPX64XXQ_get_dsp_hpic_hrdy(n)

			// direct bit set/get macros
#define	HTPX64XXQ_DSP_HPIC_HWOB_IS_OFF(n)		(HTPX64XXQ_get_dsp_hpic_hwob(n)==HTPX64XXQ_DSP_HPIC_HWOB_OFF)
#define	HTPX64XXQ_DSP_HPIC_HWOB_IS_ON(n)		(HTPX64XXQ_get_dsp_hpic_hwob(n)==HTPX64XXQ_DSP_HPIC_HWOB_ON)

#define	HTPX64XXQ_SET_DSP_HPIC_HWOB_ON(n)		HTPX64XXQ_set_dsp_hpic_hwob_on(n)

#define	HTPX64XXQ_DSP_HPIC_DSPINT_IS_OFF(n)		(HTPX64XXQ_get_dsp_hpic_dspint(n)==HTPX64XXQ_DSP_HPIC_DSPINT_OFF)
#define	HTPX64XXQ_DSP_HPIC_DSPINT_IS_ON(n)		(HTPX64XXQ_get_dsp_hpic_dspint(n)==HTPX64XXQ_DSP_HPIC_DSPINT_ON)

#define	HTPX64XXQ_SET_DSP_HPIC_DSPINT_ON(n)		HTPX64XXQ_set_dsp_hpic_dspint_on(n)

#define	HTPX64XXQ_DSP_HPIC_HINT_IS_OFF(n)		(HTPX64XXQ_get_dsp_hpic_hint(n)==HTPX64XXQ_DSP_HPIC_HINT_OFF)
#define	HTPX64XXQ_DSP_HPIC_HINT_IS_ON(n)		(HTPX64XXQ_get_dsp_hpic_hint(n)==HTPX64XXQ_DSP_HPIC_HINT_ON)

#define	HTPX64XXQ_CLR_DSP_HPIC_HINT(n)			HTPX64XXQ_clr_dsp_hpic_hint(n)
#define	HTPX64XXQ_SET_DSP_HPIC_HINT_OFF(n)		HTPX64XXQ_clr_dsp_hpic_hint(n)

#define	HTPX64XXQ_DSP_HPIC_HRDY_IS_OFF(n)		(HTPX64XXQ_get_dsp_hpic_hrdy(n)==HTPX64XXQ_DSP_HPIC_HRDY_OFF)
#define	HTPX64XXQ_DSP_HPIC_HRDY_IS_ON(n)		(HTPX64XXQ_get_dsp_hpic_hrdy(n)==HTPX64XXQ_DSP_HPIC_HRDY_ON)

#define	HTPX64XXQ_SET_DSP_HPIC_FETCH_ON(n)		HTPX64XXQ_set_dsp_hpic_fetch_on(n)


	// C64xx DSP HPIA register access
#define	HTPX64XXQ_get_dsp_hpia(n)				HTPX64XXQ_get_dsp_hpia_rg(n)
#define	HTPX64XXQ_dsp_hpia(n)					HTPX64XXQ_get_dsp_hpia_rg(n)
#define	HTPX64XXQ_set_dsp_hpia(n,d)		 		HTPX64XXQ_set_dsp_hpia_rg(n,d)

#define	HTPX64XXQ_GET_DSP_HPIA(n)				HTPX64XXQ_get_dsp_hpia_rg(n)
#define	HTPX64XXQ_DSP_HPIA(n)					HTPX64XXQ_get_dsp_hpia_rg(n)
#define	HTPX64XXQ_SET_DSP_HPIA(n,d)		 		HTPX64XXQ_set_dsp_hpia_rg(n,d)
#define	HTPX64XXQ_RESET_DSP_HPIA(n)		 		HTPX64XXQ_set_dsp_hpia_rg(n,0)


	// C64xx DSP HPID register access
#define	HTPX64XXQ_get_dsp_hpid(n)				HTPX64XXQ_get_dsp_hpid_rg(n)
#define	HTPX64XXQ_dsp_hpid(n)					HTPX64XXQ_get_dsp_hpid_rg(n)
#define	HTPX64XXQ_set_dsp_hpid(n,d)		 		HTPX64XXQ_set_dsp_hpid_rg(n,d)

#define	HTPX64XXQ_GET_DSP_HPID(n)				HTPX64XXQ_get_dsp_hpid_rg(n)
#define	HTPX64XXQ_DSP_HPID(n)					HTPX64XXQ_get_dsp_hpid_rg(n)
#define	HTPX64XXQ_SET_DSP_HPID(n,d)		 		HTPX64XXQ_set_dsp_hpid_rg(n,d)

	// C64xx DSP HPID-AINC register access
#define	HTPX64XXQ_get_dsp_hpid_ainc(n)			HTPX64XXQ_get_dsp_hpid_ainc_rg(n)
#define	HTPX64XXQ_dsp_hpid_ainc(n)				HTPX64XXQ_get_dsp_hpid_ainc_rg(n)
#define	HTPX64XXQ_set_dsp_hpid_ainc(n,d)		HTPX64XXQ_set_dsp_hpid_ainc_rg(n,d)

#define	HTPX64XXQ_GET_DSP_HPID_AINC(n)			HTPX64XXQ_get_dsp_hpid_ainc_rg(n)
#define	HTPX64XXQ_DSP_HPID_AINC(n)				HTPX64XXQ_get_dsp_hpid_ainc_rg(n)
#define	HTPX64XXQ_SET_DSP_HPID_AINC(n,d)		HTPX64XXQ_set_dsp_hpid_ainc_rg(n,d)

	// complex HPI data access with preset of HPI address register
#define	HTPX64XXQ_WRITE_DSP_HPI_DATA(n,a,d)			HTPX64XXQ_write_dsp_hpi_data(n,a,d)
#define	HTPX64XXQ_READ_DSP_HPI_DATA(n,a)			HTPX64XXQ_read_dsp_hpi_data(n,a)
#define	HTPX64XXQ_WRITE_DSP_HPI_DATA_AINC(n,a,d)	HTPX64XXQ_write_dsp_hpi_data_ainc(n,a,d)
#define	HTPX64XXQ_READ_DSP_HPI_DATA_AINC(N,A)		HTPX64XXQ_read_dsp_hpi_data_ainc(n,a)

	// Initialization of HPI port
#define	HTPX64XXQ_INIT_DSP_HPI(n)				HTPX64XXQ_init_dsp_hpi(n)


	// macros to start DSP exectuion during DSP HPI boot, i.e. release DSP core from the RESET state.
	// This macros just set DSPINT bit of DSP HPIC register.
#define	HTPX64XXQ_START_DSP_AFTER_HPI_BOOT(n)	HTPX64XXQ_set_dsp_hpic_dspint_on(n)
//==========================================================================================




//======================= Extended API AUX functions =======================================
// 	Aux local functions include functions for HPI access, HPI init, PIOX control registers init,
// 	DSP default code load function, DSP EMIF init via HPI port, etc.
//	In order to  exclude mutiple function body appearance, API AUX function bodies shall be
//	excluded from application code for 2nd, 3rd, etc application .OBJ modules via
//	__HTPX64XXQ_EXCLUDE_AUX_FUNCTIONS__ run-time compiler key:
//
//		__HTPX64XXQ_EXCLUDE_AUX_FUNCTIONS__
//			- if defined in user code (assigned to any value), then the local
//	      	  aux functions for HPI access, HPI init, HIF init, etc are excluded
//		  	  from the user source code. Default is INCLUDED.
//

#ifndef __HTPX64XXQ_EXCLUDE_AUX_FUNCTIONS__
//~~~~~~~~~~~~~~~~~~~~~~~~~ Aux local functions ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

//--- HPI access functions
// Access from PIOX/PIOX-16 I/F to C64xx DSP on-chip port is provided via PX64xxQ extended API
// AUX functions. This is the recommened solution, since extended API AUX functions perform automatic
// adjustment of variable types used for access to HPI registers upon the HPI data format set for
// addressed DSP. Extended API AUX functions for HPI access always accept and return 32-bit HPI port data.
// All PX64xQ extended API AUX functions for HPI access use DSP# as an input parameter in order
// address particular HPI port. See section EXTENDED API AUX FUNCTIONS below in this file for
// source code for functions perforing access to C64xx DSP on-chip HPI port.




	// HPI port access functions
	// --------------------------
	//	C64xx DSP HPI port access functions perform access to HPIC, HPIA, HPID and HPID-AINC
	//	registers and coplex functions for access to HPI data with address preset.
	//
	//		Notes:
	//		------
	//			1. 	All functions require DSP# ('N') input parameter	for selection of particular
	//				C64xx DSP.
	//			2.	For HPIA/HPID/HPID-AINC access functions, input/output data value is 32-bit
	//				UNSIGNED, whereas for HPIC access functions, input/output data is 16-bit LSW
	//				only with the MSW being duplication internally for HPIC writes.
	//			3.	It is assumed although responsibility of user host DSP application, that for
	//				HPI16 data format mode, HWOB bit of HPIC register must be set to the '1' state in case
	//				application does not call either HTPX64XXQ_init_dsp_hpi() or HTPX64XXQ_set_dsp_hpic_rg()
	//				API functions.
	//			4.	For performance reasons, HPI access functions do not check for whether DSP is
	//				in the RESET state and for run-time HPI timeout error. Errors check and
	//				enable/disable control for HPI access timeout is the responsibility of user host
	//				DSP application.
	//			5.	For safety reasons, in order to exclude HPI error, input HPI adress parameter is
	//				being modified to clear bits #0/1.


		// local defs for HPI registers for HPI16 and HPI32 data format modes
		// (these declarations and macros shall not be used in user code)
#define	__htp64xxq_hpi_rg_baddr(n)					(HTPX64XXQ_DSPA_HPIC_RG_W32_ADDR+((n&3)*8*_WAS_))
#define	__HTPX64XXQ_HPI32_HPIC_RG_OFFSET			0
#define	__HTPX64XXQ_HPI32_HPIA_RG_OFFSET			_WAS_
#define	__HTPX64XXQ_HPI32_HPID_AINC_RG_OFFSET		(2*_WAS_)
#define	__HTPX64XXQ_HPI32_HPID_RG_OFFSET			(3*_WAS_)

#define	__HTPX64XXQ_HPI16_HPIC_RG_LSW_OFFSET		0
#define	__HTPX64XXQ_HPI16_HPIC_RG_MSW_OFFSET		_WAS_
#define	__HTPX64XXQ_HPI16_HPIA_RG_LSW_OFFSET		(2*_WAS_)
#define	__HTPX64XXQ_HPI16_HPIA_RG_MSW_OFFSET		(3*_WAS_)
#define	__HTPX64XXQ_HPI16_HPID_AINC_RG_LSW_OFFSET	(4*_WAS_)
#define	__HTPX64XXQ_HPI16_HPID_AINC_RG_MSW_OFFSET	(5*_WAS_)
#define	__HTPX64XXQ_HPI16_HPID_RG_LSW_OFFSET		(6*_WAS_)
#define	__HTPX64XXQ_HPI16_HPID_RG_MSW_OFFSET		(7*_WAS_)


#define __HTPX64XXQ_GET_HPI32_HPIC_RG(n)			((*(__HTPX64XXQ_HPI32_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI32_HPIC_RG_OFFSET)))
#define __HTPX64XXQ_SET_HPI32_HPIC_RG(n,v)			(*(__HTPX64XXQ_HPI32_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI32_HPIC_RG_OFFSET)) =  ((__HTPX64XXQ_HPI32_RG_DATA_TYPE__)(v))
#define __HTPX64XXQ_GET_HPI32_HPIA_RG(n)			((*(__HTPX64XXQ_HPI32_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI32_HPIA_RG_OFFSET)))
#define __HTPX64XXQ_SET_HPI32_HPIA_RG(n,v)			(*(__HTPX64XXQ_HPI32_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI32_HPIA_RG_OFFSET)) =  ((__HTPX64XXQ_HPI32_RG_DATA_TYPE__)(v))
#define __HTPX64XXQ_GET_HPI32_HPID_AINC_RG(n)		((*(__HTPX64XXQ_HPI32_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI32_HPID_AINC_RG_OFFSET)))
#define __HTPX64XXQ_SET_HPI32_HPID_AINC_RG(n,v)		(*(__HTPX64XXQ_HPI32_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI32_HPID_AINC_RG_OFFSET)) =  ((__HTPX64XXQ_HPI32_RG_DATA_TYPE__)(v))
#define __HTPX64XXQ_GET_HPI32_HPID_RG(n)			((*(__HTPX64XXQ_HPI32_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI32_HPID_RG_OFFSET)))
#define __HTPX64XXQ_SET_HPI32_HPID_RG(n,v)			(*(__HTPX64XXQ_HPI32_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI32_HPID_RG_OFFSET)) =  ((__HTPX64XXQ_HPI32_RG_DATA_TYPE__)(v))


#define __HTPX64XXQ_GET_HPI16_HPIC_RG_LSW(n)		((*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPIC_RG_LSW_OFFSET)) & HTPX64XXQ_DSP_HPI16_DATA_BITMASK)
#define __HTPX64XXQ_GET_HPI16_HPIC_RG_MSW(n)		((*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPIC_RG_MSW_OFFSET)) & HTPX64XXQ_DSP_HPI16_DATA_BITMASK)
#define __HTPX64XXQ_SET_HPI16_HPIC_RG_LSW(n,v)		(*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPIC_RG_LSW_OFFSET)) =  ((__HTPX64XXQ_HPI16_RG_DATA_TYPE__)(v))
#define __HTPX64XXQ_SET_HPI16_HPIC_RG_MSW(n,v)		(*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPIC_RG_MSW_OFFSET)) =  ((__HTPX64XXQ_HPI16_RG_DATA_TYPE__)(v))

#define __HTPX64XXQ_GET_HPI16_HPIA_RG_LSW(n)		((*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPIA_RG_LSW_OFFSET)) & HTPX64XXQ_DSP_HPI16_DATA_BITMASK)
#define __HTPX64XXQ_GET_HPI16_HPIA_RG_MSW(n)		((*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPIA_RG_MSW_OFFSET)) & HTPX64XXQ_DSP_HPI16_DATA_BITMASK)
#define __HTPX64XXQ_SET_HPI16_HPIA_RG_LSW(n,v)		(*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPIA_RG_LSW_OFFSET)) =  ((__HTPX64XXQ_HPI16_RG_DATA_TYPE__)(v))
#define __HTPX64XXQ_SET_HPI16_HPIA_RG_MSW(n,v)		(*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPIA_RG_MSW_OFFSET)) =  ((__HTPX64XXQ_HPI16_RG_DATA_TYPE__)(v))

#define __HTPX64XXQ_GET_HPI16_HPID_AINC_RG_LSW(n)	((*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPID_AINC_RG_LSW_OFFSET)) & HTPX64XXQ_DSP_HPI16_DATA_BITMASK)
#define __HTPX64XXQ_GET_HPI16_HPID_AINC_RG_MSW(n)	((*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPID_AINC_RG_MSW_OFFSET)) & HTPX64XXQ_DSP_HPI16_DATA_BITMASK)
#define __HTPX64XXQ_SET_HPI16_HPID_AINC_RG_LSW(n,v)	(*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPID_AINC_RG_LSW_OFFSET)) =  ((__HTPX64XXQ_HPI16_RG_DATA_TYPE__)(v))
#define __HTPX64XXQ_SET_HPI16_HPID_AINC_RG_MSW(n,v)	(*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPID_AINC_RG_MSW_OFFSET)) =  ((__HTPX64XXQ_HPI16_RG_DATA_TYPE__)(v))

#define __HTPX64XXQ_GET_HPI16_HPID_RG_LSW(n)		((*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPID_RG_LSW_OFFSET)) & HTPX64XXQ_DSP_HPI16_DATA_BITMASK)
#define __HTPX64XXQ_GET_HPI16_HPID_RG_MSW(n)		((*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPID_RG_MSW_OFFSET)) & HTPX64XXQ_DSP_HPI16_DATA_BITMASK)
#define __HTPX64XXQ_SET_HPI16_HPID_RG_LSW(n,v)		(*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPID_RG_LSW_OFFSET)) =  ((__HTPX64XXQ_HPI16_RG_DATA_TYPE__)(v))
#define __HTPX64XXQ_SET_HPI16_HPID_RG_MSW(n,v)		(*(__HTPX64XXQ_HPI16_RG_DATA_TYPE__ *) (__htp64xxq_hpi_rg_baddr(n)+__HTPX64XXQ_HPI16_HPID_RG_MSW_OFFSET)) =  ((__HTPX64XXQ_HPI16_RG_DATA_TYPE__)(v))


#define	__HTPX64XXQ_DSP_HPI_ADDR_BITMASK			0xfffffffc

		// Sets HPIC register of DSP#n using provided 16-bit LSW D data.
void	HTPX64XXQ_set_dsp_hpic_rg(unsigned n,unsigned d)
{
	// mask-out all except LSW;
    d &= HTPX64XXQ_DSP_HPI16_DATA_BITMASK;

	// for safety reasons, add HWOB=1, which is important for HPI16 accesses
	d |= HTPX64XXQ_DSP_HPIC_HWOB_ON;

	// perform HPI access upon HPI data format for this DSP
	if(HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI16(n))
	{
		// HPI16 data format
		// perform access to 16-bit LSW/MSW
		__HTPX64XXQ_SET_HPI16_HPIC_RG_LSW(n,d);
		__HTPX64XXQ_SET_HPI16_HPIC_RG_MSW(n,d);
	}
	else
	{
		// HPI32 data format
		// access HPI registers using 32-bit data cycles
		__HTPX64XXQ_SET_HPI32_HPIC_RG(n,(((unsigned)d) | (((unsigned)d)<<16)));
	}
}


		// returns LSW of HPIC register contents for DSP#n.
unsigned	HTPX64XXQ_get_dsp_hpic_rg(unsigned n)
{
	unsigned lsw,msw;

	// perform HPI access upon HPI data format for this DSP
	if(HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI16(n))
	{
		// HPI16 data format
		// perform access to 16-bit LSW/MSW
		lsw = (unsigned) __HTPX64XXQ_GET_HPI16_HPIC_RG_LSW(n);
		msw = (unsigned) __HTPX64XXQ_GET_HPI16_HPIC_RG_MSW(n);
		// use (MSW & 0) in return in order to exclude compiler warning message re unsage of MSW
		return( lsw | (msw &0));
	}
	else
	{
		// HPI32 data format
		// access HPI registers using 32-bit data cycles
		return(__HTPX64XXQ_GET_HPI32_HPIC_RG(n) & HTPX64XXQ_DSP_HPI16_DATA_BITMASK);
	}
}



		// Sets HPIA register of DSP#n to the <D> data.
void	HTPX64XXQ_set_dsp_hpia_rg(unsigned n,unsigned d)
{
	// perform HPI access upon HPI data format for this DSP
	if(HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI16(n))
	{
		// HPI16 data format
		// perform access to 16-bit LSW/MSW
		__HTPX64XXQ_SET_HPI16_HPIA_RG_LSW(n,d&HTPX64XXQ_DSP_HPI16_DATA_BITMASK&__HTPX64XXQ_DSP_HPI_ADDR_BITMASK);
		__HTPX64XXQ_SET_HPI16_HPIA_RG_MSW(n,(d>>16)&HTPX64XXQ_DSP_HPI16_DATA_BITMASK);
	}
	else
	{
		// HPI32 data format
		// access HPI registers using 32-bit data cycles
		__HTPX64XXQ_SET_HPI32_HPIA_RG(n,d&__HTPX64XXQ_DSP_HPI_ADDR_BITMASK);
	}
}


		// returns HPIA register contents for DSP#n.
unsigned	HTPX64XXQ_get_dsp_hpia_rg(unsigned n)
{
	unsigned lsw,msw;

	// perform HPI access upon HPI data format for this DSP
	if(HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI16(n))
	{
		// HPI16 data format
		// perform access to 16-bit LSW/MSW
		lsw = (unsigned) __HTPX64XXQ_GET_HPI16_HPIA_RG_LSW(n);
		msw = (unsigned) __HTPX64XXQ_GET_HPI16_HPIA_RG_MSW(n);
		return( lsw | (msw<<16));
	}
	else
	{
		// HPI32 data format
		// access HPI registers using 32-bit data cycles
		return(__HTPX64XXQ_GET_HPI32_HPIA_RG(n));
	}
}




		// Sets HPID register of DSP#n to the <D> data.
void	HTPX64XXQ_set_dsp_hpid_rg(unsigned n,unsigned d)
{
	// perform HPI access upon HPI data format for this DSP
	if(HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI16(n))
	{
		// HPI16 data format
		// perform access to 16-bit LSW/MSW
		__HTPX64XXQ_SET_HPI16_HPID_RG_LSW(n,d&HTPX64XXQ_DSP_HPI16_DATA_BITMASK);
		__HTPX64XXQ_SET_HPI16_HPID_RG_MSW(n,(d>>16)&HTPX64XXQ_DSP_HPI16_DATA_BITMASK);
	}
	else
	{
		// HPI32 data format
		// access HPI registers using 32-bit data cycles
		__HTPX64XXQ_SET_HPI32_HPID_RG(n,d);
	}
}


		// returns HPID register contents for DSP#n.
unsigned	HTPX64XXQ_get_dsp_hpid_rg(unsigned n)
{
	unsigned lsw,msw;

	// perform HPI access upon HPI data format for this DSP
	if(HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI16(n))
	{
		// HPI16 data format
		// perform access to 16-bit LSW/MSW
		lsw = (unsigned) __HTPX64XXQ_GET_HPI16_HPID_RG_LSW(n);
		msw = (unsigned) __HTPX64XXQ_GET_HPI16_HPID_RG_MSW(n);
		return( lsw | (msw<<16));
	}
	else
	{
		// HPI32 data format
		// access HPI registers using 32-bit data cycles
		return(__HTPX64XXQ_GET_HPI32_HPID_RG(n));
	}
}



		// Sets HPID-AINC register of DSP#n to the <D> data.
void	HTPX64XXQ_set_dsp_hpid_ainc_rg(unsigned n,unsigned d)
{
	// perform HPI access upon HPI data format for this DSP
	if(HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI16(n))
	{
		// HPI16 data format
		// perform access to 16-bit LSW/MSW
		__HTPX64XXQ_SET_HPI16_HPID_AINC_RG_LSW(n,d&HTPX64XXQ_DSP_HPI16_DATA_BITMASK);
		__HTPX64XXQ_SET_HPI16_HPID_AINC_RG_MSW(n,(d>>16)&HTPX64XXQ_DSP_HPI16_DATA_BITMASK);
	}
	else
	{
		// HPI32 data format
		// access HPI registers using 32-bit data cycles
		__HTPX64XXQ_SET_HPI32_HPID_AINC_RG(n,d);
	}
}


		// returns HPID-AINC register contents for DSP#n.
unsigned	HTPX64XXQ_get_dsp_hpid_ainc_rg(unsigned n)
{
	unsigned lsw,msw;

	// perform HPI access upon HPI data format for this DSP
	if(HTPX64XXQ_DSP_HPI_FMT_STAT_IS_HPI16(n))
	{
		// HPI16 data format
		// perform access to 16-bit LSW/MSW
		lsw = (unsigned) __HTPX64XXQ_GET_HPI16_HPID_AINC_RG_LSW(n);
		msw = (unsigned) __HTPX64XXQ_GET_HPI16_HPID_AINC_RG_MSW(n);
		return( lsw | (msw<<16));
	}
	else
	{
		// HPI32 data format
		// access HPI registers using 32-bit data cycles
		return(__HTPX64XXQ_GET_HPI32_HPID_AINC_RG(n));
	}
}



		// complex function, which sets HPIA register to <A> and writes data to via HPID.
void	HTPX64XXQ_write_dsp_hpi_data(unsigned n,unsigned a,unsigned d)
{
	HTPX64XXQ_set_dsp_hpia_rg(n,a&__HTPX64XXQ_DSP_HPI_ADDR_BITMASK);
	HTPX64XXQ_set_dsp_hpid_rg(n,d);
}


		// complex function, which sets HPIA register to <A> and returns HPID data
unsigned	HTPX64XXQ_read_dsp_hpi_data(unsigned n,unsigned a)
{
	HTPX64XXQ_set_dsp_hpia_rg(n,a&__HTPX64XXQ_DSP_HPI_ADDR_BITMASK);
	return(HTPX64XXQ_get_dsp_hpid_rg(n));
}

		// complex function, which sets HPIA register to <A> and writes data to via HPID_AINC.
void	HTPX64XXQ_write_dsp_hpi_data_ainc(unsigned n,unsigned a,unsigned d)
{
	HTPX64XXQ_set_dsp_hpia_rg(n,a&__HTPX64XXQ_DSP_HPI_ADDR_BITMASK);
	HTPX64XXQ_set_dsp_hpid_ainc_rg(n,d);
}


		// complex function, which sets HPIA register to <A> and returns HPID_AINC data
unsigned	HTPX64XXQ_read_dsp_hpi_data_ainc(unsigned n,unsigned a)
{
	HTPX64XXQ_set_dsp_hpia_rg(n,a&__HTPX64XXQ_DSP_HPI_ADDR_BITMASK);
	return(HTPX64XXQ_get_dsp_hpid_ainc_rg(n));
}



		// This function performs initialization of HPI port for particular DSP#n:
		//		- clears HINT bit of HPIC
		//		- sets HWOB=1 bit of HPIC
		//		- clears HPIA register
void	HTPX64XXQ_init_dsp_hpi(unsigned n)
{

	HTPX64XXQ_set_dsp_hpic_rg(n,HTPX64XXQ_DSP_HPIC_WR_HINT_OFF);
    HTPX64XXQ_set_dsp_hpia_rg(n,0);
}




	// Default DSP reset program load function
	// ----------------------------------------
	// 	This function loads default 'B RESET' code at the DSP RESET vector address.
	// 	After DSP RESET will be released by user host DSP application, this will put DSPn to
	//	the known state and will allow to use JTAG emulator, which requires that all eight DSP
	//	cores in JTAG path must be in the RUN state.
	//
	//		Notes:
	//		------
	//			1. 	This functions requires DSP# ('N') input parameter for selection of particular
	//				C64xx DSP.
	//			2.	This function uses DSP RESET control and DSP HPI bootmode in order to load
	//				program code into DSP environment at DSP 0x00000000 address.
	//			3.	HPI16 HPI data format mode is used for all platforms compatibility reasons.
	//			4.	This function temporary enables HPI access timeout control and check for
	//				HPI access timeout error during access to the HPI port.
	//			5.  DSP will be put into RESET state on exit. DSP bootmode control, HPI data
	//				format control and HPI access timeout enable control will be restored into
	//				original state.

		// C6xxx <B RESET> command codes and address
		//    (this must be 8 DWORDs as valid fetch packet for C6x)
#define __HTPX64XXQ_RESET_LOOP_CMND_DATA_W1		0x12
#define __HTPX64XXQ_RESET_LOOP_CMND_DATA_W2		0x0
#define __HTPX64XXQ_RESET_LOOP_CMND_DATA_W3		0x0
#define __HTPX64XXQ_RESET_LOOP_CMND_DATA_W4		0x0
#define __HTPX64XXQ_RESET_LOOP_CMND_DATA_W5		0x0
#define __HTPX64XXQ_RESET_LOOP_CMND_DATA_W6		0x0
#define __HTPX64XXQ_RESET_LOOP_CMND_DATA_W7		0x0
#define __HTPX64XXQ_RESET_LOOP_CMND_DATA_W8		0x0

#define __HTPX64XXQ_RESET_LOOP_CMND_ADDR_W1 	0x0


int		HTPX64XXQ_load_dsp_default_program(unsigned n)
{
	int r;
	__HTPX64XXQ_CNTR_RG_DATA_TYPE__ cntr2_rg_entry;


	// validate input 'N' parameter
	if(n>HTPX64XXQ_MAX_DSP_NUMBER) return(HTPX64XXQ_PARAM_ERR);

	// assume further error code is HPI timeout error
	r = HTPX64XXQ_HPI_ERR;

	// put DSP to the RESET state
	HTPX64XXQ_SET_DSP_RESET_ON(n);

	// 	save contents of CNTR2 register for further restore of
	//	DSP bootmode control, HPI data format control and HPI access timeout enable control
	cntr2_rg_entry = HTPX64XXQ_get_cntr2_rg();

	//	now select DSP HPI bootmode, HPI16 data format and enable HPI access timeout
	HTPX64XXQ_set_cntr2_rg(HTPX64XXQ_DSP_BMODE_CNTR_HPI | HTPX64XXQ_DSP_HPI_FMT_CNTR_HPI16 | HTPX64XXQ_DSP_HPI_ERR_EN_ON);

	//  release DSP reset control
	HTPX64XXQ_SET_DSP_RESET_OFF(n);

	//  clear HPI timeout error
	HTPX64XXQ_CLR_DSP_HPI_ERR(n);

	// init HPI port
	HTPX64XXQ_INIT_DSP_HPI(n);

	// check for HPI error condition
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

	// load <B RESET> code datawords at the DSP reset vector address via HPI port
	HTPX64XXQ_write_dsp_hpi_data_ainc(n,__HTPX64XXQ_RESET_LOOP_CMND_ADDR_W1,__HTPX64XXQ_RESET_LOOP_CMND_DATA_W1);
	HTPX64XXQ_set_dsp_hpid_ainc_rg(n,__HTPX64XXQ_RESET_LOOP_CMND_DATA_W2);
	HTPX64XXQ_set_dsp_hpid_ainc_rg(n,__HTPX64XXQ_RESET_LOOP_CMND_DATA_W3);
	HTPX64XXQ_set_dsp_hpid_ainc_rg(n,__HTPX64XXQ_RESET_LOOP_CMND_DATA_W4);
	HTPX64XXQ_set_dsp_hpid_ainc_rg(n,__HTPX64XXQ_RESET_LOOP_CMND_DATA_W5);
	HTPX64XXQ_set_dsp_hpid_ainc_rg(n,__HTPX64XXQ_RESET_LOOP_CMND_DATA_W6);
	HTPX64XXQ_set_dsp_hpid_ainc_rg(n,__HTPX64XXQ_RESET_LOOP_CMND_DATA_W7);
	HTPX64XXQ_set_dsp_hpid_ainc_rg(n,__HTPX64XXQ_RESET_LOOP_CMND_DATA_W8);

	// check for HPI error condition
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

	// verify written data
	if(		HTPX64XXQ_read_dsp_hpi_data_ainc(n,__HTPX64XXQ_RESET_LOOP_CMND_ADDR_W1)!=__HTPX64XXQ_RESET_LOOP_CMND_DATA_W1
		||	HTPX64XXQ_get_dsp_hpid_ainc_rg(n)!=__HTPX64XXQ_RESET_LOOP_CMND_DATA_W2
		||	HTPX64XXQ_get_dsp_hpid_ainc_rg(n)!=__HTPX64XXQ_RESET_LOOP_CMND_DATA_W3
		||	HTPX64XXQ_get_dsp_hpid_ainc_rg(n)!=__HTPX64XXQ_RESET_LOOP_CMND_DATA_W4
		||	HTPX64XXQ_get_dsp_hpid_ainc_rg(n)!=__HTPX64XXQ_RESET_LOOP_CMND_DATA_W5
		||	HTPX64XXQ_get_dsp_hpid_ainc_rg(n)!=__HTPX64XXQ_RESET_LOOP_CMND_DATA_W6
		||	HTPX64XXQ_get_dsp_hpid_ainc_rg(n)!=__HTPX64XXQ_RESET_LOOP_CMND_DATA_W7
		||	HTPX64XXQ_get_dsp_hpid_ainc_rg(n)!=__HTPX64XXQ_RESET_LOOP_CMND_DATA_W8
	  )
	{
		  r=HTPX64XXQ_HW_ERR;
		  goto exit;
	}

	// check for HPI error again after HPI data verification
	// (although it is actually not required once verufifcation has passed OK)
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

	// initialize HPI port
	HTPX64XXQ_init_dsp_hpi(n);

    // set no error
	r=HTPX64XXQ_OK;

 exit:
 	// set DSP into the RESET state
	HTPX64XXQ_SET_DSP_RESET_ON(n);

	// restore CNTR2 register on exit
	HTPX64XXQ_set_cntr2_rg(cntr2_rg_entry);

	// return error completion code
	return(r);
}





	// DSP init function
	// ------------------
	// 	This function configures DSP on-chip EMIF-A/B control registers and GPIO control registers
	//	for further access to DSP external SSRAM/SDRAM memory via DSP on-chip HPI port. This
	//	API function must be typically called just after DSP release from the reset state in
	//	DSP HPI bootmode before any access to DSP external SSRAM/SDRAM memories.
	//
	//		Notes:
	//		------
	//			1.	This function is analogue for TPX64XXQ_init_dsp() API function for C64xx DSP
	//				environment (refer to PX64XXQ_DSP.H include file).
	//			2. 	Data written to DSP on-chip EMIF-A/B control registers and GPIO control
	//				registers is not verified, since some bits can be internally set into
	//				different state.
	//			3. 	This functions requires DSP# ('N') input parameter for selection of particular
	//				C64xx DSP.
	//			4.	This function puts DSP into RESET state, however DSP bootmode control and HPI
	//				data format control are not altered and shall be set previously by us host DSP
	//				application.
	//			5.	This function temporary enables HPI access timeout control and check for
	//				HPI access timeout error during access to the HPI port.
	//			6.  DSP will stay in the RUN state on exit if no errors have been found and
	//				will be set to the RESET state if any error has been found. HPI access timeout
	//				enable control will be restored into original state.


		// C64xx DSP on-chip EMIF control regs area
		// (shall not be used in user host DSP code)
#define	__HTPX64XXQ_C64XX_EMIFA_GCR        				0x01800000
#define	__HTPX64XXQ_C64XX_EMIFA_CE0        				0x01800008
#define	__HTPX64XXQ_C64XX_EMIFA_CE1        				0x01800004
#define	__HTPX64XXQ_C64XX_EMIFA_CE2        				0x01800010
#define	__HTPX64XXQ_C64XX_EMIFA_CE3        				0x01800014
#define	__HTPX64XXQ_C64XX_EMIFA_SDRAM      				0x01800018
#define	__HTPX64XXQ_C64XX_EMIFA_SDRAM_TIMER				0x0180001c

#define	__HTPX64XXQ_C64XX_EMIFA_SDRAM_EXT   			0x01800020
#define	__HTPX64XXQ_C64XX_EMIFA_CE0_SEC     			0x01800048
#define	__HTPX64XXQ_C64XX_EMIFA_CE1_SEC     			0x01800044
#define	__HTPX64XXQ_C64XX_EMIFA_CE2_SEC     			0x01800050
#define	__HTPX64XXQ_C64XX_EMIFA_CE3_SEC     			0x01800054

#define	__HTPX64XXQ_C64XX_EMIFB_GCR        				0x01A80000
#define	__HTPX64XXQ_C64XX_EMIFB_CE0        				0x01A80008
#define	__HTPX64XXQ_C64XX_EMIFB_CE1        				0x01A80004
#define	__HTPX64XXQ_C64XX_EMIFB_CE2        				0x01A80010
#define	__HTPX64XXQ_C64XX_EMIFB_CE3        				0x01A80014
#define	__HTPX64XXQ_C64XX_EMIFB_SDRAM      				0x01A80018
#define	__HTPX64XXQ_C64XX_EMIFB_SDRAM_TIMER				0x01A8001c

#define	__HTPX64XXQ_C64XX_EMIFB_SDRAM_EXT    			0x01A80020
#define	__HTPX64XXQ_C64XX_EMIFB_CE0_SEC     			0x01A80048
#define	__HTPX64XXQ_C64XX_EMIFB_CE1_SEC     			0x01A80044
#define	__HTPX64XXQ_C64XX_EMIFB_CE2_SEC     			0x01A80050
#define	__HTPX64XXQ_C64XX_EMIFB_CE3_SEC     			0x01A80054

#define __HTPX64XXQ_C64XX_GPEN							0x01B00000
#define __HTPX64XXQ_C64XX_GPDIR							0x01B00004
#define __HTPX64XXQ_C64XX_GPVAL							0x01B00008

		// data to be written into EMIF/GPIO registers
			// 32-bit EMIF-A control registers settings
			// (refer to the corresponding definitions without '__H' prefix in PX64XXQ_DSP.H header file for more details)
#define __HTPX64XXQ_EMIFA_GCR_DATA						0x0001207c
#define __HTPX64XXQ_EMIFA_CE0_DATA						0x40
#define __HTPX64XXQ_EMIFA_CE0_SEC_DATA					0x00000002
#define __HTPX64XXQ_EMIFA_CE1_DATA						0xffffff03
#define __HTPX64XXQ_EMIFA_CE1_SEC_DATA					0x00000002
#define __HTPX64XXQ_EMIFA_CE2_DATA						0x30
#define __HTPX64XXQ_EMIFA_CE2_SEC_DATA					0x00000002
#define __HTPX64XXQ_EMIFA_CE3_DATA						0xffffff03
#define __HTPX64XXQ_EMIFA_CE3_SEC_DATA					0x00000002

#define __HTPX64XXQ_EMIFA_SDRAM16M_DATA_600MHZ			0x63115000
#define __HTPX64XXQ_EMIFA_SDRAM4M_DATA_600MHZ			0x57115000
#define __HTPX64XXQ_EMIFA_SDRAM_EXT_DATA_600MHZ			0x00014d29
#define	__HTPX64XXQ_EMIFA_SDRAM16M_TIMER_DATA_600MHZ	781
#define	__HTPX64XXQ_EMIFA_SDRAM4M_TIMER_DATA_600MHZ		1562

#define __HTPX64XXQ_EMIFA_SDRAM16M_DATA_720MHZ			0x63117000
#define __HTPX64XXQ_EMIFA_SDRAM4M_DATA_720MHZ			0x57117000
#define __HTPX64XXQ_EMIFA_SDRAM_EXT_DATA_720MHZ			0x00014d3b
#define	__HTPX64XXQ_EMIFA_SDRAM16M_TIMER_DATA_720MHZ	937
#define	__HTPX64XXQ_EMIFA_SDRAM4M_TIMER_DATA_720MHZ		1875

			// 16-bit EMIF-B control registers settings
			// (refer to the corresponding definitions without '__H' prefix in PX64XXQ_DSP.H header file for more details)
#define __HTPX64XXQ_EMIFB_GCR_DATA						0x0001207c
#define __HTPX64XXQ_EMIFB_CE0_DATA						0x10914211
#define __HTPX64XXQ_EMIFB_CE1_DATA						0xffffff03
#define __HTPX64XXQ_EMIFB_CE2_DATA						0xffffff03
#define __HTPX64XXQ_EMIFB_CE3_DATA						0xffffff03

			// settings for GPIO control registers
			// (refer to the corresponding definitions without '__H' prefix in PX64XXQ_DSP.H header file for more details)
#define	__HTPX64XXQ_GPIO_SX_XIO0_OE_INPUT				0x4000
#define	__HTPX64XXQ_GPIO_SX_XIO1_OE_INPUT				0x8000
#define	__HTPX64XXQ_GPIO_SX_RESET_BITMASK				0x00000001
#define	__HTPX64XXQ_GPIO_SX_XIO0_OE_BITMASK				0x00004000
#define	__HTPX64XXQ_GPIO_SX_XIO1_OE_BITMASK				0x00008000


int	HTPX64XXQ_init_dsp(unsigned n)
{
	int r;
	unsigned x;
	__HTPX64XXQ_CNTR_RG_DATA_TYPE__	cntr2_rg_entry;

	// 	validate input 'N' parameter
	if(n>HTPX64XXQ_MAX_DSP_NUMBER) return(HTPX64XXQ_PARAM_ERR);

	// 	assume further error code is HPI timeout error
	r = HTPX64XXQ_HPI_ERR;

	// 	check whether this DSP has been already released from the RESET state
	//	If not yet released, then release it without altering DSP bootmode control and HPI data format control.
	if(HTPX64XXQ_DSP_RESET_IS_ON(n))    HTPX64XXQ_SET_DSP_RESET_OFF(n);

	// 	save contents of CNTR2 register for further restore of
	//	DSP bootmode control, HPI data format control and HPI access timeout enable control
	cntr2_rg_entry = HTPX64XXQ_get_cntr2_rg();

	//	temporary enable HPI access timeout
	HTPX64XXQ_SET_DSP_HPI_ERR_EN_ON;

	//  clear HPI timeout error if any
	HTPX64XXQ_CLR_DSP_HPI_ERR(n);

	// init DSP HPI port
	HTPX64XXQ_INIT_DSP_HPI(n);

	// check for HPI error condition
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

	// EMIF-B initialization.
	// This must be done prior EMIF-A initialization since all on-board ID registers are read out from
	// this EMIF via IOX control registers.
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFB_GCR,__HTPX64XXQ_EMIFB_GCR_DATA);
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFB_CE0,__HTPX64XXQ_EMIFB_CE0_DATA);
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFB_CE1,__HTPX64XXQ_EMIFB_CE1_DATA);
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFB_CE2,__HTPX64XXQ_EMIFB_CE2_DATA);
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFB_CE3,__HTPX64XXQ_EMIFB_CE3_DATA);

	// check for HPI error condition
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

	// wait a little to ensure that EMIF-B has been configured.
	for(x=0;x<100;x++);


	// EMIF-A initialization.
	// Note, that TPX64xxQ boards may differ in SSRAM/SDRAM capacity and DSP speed grade,
	// which result in different SDRAM control register initialization.
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_GCR,__HTPX64XXQ_EMIFA_GCR_DATA);

	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE0,__HTPX64XXQ_EMIFA_CE0_DATA);
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE0_SEC,__HTPX64XXQ_EMIFA_CE0_SEC_DATA);

	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE1,__HTPX64XXQ_EMIFA_CE1_DATA);
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE1_SEC,__HTPX64XXQ_EMIFA_CE1_SEC_DATA);

	if(HTPX64XXQ_DSP_SDRAM_IS_INSTALLED)
	{
		// if SDRAM is installed, then configure EMIF-A CE2 area as SDRAM
		HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE2,__HTPX64XXQ_EMIFA_CE2_DATA);
		HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE2_SEC,__HTPX64XXQ_EMIFA_CE2_SEC_DATA);

		// set SDRAM control registers upon the DSP speed grade and SDRAM capacity
		if(HTPX64XXQ_DSP_SPEED_ID_IS_600MHZ)
		{
			// perform SDRAM setting for 600MHz DSP
			if(HTPX64XXQ_DSP_SDRAM_LEN_IS_64MB)
			{
				// configure for 16Mx32 SDRAM and DSP speed 600MHz
				HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM,__HTPX64XXQ_EMIFA_SDRAM16M_DATA_600MHZ);
				HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM_TIMER,__HTPX64XXQ_EMIFA_SDRAM16M_TIMER_DATA_600MHZ);
			}
			else
			{
				// configure for 4Mx32 SDRAM and DSP speed 600MHz
				HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM,__HTPX64XXQ_EMIFA_SDRAM4M_DATA_600MHZ);
				HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM_TIMER,__HTPX64XXQ_EMIFA_SDRAM4M_TIMER_DATA_600MHZ);
			}

			HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM_EXT,__HTPX64XXQ_EMIFA_SDRAM_EXT_DATA_600MHZ);
		}

		if(HTPX64XXQ_DSP_SPEED_ID_IS_720MHZ)
		{
			// perform SDRAM setting for 720MHz DSP
			if(HTPX64XXQ_DSP_SDRAM_LEN_IS_64MB)
			{
				// configure for 16Mx32 SDRAM and DSP speed 720MHz
				HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM,__HTPX64XXQ_EMIFA_SDRAM16M_DATA_720MHZ);
				HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM_TIMER,__HTPX64XXQ_EMIFA_SDRAM16M_TIMER_DATA_720MHZ);
			}
			else
			{
				// configure for 4Mx32 SDRAM and DSP speed 720MHz
				HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM,__HTPX64XXQ_EMIFA_SDRAM4M_DATA_720MHZ);
				HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM_TIMER,__HTPX64XXQ_EMIFA_SDRAM4M_TIMER_DATA_720MHZ);
	 		}

			HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_SDRAM_EXT,__HTPX64XXQ_EMIFA_SDRAM_EXT_DATA_720MHZ);
		}
	}
	else
	{
		// otherwise, if no SDRAM is installed, configure EMIF-A CE2 area as SBSRAM
		// (this will exclude refresh cycles)
		HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE2,__HTPX64XXQ_EMIFA_CE0_DATA);
		HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE2_SEC,__HTPX64XXQ_EMIFA_CE0_SEC_DATA);
	}

	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE3,__HTPX64XXQ_EMIFA_CE3_DATA);
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_EMIFA_CE3_SEC,__HTPX64XXQ_EMIFA_CE3_SEC_DATA);

	// check for HPI error
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;


	// read from dedicated SSRAM location, which corresponds to DSP-to-DSP request
	// to this node (in order to clear this request) and write to list SSRAM location
	// in order to clear transferred token
	if(HTPX64XXQ_DSP_SSRAM_IS_INSTALLED)
	{
		x=HTPX64XXQ_read_dsp_hpi_data(n,HTPX64XXQ_DSPE_SSRAM_DSPA_RQ_ADDR-(n*4));
		HTPX64XXQ_write_dsp_hpi_data(n,HTPX64XXQ_DSPE_SSRAM_DSPA_RQ_ADDR-(n*4),x&0);	// use X&0==0 in order to exclude compiler warning re never usage of X
 	}
	// check for HPI error
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

	// Initialize GPIO control registers in order to control MXSIOX RESET (GPIO-0)
	// and TM/XIO-0/1 OE control (GPIO-14/15). All GPIO-0/14/15 pins shall be configured as OUTPUTs.
	// GPIO-0 pin must be set to deault '0' output vale in order to set SX-RESET
	// into the 'ON' state (active low). GPIO-14/15 outputs shall be set to the '1'
	// state in order to configure SIOX TM/XIO pins as INPUTs.
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_GPEN, __HTPX64XXQ_GPIO_SX_RESET_BITMASK | __HTPX64XXQ_GPIO_SX_XIO0_OE_BITMASK | __HTPX64XXQ_GPIO_SX_XIO1_OE_BITMASK);
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_GPDIR,__HTPX64XXQ_GPIO_SX_RESET_BITMASK | __HTPX64XXQ_GPIO_SX_XIO0_OE_BITMASK | __HTPX64XXQ_GPIO_SX_XIO1_OE_BITMASK);
	HTPX64XXQ_write_dsp_hpi_data(n,__HTPX64XXQ_C64XX_GPVAL,__HTPX64XXQ_GPIO_SX_XIO0_OE_INPUT | __HTPX64XXQ_GPIO_SX_XIO1_OE_INPUT);

	// do not initialize DSP on-chip McBSP ports as it is done in TPX64XXQ_init_dsp() DSP API function
	// since this does not effect data upload from host DSP via HPI port

	// initialize HPI port
	HTPX64XXQ_init_dsp_hpi(n);

	// check for HPI error
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

    // set no error
	r=HTPX64XXQ_OK;

 exit:
 	// ??? set DSP into the RESET state is error has been occured
	if(r!=HTPX64XXQ_OK) HTPX64XXQ_SET_DSP_RESET_ON(n);

	// restore CNTR2 register on exit
	HTPX64XXQ_set_cntr2_rg(cntr2_rg_entry);

	// return error completion code
	return(r);
}




	// PIOX/PIOX-16 I/F software reset function.
	// -----------------------------------------
	// 	This function resets all DSP cores, clears HPI errors, disable host PIOX IRQs
	// 	enables HPI timout errors.

int	HTPX64XXQ_init_piox()
{
   	HTPX64XXQ_set_cntr1_rg(0);
   	HTPX64XXQ_set_cntr2_rg(0);
   	HTPX64XXQ_SET_DSP_HPI_ERR_EN_ON;
   	HTPX64XXQ_CLR_DSP_HPI_ERR_ALL;
    HTPX64XXQ_set_hirq_sel_rg(0,0);
    HTPX64XXQ_set_hirq_sel_rg(1,0);
    HTPX64XXQ_set_hirq_sel_rg(2,0);
    HTPX64XXQ_set_hirq_sel_rg(3,0);

	// verify initilized registers contents
   	if(	 HTPX64XXQ_get_cntr1_rg()!=0
	  || (HTPX64XXQ_get_cntr2_rg() & (HTPX64XXQ_DSP_BMODE_CNTR_BITMASK | HTPX64XXQ_DSP_HPI_FMT_CNTR_BITMASK | HTPX64XXQ_DSP_HPI_ERR_EN_BITMASK))!=HTPX64XXQ_DSP_HPI_ERR_EN_ON
	  || HTPX64XXQ_get_int_stat_rg()!=0
	  || HTPX64XXQ_get_hirq_sel_rg(0)!=0
	  || HTPX64XXQ_get_hirq_sel_rg(1)!=0
	  || HTPX64XXQ_get_hirq_sel_rg(2)!=0
	  || HTPX64XXQ_get_hirq_sel_rg(3)!=0
	 )
		return(HTPX64XXQ_HW_ERR);

	return(HTPX64XXQ_OK);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#endif	// end of API AUX functions code






#ifndef __HTPX64XXQ_EXCLUDE_CBSF_PROGRAM_LOADER__
//~~~~~~~~~~~~~~~~~~~~~~ C6xxx C-BSF target DSP code loader function ~~~~~~~~~~~~~~~~~~~~~
//	These API functions include function code for C6xxx C-BSF code loader and allows
//	to load C64xx DSP code to any of TORNADO-PX64xxQ on-board DSP from host TORNADO DSP
//	application.
//	In order to  exclude mutiple function body appearance, API AUX function bodies shall be
//	excluded from application code for 2nd, 3rd, etc application .OBJ modules via
//	__HTPX64XXQ_EXCLUDE_CBSF_PROGRAM_LOADER__ run-time compiler key:
//
//		__HTPX64XXQ_EXCLUDE_CBSF_PROGRAM_LOADER__
//			- if defined in user code (assigned to any value), then the C6xxx C-BSF loader
//		  	  function (HPTX64XXQ_load_dsp_program() is excluded from the user source code.
//		  	  Default is INCLUDED.
//
// C6xxx C-BSF program data must appear as 32-bit unsigned data array in host TORNADO
// DSP code, which can be converted from source C6xxx *.OUT file via C6X_CBSF.EXE PC utility.
// For more details refer to TORNADO-PX64xxQ User's Guide.
//
// Notes:
//	1.	C6xxx C-BSF file can be generated from the corresponding .OUT file using C6X_CBSF.EXE
//		utility, which is the part of TORNADO-PX64xxQ s/w utilities. Note, that C6X_CBSF.EXE
//		utility runs under DOS and Windows, and input/output filenames for this utility must
//		meet DOS file name conventions (8+3).
//	2.	This API function uses DSP reset control and DSP HPI bootmode. After successfull
//		program upload, DSP is left in the idle state with HPI bootmode. Host DSP application
//		must start DSP during valid HPI boot (generate DSPINT) in order to start program
//		execution. In case of any error found, DSP is put to the RESET state.
//	3.	HPI timeout error control is temporary enabled during program upload.
//	4. 	This function returns error completion code.
//	5. 	C6xxx DSP on-chip target memory areas and control registers can be loaded.
//	6. 	All C6xxx DSP external memory areas can be loaded. This function calls HTPX64XXQ_init_dsp()
//		API AUX function in order to initialize C64xx DSP EMIF-A/B registers in accordance with
//		TORNADO-PX64xxQ DSP environment definitions.
//	7. 	User host DSP application must configure HPI BOOTMODE CONTROL bit of CONTR2 register in order
//		to have C64xx DSP to boot with desired HPI data format during execution of this API function.
//
//
//	v.1A-1 fixed problems:
//	----------------------
//	1.	This C6xxx C-BSF loader has been update in v.1A-1 to support upload of input sections that
//		has load address at the x4 boundary. This happens with CINIT-COPY sections (CINIT section from 
//		the .OUT file linked with the -CR linker option, i.e. RAM initialization model), which can contain 
//		direct data initialization for byte or 16-bit word, which is can be allocated at any address. 
//		TI C6xxx C-compiler typically provides best efforts to allocate a set of non-x4 byte vars at 
//		the x4 address boundaries, however in case there is only one byte or w16 variable from total two vars
//		(and the other one is w32), then it can be allocated at the non-x4 address and CINIT-COPY section may
//		contain only one such variable for initialization.	C6X_CBSF.EXE utility is now also able to correctly
//		generate such output C-BSF file.


static int __htpx64xxq_write_cbsf_byte(unsigned dsp_node, unsigned addr, unsigned char d)
	// this is the local function, which is used to write one bytes at provided address
	// into DSP environment via HPI port. This function is used during upload of CINIT-RAM records, which
	// contains non-x4 byte aligned data (int short, char, etc) in order to write prefix/suffix data words
	// of the record.
{
 	unsigned hpi_w32_read, hpi_w32_write, byte_mask, byte_shift, hpi_addr_w32;

	// validate input parameters
	// Note, that input ADDR parameter must be at the 4x byte boundary.
 	if(dsp_node>HTPX64XXQ_MAX_DSP_NUMBER)
	  	return(HTPX64XXQ_PARAM_ERR);

	// at first, calculate W32 address, which contains this byte
	hpi_addr_w32 =  (addr-(addr%4L));

	// now read HPI W32 dataword fm DSP environment
	hpi_w32_read = HTPX64XXQ_read_dsp_hpi_data(dsp_node,hpi_addr_w32);

	// check for HPI error condition
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(dsp_node)) return(HTPX64XXQ_HPI_ERR);

	// now generate w32 to write using provided byte and address
	switch(addr%4L)
	{
		case 0:
			byte_mask = 0xff;
			byte_shift = 0;
			break;

		case 1:
			byte_mask = 0xff00;
			byte_shift = 8;
			break;

		case 2:
			byte_mask = 0xff0000;
			byte_shift = 16;
			break;

		default:
			byte_mask = 0xff000000;
			byte_shift = 24;
			break;
	}

    hpi_w32_read &= ~byte_mask;
    hpi_w32_write = (hpi_w32_read | ((((unsigned )d)<<byte_shift)&byte_mask));

	// write generated W32 dataword via HPI
	HTPX64XXQ_write_dsp_hpi_data(dsp_node,hpi_addr_w32,hpi_w32_write);

	// check for HPI error condition
	if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(dsp_node)) return(HTPX64XXQ_HPI_ERR);

	// return successfully
	return(HTPX64XXQ_OK);
}



	// C6x C-BSF section type IDs
#define	__HTPX64XXQ_C6X_CBSF_SHD_MEM_ID						0x90003b09	// memory write CBSF-section (text, bss, etc)
#define	__HTPX64XXQ_C6X_CBSF_SHD_CINIT_COPY_RECORD_ID		0x90013b09	// memory write CBSF-section (CINIT-RAM copy section record)
#define	__HTPX64XXQ_C6X_CBSF_SHD_TERM_ID					0xffff3b09L	// last CBSF-section ID


int		HTPX64XXQ_load_dsp_program(unsigned n, unsigned *p)
{
	int r;
	__HTPX64XXQ_CNTR_RG_DATA_TYPE__	cntr2_rg_entry;
	unsigned		shd_id, shd_addr, shd_len,
					*d;
	unsigned prefix_bytes, w32_bytes, suffix_bytes,q,i, ptr_converter,w32;
	unsigned char *byte_ptr, *w32_byte_ptr;


	// validate input parameters
	if(p==NULL || n>HTPX64XXQ_MAX_DSP_NUMBER) return(HTPX64XXQ_PARAM_ERR);

	// 	save contents of CNTR2 register for further restore of
	//	DSP bootmode control, HPI data format control and HPI access timeout enable control
	cntr2_rg_entry = HTPX64XXQ_get_cntr2_rg();

	// select DSP bootmode to HPI in order to keep DSP kernel in the RESET state while testing DSP memory
    HTPX64XXQ_SET_DSP_BMODE_CNTR_HPI;

	//	temporary enable HPI access timeout
	HTPX64XXQ_SET_DSP_HPI_ERR_EN_ON;

	// initialize target DSP EMIF-A/B and GPIO
	// Note, that this function also releases
	if((r=HTPX64XXQ_init_dsp(n))!= HTPX64XXQ_OK) goto exit;


	// scan over C-BSF section descriptors and load data to DSP on-chip memory
	d = p;
	while(1)
	{
		// fill-in BSF section header
		shd_id 	 = *(d++);
		shd_addr = *(d++);
		shd_len  = *(d++);

		// validate SHD params
		r=HTPX64XXQ_INV_CBSF_DATA_FMT_ERR;	// assume error

		switch(shd_id)
		{
			case __HTPX64XXQ_C6X_CBSF_SHD_MEM_ID:
		  	case __HTPX64XXQ_C6X_CBSF_SHD_CINIT_COPY_RECORD_ID:

				// section with valid data for write to DSP environment.
				// verify input address and length
				if(	   shd_len==0
					|| (shd_addr+shd_len-1)>HTPX64XXQ_DSPE_ADDR_MAX
				  )
					goto exit;
				break;


		  case __HTPX64XXQ_C6X_CBSF_SHD_TERM_ID:
				// termination SHD ID. Check for SHD_LEN and SHD_ADDR fields
				// for '0' value and exit successfully. Otherwise, exit with error.
				if( shd_addr==0 && shd_len==0)
				{
					// this is correct CBSF termination header data
					// now initialize HPI and exit without error

					// 	assume further error code is HPI timeout error
					// initialize HPI port and check for errors
					r=HTPX64XXQ_HPI_ERR;
					HTPX64XXQ_init_dsp_hpi(n);
					if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

				    // set no error
					r=HTPX64XXQ_OK;

					goto exit;
				}
				else
					// this is incorrect CBSF termination header data
					// Exit with error.
					goto exit;

				// invalid SHD ID. Exit with error set.
		  default:
				goto exit;
	  	  }


		// now upload data from CBSF array to DSP on-chip memory via HPI port

		// 	assume further error code is HPI timeout error
		r=HTPX64XXQ_HPI_ERR;

		// added in v.1A-1:
		// ----------------
		//		Although it is easy to use BYTE read/write via HPI in order to
		//		get around non-x4 start address and section length, this will deliver low
		//		upload performance at approximately /4 times. In order to exclude this in
		//		case start address is not at x4 boundary, at first, the corresponding number of
		//		aligned bytes is read fm the file to the local 4-byte array in order to align
		//		next upload address at x4 boundary and use 32-bit HPI access.

		// assign source bte pointer
		byte_ptr = (unsigned char *) d;

		// check for the start address is allocated at x4 bytes boundary
		// In this case, we must first read prefix bytes to local array and upload
		// them as bytes via HPI port in order to allow to proceed further with 32-bit HPI access.
		if((shd_addr%4L)!=0)
		{
			// calcuclate how many prefix bytes have to be read
			prefix_bytes = 4L-(shd_addr%4L);

			// take care of section length
			q = (prefix_bytes<shd_len)? prefix_bytes:shd_len;

			// copy prefix bytes fm input CBSF array to target DSP environment via HPI using byte access
			for(i=0;i<q;i++,shd_addr++)
			{
				if((r=__htpx64xxq_write_cbsf_byte(n,shd_addr,*(byte_ptr++)))!=HTPX64XXQ_OK)
					goto exit;
			}

			// now update section LEN for further 32-bit HPI access after write of prefix bytes
			shd_len -= q;
		}

		// check for the section length is still >0, otherwise finish with this section
    	if(shd_len!=0)
		{
		    // separate section length into integer number of 32-bit words and number of residual bytes
			w32_bytes = (shd_len - (shd_len%4L));
			suffix_bytes = (shd_len%4L);

			// now load part of CBSF section data, which length is x4 bytes
			// perform this upoad via 32-bit HPI access and local 32-bit word
			if(w32_bytes!=0)
			{
		       	// set HPIA register for further HPID_AINC access
				HTPX64XXQ_set_dsp_hpia_rg(n,shd_addr);
				// check for HPI error condition
				if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

				// perform the upload loop
				for(i=0;i<w32_bytes;)
				{
					// read four bytes from source array to local W32
					// Note that we must do this using byte pointers, since we are now not sure that source
					// CBSF data is aligned at x4 address boundary.
					w32_byte_ptr = (unsigned char *) &w32;
					*(w32_byte_ptr++) = *(byte_ptr++);
					*(w32_byte_ptr++) = *(byte_ptr++);
					*(w32_byte_ptr++) = *(byte_ptr++);
					*(w32_byte_ptr++) = *(byte_ptr++);

					// write HPI data using AINC feature
					HTPX64XXQ_set_dsp_hpid_ainc_rg(n,w32);

					// check for HPI error condition
					if(HTPX64XXQ_DSP_HPI_ERR_IS_ON(n)) goto exit;

					// modify address/length
					shd_addr += 4;
					i += 4;
				}
			}


			// now check whether suffix bytes shall be uploaded
			// perform this upload via 8-bit HPI access functions
			if(suffix_bytes!=0)
			{
				// copy suffix bytes fm input CBSF array to target DSP environment via HPI using byte access
				for(i=0;i<suffix_bytes;i++,shd_addr++)
				{
					if((r=__htpx64xxq_write_cbsf_byte(n,shd_addr,*(byte_ptr++)))!=HTPX64XXQ_OK)
						goto exit;
				}
			}
		}

		// get here at the end of each section processing and align main source W32 CBSF array pointer (*D) 
		// from byte pointer (*BYTE_PTR) to point at the nearest upper W32. This is required since source 
		// CBSF array appears as a set of 32-bit datawords.
		ptr_converter = (unsigned) byte_ptr;
		d = (unsigned *) (ptr_converter + (((ptr_converter%4)!=0) ? (4-(ptr_converter%4)):0));
	}// end of CBSF sections scan

exit:
 	// set DSP into the RESET state is error has been occured
	if(r!=HTPX64XXQ_OK) HTPX64XXQ_SET_DSP_RESET_ON(n);

	// restore CNTR2 register on exit
	HTPX64XXQ_set_cntr2_rg(cntr2_rg_entry);

	// return error completion code
	return(r);
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#endif	// end of C6xxx C-BSF loader
//========================================================================================

#endif	// end of HTPX64xxQ defs

